import React from "react";

import { FormApi } from "final-form";
import { useMutation } from "react-apollo";
import { Field, Form } from "react-final-form";

import {
  UpdateProperyIsArchived_admin_updateProperty,
  UpdateProperyIsArchivedVariables,
} from "../../entities/listing/graphql/mutations/__generated__/UpdateProperyIsArchived";
import UpdateProperyIsArchived from "../../entities/listing/graphql/mutations/UpdateProperyIsArchived.graphql";
import { GetProperty_admin_Property } from "../../entities/listing/graphql/queries/__generated__/GetProperty";
import { CheckboxToggle } from "../../ui/molecules/CheckboxToggle";
import { ModalConfirm } from "../../ui/molecules/Modal";

import css from "./index.module.css";

export interface ArchiveProps {
  property: GetProperty_admin_Property;
}

interface FormValues {
  is_archived?: boolean;
}

interface ModalContext {
  visiable: boolean;
  form?: FormApi<FormValues>;
  checkboxValue?: boolean;
}

const Archive: React.FC<ArchiveProps> = ({ property }) => {
  const [modalContext, setModalContext] = React.useState<ModalContext>({
    visiable: false,
  });

  const [updateProperyIsArchived, { loading }] = useMutation<
    UpdateProperyIsArchived_admin_updateProperty,
    UpdateProperyIsArchivedVariables
  >(UpdateProperyIsArchived);

  const initialValues: FormValues = {
    is_archived: !!property.is_archived,
  };

  const handleSubmit = React.useCallback(
    ({ is_archived = false }: FormValues) => {
      return updateProperyIsArchived({
        variables: { is_archived, id: property.id },
      });
    },
    [property.id, updateProperyIsArchived]
  );

  const onSubmit = React.useCallback(() => {
    const form = modalContext.form;
    if (form) {
      form.change("is_archived", !!modalContext.checkboxValue);
      form.submit();
    }

    setModalContext({ visiable: false });
  }, [modalContext, setModalContext]);

  const onArchiveCheckboxChange = React.useCallback((isArchived, form) => {
    if (isArchived) {
      setModalContext({ visiable: true, form, checkboxValue: isArchived });
    } else {
      form.change("is_archived", !!isArchived);
      form.submit();
    }
  }, []);

  return (
    <>
      <div className={css.container}>
        <Form<FormValues>
          initialValues={initialValues}
          onSubmit={handleSubmit}
          render={({ form, submitting }) => (
            <Field
              component={CheckboxToggle}
              data-testid="toggle-archived"
              disabled={loading || submitting}
              labelText="Archived"
              name="is_archived"
              onChange={(evt) =>
                onArchiveCheckboxChange(evt.target.checked, form)
              }
              type="checkbox"
            />
          )}
        />
      </div>
      {modalContext.visiable && (
        <ModalConfirm
          onCancel={() => setModalContext({ visiable: false })}
          onConfirm={onSubmit}
        >
          <strong>Are you sure you want to archive this listing?</strong>
          <p>All viewings will be canceled.</p>
        </ModalConfirm>
      )}
    </>
  );
};

export default Archive;
