import React, { FC, useCallback, useMemo } from "react";

import styled from "@emotion/styled";
import gql from "graphql-tag";
import { useMutation } from "react-apollo";
import { Field, Form } from "react-final-form";

import { FETCH_USER_NOTIFICATIONS } from "../../entities/platform-users/UserDetails/Notifications";
import {
  common_api_generic_query_QueryFilterOperatorEnum,
  notes_api_CorrelationTypeEnum,
} from "../../globalTypes";
import { PrimaryButton } from "../../ui/atoms/Button";
import { TextInput } from "../../ui/atoms/Input";
import Spinner from "../../ui/atoms/Spinner";
import ErrorMessage from "../../ui/molecules/ErrorMessage";
import { Modal } from "../../ui/molecules/Modal";
import { TextareaFieldTouched } from "../../ui/molecules/TextareaField";
import {
  composeValidators,
  dateInRange,
  required,
} from "../../util/validations";
import { CreateNote, CreateNoteVariables } from "./__generated__/CreateNote";
import {
  UpdateNote_admin,
  UpdateNoteVariables,
} from "./__generated__/UpdateNote";
import { CommentWrapper, DateWrapper, RowWrapper } from "./styles";

type Props = {
  userId: string;
  reminderId?: string;
  form?: {
    comment: string;
    date: string;
  };
  onCancel: () => void;
  onSubmit: () => void;
};

export const SetReminderModal: FC<Props> = ({
  userId,
  reminderId,
  form,
  onCancel,
  onSubmit,
}) => {
  const [
    createNotification,
    { loading: createLoading, error: createError },
  ] = useMutation<CreateNote, CreateNoteVariables>(CREATE_NOTIFICATION);

  const [
    updateNotification,
    { loading: updateLoading, error: updateError },
  ] = useMutation<UpdateNote_admin, UpdateNoteVariables>(UPDATE_NOTIFICATION);

  const refetchQueries = useMemo(
    () => [
      {
        query: FETCH_USER_NOTIFICATIONS,
        variables: {
          pagination: { page: 1, perPage: 100 },
          filter: [
            {
              fields: ["correlation_type"],
              value: [notes_api_CorrelationTypeEnum.CORRELATION_TYPE_USER],
              operator: common_api_generic_query_QueryFilterOperatorEnum.EQUALS,
            },
            {
              fields: ["correlation_id"],
              value: [userId],
              operator: common_api_generic_query_QueryFilterOperatorEnum.EQUALS,
            },
          ],
        },
      },
    ],
    [userId]
  );

  const handleSubmit = useCallback(
    async (values: Record<string, string>) => {
      if (createLoading) {
        return;
      }

      if (form && reminderId) {
        await updateNotification({
          variables: {
            id: reminderId,
            text: values.comment,
            scheduled: new Date(values.date).toUTCString(),
          },
          refetchQueries,
        });
      } else {
        await createNotification({
          variables: {
            text: values.comment,
            scheduled: new Date(values.date).toUTCString(),
            correlation_type:
              notes_api_CorrelationTypeEnum.CORRELATION_TYPE_USER,
            correlation_id: userId,
          },
          refetchQueries,
        });
      }

      if (!createError && !updateError) {
        onSubmit();
      }
    },
    [
      createLoading,
      form,
      reminderId,
      createError,
      updateError,
      updateNotification,
      refetchQueries,
      createNotification,
      userId,
      onSubmit,
    ]
  );

  const minDate = new Date();
  minDate.setHours(0, 0, 0, 0);

  return (
    <Modal onCancel={onCancel} title={"Reminder Notes"}>
      <Form
        initialValues={form}
        onSubmit={handleSubmit}
        render={({ handleSubmit }) => (
          <form onSubmit={handleSubmit}>
            <CommentWrapper>
              <Field<string>
                component={TextareaFieldTouched}
                name="comment"
                validate={required("Value is required")}
              />
            </CommentWrapper>

            <RowWrapper>
              <DateWrapper>
                <Field
                  component={TextInput}
                  name="date"
                  type="date"
                  validate={composeValidators(
                    required("Date is required"),
                    dateInRange(
                      { min: minDate },
                      "Notification date can not be in past"
                    )
                  )}
                />
                <Field component={ErrorMessage} name="date" />
              </DateWrapper>
              <AddButton>
                {createLoading || updateLoading ? (
                  <Spinner color="#fff" size="sm" />
                ) : form ? (
                  "Edit reminder"
                ) : (
                  "Set reminder"
                )}
              </AddButton>
            </RowWrapper>
          </form>
        )}
      />
    </Modal>
  );
};

const AddButton = styled(PrimaryButton)`
  height: 40px;
  width: 200px;
  align-items: center;
`;

const CREATE_NOTIFICATION = gql`
  mutation CreateNote(
    $text: String
    $user: String
    $scheduled: String
    $correlation_type: notes_api_CorrelationTypeEnum
    $correlation_id: String
  ) {
    admin {
      createNote(
        text: $text
        user: $user
        scheduled: $scheduled
        correlation_type: $correlation_type
        correlation_id: $correlation_id
      ) {
        note {
          id
        }
      }
    }
  }
`;

const UPDATE_NOTIFICATION = gql`
  mutation UpdateNote($text: String, $scheduled: String, $id: String) {
    admin {
      updateNote(text: $text, scheduled: $scheduled, id: $id) {
        note {
          id
        }
      }
    }
  }
`;
