import React, { FC } from "react";

import { FORM_ERROR } from "final-form";
import { Query, useApolloClient } from "react-apollo";
import { Field, FieldRenderProps, Form } from "react-final-form";
import { useDispatch } from "react-redux";
import {
  Link,
  RouteComponentProps,
  useHistory,
  useParams,
} from "react-router-dom";
import { oc } from "ts-optchain";

import { actions, Severity } from "../../notifications.slice";
import { PrimaryButton, SecondaryButton } from "../../ui/atoms/Button";
import { CheckboxBlock } from "../../ui/atoms/CheckboxInput";
import { TextInput } from "../../ui/atoms/Input";
import SystemError from "../../ui/atoms/SystemError";
import {
  ButtonsGroup,
  UserForm,
  UserFormAction,
  UserFormGroup,
  UserFormOverlay,
  UserFormRow,
  UserFormWrapper,
} from "../../ui/atoms/UserForm";
import ErrorMessage from "../../ui/molecules/ErrorMessage";
import strip_typenames from "../../util/strip_typenames";
import {
  composeValidators,
  required,
  validEmail,
  validUkNumber,
} from "../../util/validations";
import {
  UpdateAdmin,
  UpdateAdminVariables,
} from "./graphql/mutations/__generated__/UpdateAdmin";
import UPDATE_ADMIN from "./graphql/mutations/UpdateAdmin.graphql";
import {
  AdminDetails,
  AdminDetailsVariables,
} from "./graphql/queries/__generated__/AdminDetails";
import ADMIN_DETAILS from "./graphql/queries/Admin.graphql";

type ParamsProps = {
  id?: string;
};
type State = {};
type Props = RouteComponentProps<ParamsProps> & {
  client: any;
};

const InputField: FC<FieldRenderProps<any, any>> = (props) => (
  <>
    <TextInput {...props} />
    <ErrorMessage {...props} />
  </>
);

const Edit: FC = () => {
  const client = useApolloClient();
  const history = useHistory();
  const dispatch = useDispatch();
  const handleSubmit = (values) =>
    client
      .mutate<UpdateAdmin, UpdateAdminVariables>({
        mutation: UPDATE_ADMIN,
        variables: {
          account: {
            ...values,
            admin: true,
          },
        },
      })
      .then((res) => {
        const errors = res.errors ?? [];
        if (errors.length > 0) {
          return { [FORM_ERROR]: errors.map((e) => e.message).join(", ") };
        } else {
          dispatch(
            actions.showNotification({
              message: "User data updated",
              severity: Severity.Success,
            })
          );
          return undefined;
        }
      });
  const { id: adminId } = useParams<{ id: string }>();

  if (!adminId) {
    return (
      <UserForm>
        <h3>Wrong link.</h3>
        <p>
          <Link to="/admin">return to admins list</Link>
        </p>
      </UserForm>
    );
  }

  return (
    <UserForm>
      <h2>Admin User Card</h2>

      <UserFormWrapper>
        <Query<AdminDetails, AdminDetailsVariables>
          query={ADMIN_DETAILS}
          variables={{ id: adminId }}
        >
          {({ data, loading, error }) => {
            if (loading) {
              return <>Loading...</>;
            } else if (error) {
              return <SystemError error={error} />;
            }

            return (
              <Form
                initialValues={strip_typenames(oc(data).admin.Admin(null))}
                onSubmit={handleSubmit}
                render={({ handleSubmit, submitting, valid }) => (
                  <form onSubmit={handleSubmit}>
                    {submitting && <UserFormOverlay />}
                    <UserFormGroup>
                      <UserFormRow>
                        <label htmlFor="details.first_name">First name</label>
                        <Field
                          component={InputField}
                          name="details.first_name"
                          validate={required("First name is require field")}
                        />
                      </UserFormRow>
                      <UserFormRow>
                        <label htmlFor="details.last_name">Last name</label>
                        <Field
                          component={InputField}
                          name="details.last_name"
                          validate={required("Last name is require field")}
                        />
                      </UserFormRow>
                    </UserFormGroup>

                    <UserFormGroup>
                      <h3>Contact details</h3>
                      <UserFormRow>
                        <label htmlFor="contact.email_primary">Email</label>
                        <Field
                          component={InputField}
                          name="contact.email_primary"
                          validate={composeValidators(
                            required("Email is require field"),
                            validEmail()
                          )}
                        />
                      </UserFormRow>
                      <UserFormRow>
                        <label htmlFor="contact.phone_primary">Phone</label>
                        <Field
                          component={InputField}
                          name="contact.phone_primary"
                          validate={composeValidators(
                            required("Phone number is require field"),
                            validUkNumber()
                          )}
                        />
                      </UserFormRow>
                    </UserFormGroup>
                    <UserFormRow>
                      <Field
                        name="block"
                        render={({ input }) => (
                          <CheckboxBlock>
                            <input type="checkbox" {...input} />
                            <span>Blocked</span>
                          </CheckboxBlock>
                        )}
                        type="checkbox"
                      />
                    </UserFormRow>
                    <UserFormAction>
                      <ButtonsGroup>
                        <SecondaryButton type="button">
                          Change Password
                        </SecondaryButton>
                      </ButtonsGroup>
                      <ButtonsGroup>
                        <SecondaryButton
                          onClick={() => history.push("/admin")}
                          type="button"
                        >
                          Cancel
                        </SecondaryButton>
                        <PrimaryButton
                          disabled={submitting || !valid}
                          type="submit"
                        >
                          Update
                        </PrimaryButton>
                      </ButtonsGroup>
                    </UserFormAction>
                  </form>
                )}
              />
            );
          }}
        </Query>
      </UserFormWrapper>
    </UserForm>
  );
};

export default Edit;
