import React, { useState } from "react";

import { Query } from "react-apollo";
import { Form } from "react-final-form";
import { Link } from "react-router-dom";
import { oc } from "ts-optchain";

import { common_api_generic_query_QueryParamsFilterInput as ParamsFilterInput } from "../../globalTypes";
import { PrimaryButton } from "../../ui/atoms/Button";
import SystemError from "../../ui/atoms/SystemError";
import { Table, Tbody, Thead } from "../../ui/atoms/Table";
import TableOverlay from "../../ui/atoms/TableOverlay";
import TableWrapper from "../../ui/atoms/TableWrapper";
import FilterSearch from "../../ui/molecules/FilterSearch";
import Pagebar from "../../ui/molecules/Pagebar";
import Pagination from "../../ui/molecules/Pagination";
import Sortable from "../../ui/molecules/Sortable";
import { makeFilters } from "../../util/search_filters";
import {
  AllAdmins,
  AllAdminsVariables,
} from "./graphql/queries/__generated__/AllAdmins";
import ALL_ADMINS from "./graphql/queries/AllAdmins.graphql";
import ListHead from "./ListHead";
import ListItem from "./ListItem";

export type ListProps = {
  canAdd?: boolean;
  canAssign?: boolean;
  propertyId?: string;
  offerId?: string;
  invoiceId?: string;
  setModalVisibility?: (open: boolean) => void;
};

const List: React.FC<ListProps> = ({
  canAdd = true,
  canAssign = false,
  propertyId,
  offerId,
  invoiceId,
  setModalVisibility,
}) => {
  const [filter, setFilter] = useState<ParamsFilterInput[]>([]);

  const handleFilterSubmit = (values) => {
    setFilter(
      makeFilters({
        values,
        filters: [
          {
            fields: [
              "details.first_name",
              "details.last_name",
              "contact.email_primary",
              "contact.phone_primary",
            ],
            key: "search",
            operator: "CONTAINS_ANY",
            transformation: {
              preprocessor: "REGEXP_REPLACE",
              param: ["[- /]", ""],
            },
          },
        ],
      })
    );
  };

  return (
    <>
      <h2>Admin Users</h2>

      <div className="d-f jc-sb">
        <Form
          onSubmit={handleFilterSubmit}
          render={({ handleSubmit, values, form }) => (
            <form onSubmit={handleSubmit}>
              <FilterSearch
                handleReset={() => {
                  form.reset();
                  handleFilterSubmit({});
                }}
                name="search"
                placeholder="Enter full name, email or phone number"
                values={values}
              />
            </form>
          )}
        />

        {canAdd && (
          <Link className="link-button" to="/admin/add">
            <PrimaryButton>Add new Admin User</PrimaryButton>
          </Link>
        )}
      </div>

      <Sortable field={["id"]}>
        {({ field, order, setSort }) => (
          <Pagination filter={filter}>
            {({ page, setPage, perPage }) => {
              return (
                <div>
                  <Query<AllAdmins, AllAdminsVariables>
                    query={ALL_ADMINS}
                    variables={{
                      filter,
                      pagination: { page, perPage },
                      sort: field.map((v) => ({ field: v, order })),
                    }}
                  >
                    {({ data, loading, error }) => {
                      if (error) {
                        return <SystemError error={error} />;
                      }

                      const getOrder = (fieldName) => {
                        return field.includes(fieldName)
                          ? order === 1
                            ? -1
                            : 1
                          : 1;
                      };

                      return (
                        <TableWrapper>
                          {loading && <TableOverlay />}
                          <Table>
                            <Thead>
                              <ListHead
                                field={field}
                                getOrder={getOrder}
                                order={order}
                                setSort={setSort}
                              />
                            </Thead>
                            <Tbody>
                              {oc(data)
                                .admin.allAdmins.data([])
                                .map((admin) => {
                                  if (!admin) {
                                    return null;
                                  }
                                  return (
                                    <ListItem
                                      key={oc(admin).id("")}
                                      adminId={oc(admin).id("")}
                                      block={oc(admin).block(false)}
                                      canAssign={canAssign}
                                      contact={admin.contact as any}
                                      details={admin.details as any}
                                      invoiceId={invoiceId}
                                      offerId={offerId}
                                      propertyId={propertyId}
                                      setModalVisibility={setModalVisibility}
                                    />
                                  );
                                })}
                            </Tbody>
                          </Table>

                          <Pagebar
                            changePage={setPage}
                            curPage={page}
                            showNPages={5}
                            total={Math.ceil(
                              oc(data).admin.allAdmins.total(1) / perPage
                            )}
                          />
                        </TableWrapper>
                      );
                    }}
                  </Query>
                </div>
              );
            }}
          </Pagination>
        )}
      </Sortable>
    </>
  );
};

export default List;
