import React, { FC, useCallback, useMemo, useState } from "react";

import styled from "@emotion/styled";
import AntdButton from "antd-button-color";
import { format } from "date-fns";
import { useMutation, useQuery } from "react-apollo";

import {
  common_api_generic_query_QueryFilterOperatorEnum as OperatorEnum,
  expenses_api_AdvanceStatusEnum,
} from "../../globalTypes";
import { Checked } from "../../ui/assets/checked";
import * as Button from "../../ui/atoms/Button";
import { ModalAlert, ModalConfirm, ModalWide } from "../../ui/molecules/Modal";
import { formatPrice } from "../../util/price";
import { DEFAULT_FILTER } from "./constants";
import {
  ChangeAdvancedRentStatus,
  ChangeAdvancedRentStatusVariables,
} from "./graphql/__generated__/ChangeAdvancedRentStatus";
import {
  GetAdminRecords,
  GetAdminRecordsVariables,
} from "./graphql/__generated__/GetAdminRecords";
import CHANGE_AR_STATUS from "./graphql/change-advanced-rent-status.graphql";
import GET_RECORDS from "./graphql/records.graphql";

type Props = {
  propertyId: string;
  textButton?: string;
};

export const AdvancedRent: FC<Props> = ({
  propertyId,
  textButton = "Advanced rent",
}) => {
  const [isOpen, setOpen] = useState(false);
  const [solution, setSolution] = useState<"confirm" | "deny" | undefined>();
  const [solutionResult, setSolutionResult] = useState<string | undefined>();

  const [makeSolution, { loading: makeSolutionLoading }] = useMutation<
    ChangeAdvancedRentStatus,
    ChangeAdvancedRentStatusVariables
  >(CHANGE_AR_STATUS);

  const { data, loading, error, refetch } = useQuery<
    GetAdminRecords,
    GetAdminRecordsVariables
  >(GET_RECORDS, {
    variables: {
      params: {
        filter: [
          ...DEFAULT_FILTER.filter(
            (f) =>
              f.fields?.includes("type") || f.fields?.includes("income_type")
          ),
          {
            fields: ["property_id"],
            value: [propertyId],
            operator: OperatorEnum.EQUALS,
          },
          {
            operator: OperatorEnum.EQUALS,
            fields: ["value.advance_status"],
            value: ["1"],
          },
        ],
        pagination: {
          page: 1,
          per_page: 10,
        },
      },
    },
    skip: !Boolean(propertyId),
  });

  const recordsData = data?.admin?.records?.records;

  const handleOpen = useCallback(() => {
    setOpen(true);
  }, []);

  const handleClose = useCallback(() => {
    setOpen(false);
  }, []);

  const handleDeny = useCallback(() => {
    setSolution("deny");
    handleClose();
  }, [handleClose]);

  const handleAccept = useCallback(() => {
    setSolution("confirm");
    handleClose();
  }, [handleClose]);

  const handleCancel = useCallback(() => {
    handleOpen();
    setSolution(undefined);
  }, [handleOpen]);

  const handleSubmit = useCallback(() => {
    if (solution === undefined) {
      return;
    }

    const request_id = recordsData?.find(
      (i) => i.type?.income?.advance?.request_id != null
    )?.type?.income?.advance?.request_id;
    const status =
      solution === "confirm"
        ? expenses_api_AdvanceStatusEnum.ADVANCE_STATUS_GRANTED
        : expenses_api_AdvanceStatusEnum.ADVANCE_STATUS_DENIED;

    if (request_id != null) {
      makeSolution({
        variables: {
          request_id,
          status,
        },
      })
        .then(
          (res) => {
            if (res.data?.admin?.updateAdvanceStatus?.records != null) {
              setSolutionResult("Advanced rent status was changed.");
            } else if ((res.errors ?? []).length > 0) {
              setSolutionResult(res.errors?.join(", "));
            }

            refetch();
          },
          (error) => {
            setSolutionResult(
              error instanceof Error ? error.message : String(error)
            );
          }
        )
        .catch((error) =>
          setSolutionResult(
            error instanceof Error ? error.message : String(error)
          )
        );
    }
  }, [recordsData, makeSolution, refetch, solution]);

  const filteredData = useMemo(() => {
    let result = {
      request: 0,
      willGet: 0,
      rentFee: 0,
      collectionFee: 0,
    };

    for (const item of recordsData ?? []) {
      result = {
        request: result.request + item.amount,
        rentFee:
          result.rentFee + (item.type.income?.advance?.fee_value_advance ?? 0),
        collectionFee:
          result.collectionFee +
          (item.type.income?.advance?.fee_value_rent_plan ?? 0),
        willGet: result.willGet + item.amount,
      };
    }

    return result;
  }, [recordsData]);

  return (
    <>
      <AntdButton
        disabled={
          loading ||
          (data?.admin?.records?.total ?? 0) === 0 ||
          Boolean(error) ||
          makeSolutionLoading
        }
        onClick={handleOpen}
        style={{ paddingRight: 0, paddingLeft: 0 }}
        type="link"
      >
        {textButton}
      </AntdButton>
      {solutionResult != null ? (
        <ModalAlert onCancel={() => setSolutionResult(undefined)}>
          <span>{solutionResult}</span>
        </ModalAlert>
      ) : null}
      {solution != null ? (
        <ModalConfirm onCancel={handleCancel} onConfirm={handleSubmit}>
          <span>
            {solution === "confirm" ? (
              <>You confirm that the landlord will be paid the advanced rent</>
            ) : (
              <>You deny the landlord the advanced rent</>
            )}
          </span>
        </ModalConfirm>
      ) : null}

      <ModalWide
        isOpen={isOpen}
        onCancel={handleClose}
        title="Advanced Rent"
        width={440}
      >
        {data?.admin?.records?.records?.map((i) => (
          <Item key={i.id}>
            <Checked />
            <Month>{format(new Date(i.time), "MMM yyyy")}</Month>
            <Sum>
              {formatPrice(i.amount + (i.type.income?.advance?.fee_value ?? 0))}
            </Sum>
          </Item>
        ))}

        <Details>
          <DetailsRow>
            <DRTitle>Request</DRTitle>
            <DRValue>{formatPrice(filteredData?.request)}</DRValue>
          </DetailsRow>
          <DetailsRow>
            <DRTitle>Advanced rent Fee</DRTitle>
            <DRValue>{`-${formatPrice(filteredData?.rentFee)}`}</DRValue>
          </DetailsRow>
          <DetailsRow>
            <DRTitle>rent collection Fee</DRTitle>
            <DRValue>{`-${formatPrice(filteredData?.collectionFee)}`}</DRValue>
          </DetailsRow>
          <DetailsRow>
            <DRTitle>Landlord will get</DRTitle>
            <DRValueGet>{formatPrice(filteredData?.willGet)}</DRValueGet>
          </DetailsRow>
        </Details>
        <ActionRow>
          <Button.DeleteButton onClick={handleDeny} type="button">
            Deny
          </Button.DeleteButton>
          &nbsp;&nbsp;&nbsp;
          <Button.SuccessButton onClick={handleAccept}>
            Confirm
          </Button.SuccessButton>
        </ActionRow>
      </ModalWide>
    </>
  );
};

const ActionRow = styled.div`
  display: flex;
  justify-content: space-between;
  margin-top: 32px;

  button {
    width: 48%;
    padding-left: 0;
    padding-right: 0;
  }
`;

const Item = styled.div`
  display: flex;
  background: #afc4d233;
  padding: 16px;
  align-items: center;
  justify-content: space-between;
  border-radius: 5px;
  border: solid 1px #aec5d5;
  margin: 0 0 12px;
`;

const Month = styled.div`
  font-size: 18px;
  color: #2b2f41;
  margin: 0 auto 0 12px;
`;

const Sum = styled.div`
  font-size: 18px;
  color: #2b2f41;
  margin: 0 0 0 auto;
  font-weight: 600;
`;

const Details = styled.div`
  margin: 24px 0;
`;

const DetailsRow = styled.div`
  display: flex;
  align-items: flex-end;
  justify-content: space-between;
  margin: 16px 0 0;
`;

const DRTitle = styled.div`
  text-transform: uppercase;
  font-size: 12px;
  flex: 1;
  line-height: 1;
  display: flex;
  color: #96b0bc;
  font-weight: 700;

  &:after {
    content: "";
    flex: 1;
    margin: 0 10px 2px;
    align-self: flex-end;
    border-bottom: dashed #afc4d2 1px;
  }
`;

const DRValue = styled.div`
  font-size: 16px;
  font-weight: 600;
  line-height: 1;
  color: #96b0bc;
`;

const DRValueGet = styled(DRValue)`
  color: #2b2f41;
`;
