import React, { FC, useMemo } from "react";

import { format } from "date-fns";
import { useQuery } from "react-apollo";

import { expenses_api_AdvanceStatusEnum } from "../../globalTypes";
import EmailLink from "../../ui/atoms/EmailLink";
import Spinner from "../../ui/atoms/Spinner";
import { NoWrap, Td, Tr } from "../../ui/atoms/Table";
import { formatPrice } from "../../util/price";
import { AdvancedRent } from "./AdvancedRent";
import { AdvanceStatus } from "./AdvanceStatus";
import { GetAdminRecords_admin_records_records } from "./graphql/__generated__/GetAdminRecords";
import {
  GetARRProperty,
  GetARRPropertyVariables,
} from "./graphql/__generated__/GetARRProperty";
import GET_PROPERTY from "./graphql/get-property.graphql";

type Props = {
  record: GetAdminRecords_admin_records_records;
  totalAmount: number;
  feeValue: number;
  monthCount: number;
};

export const ListItem: FC<Props> = ({
  record,
  totalAmount,
  feeValue,
  monthCount,
}) => {
  const { data, loading } = useQuery<GetARRProperty, GetARRPropertyVariables>(
    GET_PROPERTY,
    {
      variables: {
        id: record.property_id,
      },
      skip: !Boolean(record.property_id),
    }
  );
  const advanceStatus = record.type.income?.advance?.advance_status;
  const { location, people } = data?.admin?.Property ?? {};
  const { details, contact } = people?.landlord ?? {};
  const { city, postcode } = location ?? {};
  const totalAmountWithFeeValue = totalAmount + feeValue;

  const address = useMemo(() => {
    if (!location) {
      return "";
    }

    const { address1, address2, address3, address4 } = location;

    return [address1, address2, address3, address4].filter(Boolean).join(", ");
  }, [location]);

  return (
    <Tr>
      <Td colSpan={4}>
        {loading ? (
          <Spinner size="sm" />
        ) : (
          <>
            {address}
            {city && (
              <div>
                <b>{city}</b>
              </div>
            )}
            {postcode && (
              <div>
                <b>{postcode}</b>
              </div>
            )}
          </>
        )}
      </Td>
      <Td colSpan={2}>
        {loading ? (
          <Spinner size="sm" />
        ) : (
          <>
            <NoWrap>
              <b>
                {details?.first_name}
                &nbsp;
                {details?.last_name}
              </b>
            </NoWrap>
            {contact?.email_primary && (
              <NoWrap title={contact.email_primary}>
                <EmailLink href={`mailto:${contact.email_primary ?? ""}`}>
                  {contact.email_primary}
                </EmailLink>
              </NoWrap>
            )}
            {contact?.phone_primary && <div>{contact.phone_primary}</div>}
          </>
        )}
      </Td>
      <Td colSpan={2}>
        {Boolean(record?.time)
          ? format(new Date(record.time), "MMMM yyyy")
          : "- - -"}
      </Td>
      <Td colSpan={2}>{monthCount}</Td>
      <Td colSpan={2}>{formatPrice(totalAmountWithFeeValue)}</Td>
      <Td colSpan={2}>{formatPrice(feeValue)}</Td>
      <Td colSpan={2}>
        {Boolean(record.type.income?.advance?.created_at)
          ? format(
              new Date(record.type.income?.advance?.created_at as string),
              "dd MMM yyyy"
            )
          : "- - -"}
      </Td>
      <Td colSpan={2}>
        {advanceStatus ===
        expenses_api_AdvanceStatusEnum.ADVANCE_STATUS_REQUESTED ? (
          <AdvancedRent
            propertyId={record.property_id}
            textButton="Make solution"
          />
        ) : (
          <AdvanceStatus status={advanceStatus} />
        )}
      </Td>
    </Tr>
  );
};
