import React, { useState } from "react";

import styled from "@emotion/styled";
import { Query } from "react-apollo";
import { Field, Form } from "react-final-form";

import {
  common_api_generic_query_QueryFilterOperatorEnum,
  common_api_generic_query_QueryParamsFilterInput,
} from "../../globalTypes";
import { PrimaryButton, SecondaryButton } from "../../ui/atoms/Button";
import {
  FilterSort as FilterSortOrigin,
  FilterSortItem as FilterSortItemOrigin,
} from "../../ui/atoms/FilterSort";
import { TextInput } from "../../ui/atoms/Input";
import { SearchFieldWide } from "../../ui/atoms/SearchField";
import SystemError from "../../ui/atoms/SystemError";
import { Table as TableOrigin, Tbody } from "../../ui/atoms/Table";
import TableOverlay from "../../ui/atoms/TableOverlay";
import TableWrapper from "../../ui/atoms/TableWrapper";
import Dropdown from "../../ui/molecules/Dropdown";
import Pagebar from "../../ui/molecules/Pagebar";
import Pagination from "../../ui/molecules/Pagination";
import Sortable from "../../ui/molecules/Sortable";
import { wrapInputEmptyOption } from "../../util/dropdown";
import getSortOrder from "../../util/ger_sort_order";
import { Option, StatusOptions, TypeOptions } from "./constants";
import {
  AllInvitations,
  AllInvitations_admin_allInvitations_data,
  AllInvitationsVariables,
} from "./graphql/queries/__generated__/AllInvitations";
import ALL_INVITATIONS from "./graphql/queries/allInvitations.graphql";
import ListHead from "./ListHead";
import ListItem from "./ListItem";

type InvitationsFilterState = common_api_generic_query_QueryParamsFilterInput[];

type FormProps = {
  search: string;
  status: Option;
  type: Option;
  created_at: string;
};

const List = () => {
  const [filter, setFilter] = useState<InvitationsFilterState>([]);

  const handleFilterSubmit = (values) => {
    const newFilter: InvitationsFilterState = [];

    if (values.search) {
      newFilter.push({
        fields: [
          "property.property_id",
          "offer",
          "host_id",
          "invitee.user_id",
          "invitee.name",
          "invitee.email",
          "id",
        ],
        value: [values.search],
        operator: common_api_generic_query_QueryFilterOperatorEnum.EQUALS,
      });
    }

    if (values.status) {
      newFilter.push({
        operator: common_api_generic_query_QueryFilterOperatorEnum.EQUALS,
        fields: ["status"],
        value: [values.status.value],
      });
    }

    if (values.created_at) {
      newFilter.push({
        operator: common_api_generic_query_QueryFilterOperatorEnum.EQUALS,
        fields: ["created_at"],
        value: [values.created_at],
      });
    }

    if (values.type) {
      newFilter.push({
        operator: common_api_generic_query_QueryFilterOperatorEnum.EQUALS,
        fields: ["type"],
        value: [values.type.value],
      });
    }

    setFilter(newFilter);
  };

  return (
    <div>
      <h2>Invitations List</h2>

      <Form<FormProps>
        onSubmit={handleFilterSubmit}
        render={({ handleSubmit, values, form, pristine }) => (
          <form onSubmit={handleSubmit}>
            <SearchFieldWide>
              <Field
                component={TextInput}
                name="search"
                placeholder="Invitation ID, User ID or Property ID"
              />
              <PrimaryButton
                disabled={!values.search || values.search.length === 0}
                onClick={() => handleFilterSubmit(values)}
                type="submit"
              >
                Search
              </PrimaryButton>
              {!pristine ? (
                <>
                  &nbsp;&nbsp;
                  <SecondaryButton
                    onClick={() => {
                      setFilter([]);
                      form.reset();
                    }}
                    type="reset"
                  >
                    Clear
                  </SecondaryButton>
                </>
              ) : null}
            </SearchFieldWide>
            <FilterSort>
              <FilterSortItem>
                <Field
                  name="status"
                  render={({ input }) => {
                    return (
                      <Dropdown
                        input={wrapInputEmptyOption(input)}
                        options={StatusOptions}
                        placeholder="Status"
                      />
                    );
                  }}
                />
              </FilterSortItem>
              <FilterSortItem>
                <Field
                  name="type"
                  render={({ input }) => {
                    return (
                      <Dropdown
                        input={wrapInputEmptyOption(input)}
                        options={TypeOptions}
                        placeholder="Invitation Type"
                      />
                    );
                  }}
                />
              </FilterSortItem>
              <FilterSortItem>
                <Field
                  className="input-sort-date"
                  component={TextInput}
                  name="created_at"
                  type="date"
                />
              </FilterSortItem>

              <FilterSortItem>
                <PrimaryButton
                  disabled={pristine}
                  onClick={() => handleFilterSubmit(values)}
                >
                  Filter
                </PrimaryButton>
              </FilterSortItem>
            </FilterSort>
          </form>
        )}
      />

      <Sortable field={["created_at"]} order={-1}>
        {({ field, order, setSort }) => (
          <Pagination filter={filter} perPage={25}>
            {({ page, setPage, perPage }) => {
              return (
                <div>
                  <Query<AllInvitations, AllInvitationsVariables>
                    query={ALL_INVITATIONS}
                    variables={{
                      filter,
                      pagination: { page, perPage },
                      sort: field.map((v) => ({ field: v, order })),
                    }}
                  >
                    {({ data, loading, error }) => {
                      if (error) {
                        return <SystemError error={error} />;
                      }

                      const getOrder = (fieldName) =>
                        getSortOrder({ field, fieldName, order });
                      const onSetPage = (pageNumber: number) =>
                        setPage(pageNumber);

                      const totalRows =
                        data?.admin?.allInvitations?.total || 20;

                      return (
                        <TableWrapper>
                          {loading && <TableOverlay />}

                          <Pagebar
                            changePage={onSetPage}
                            curPage={page}
                            showNPages={3}
                            total={Math.ceil(totalRows / perPage)}
                          />

                          <Table>
                            <ListHead
                              field={field}
                              getOrder={getOrder}
                              order={order}
                              setSort={setSort}
                            />
                            <Tbody>
                              {(data?.admin?.allInvitations?.data ?? []).map(
                                (
                                  item: AllInvitations_admin_allInvitations_data
                                ) => {
                                  if (item.id) {
                                    return (
                                      <ListItem key={item.id} data={item} />
                                    );
                                  }
                                  return null;
                                }
                              )}
                            </Tbody>
                          </Table>
                          <Pagebar
                            changePage={onSetPage}
                            curPage={page}
                            showNPages={3}
                            total={Math.ceil(totalRows / perPage)}
                          />
                        </TableWrapper>
                      );
                    }}
                  </Query>
                </div>
              );
            }}
          </Pagination>
        )}
      </Sortable>
    </div>
  );
};

const FilterSort = styled(FilterSortOrigin)`
  flex-wrap: wrap;
  padding-bottom: 0;
`;

const FilterSortItem = styled(FilterSortItemOrigin)`
  margin-bottom: 10px;

  input.input-sort-date {
    width: 200px;
  }
`;

const Table = styled(TableOrigin)`
  table-layout: fixed;
`;

export default List;
