import React, { FC, useCallback, useMemo } from "react";

import styled from "@emotion/styled";
import { useMutation, useQuery } from "react-apollo";
import { Field, Form } from "react-final-form";

import { document_api_DocumentStateEnum } from "../../globalTypes";
import { PrimaryButton, SecondaryButton } from "../../ui/atoms/Button";
import {
  UserFormAction,
  UserFormItem,
  UserFormRow,
  UserFormRowFlex,
} from "../../ui/atoms/UserForm";
import Dropdown from "../../ui/molecules/Dropdown";
import { InputFieldTouched } from "../../ui/molecules/InputField";
import { ModalWide } from "../../ui/molecules/Modal";
import { formatDateForDatepicker } from "../../util/date";
import { enumToList, formatGenericEnumValue } from "../../util/enums";
import { required } from "../../util/validations";
import { DocumentTypeOptions } from "./constants";
import {
  UpdateDocument,
  UpdateDocumentVariables,
} from "./graphql/mutations/__generated__/UpdateDocument";
import EDIT_DOCUMENT from "./graphql/mutations/UpdateDocument.graphql";
import {
  GetProperty_admin_Property_documents,
  GetProperty_admin_Property_documents_product,
} from "./graphql/queries/__generated__/GetProperty";
import {
  Products,
  Products_admin_products_product,
  ProductsVariables,
} from "./graphql/queries/__generated__/Products";
import GET_PROPERTY from "./graphql/queries/GetProperty.graphql";
import GET_PRODUCTS from "./graphql/queries/Products.graphql";

type Props = {
  document: GetProperty_admin_Property_documents;
  onClose: () => void;
  propertyId: string;
  landlordId: string;
};

interface FormValues {
  type: {
    code?: string;
    type_code: string;
    label: string;
  };
  state: document_api_DocumentStateEnum;
  expiration?: string;
  description?: string;
  linkToPurchase: {
    value:
      | Products_admin_products_product
      | GetProperty_admin_Property_documents_product
      | null;
    label: string | null;
  };
}

const DocumentEditModal: FC<Props> = ({
  document,
  onClose,
  propertyId,
  landlordId,
}) => {
  const [editDocument, { loading, error }] = useMutation<
    UpdateDocument,
    UpdateDocumentVariables
  >(EDIT_DOCUMENT);

  const { data } = useQuery<Products, ProductsVariables>(GET_PRODUCTS, {
    variables: {
      propertyId,
      landlordId,
    },
  });

  const options = (data?.admin?.products?.product ?? []).map((v) => ({
    value: v,
    label: `Product ID: ${v.id}; ${v.service?.service.title}`,
  }));

  const handleSubmit = useCallback(
    (values: FormValues) => {
      const { expiration, type, ...vars } = values;
      const product_id = values.linkToPurchase.value?.id;
      const expirationDate = expiration
        ? new Date(expiration).toUTCString()
        : undefined;

      editDocument({
        variables: {
          id: document.id,
          property: propertyId,
          file: document.file.id,
          expiration: expirationDate,
          code: type.code,
          type_code: type.type_code,
          product_id,
          ...vars,
        },
        refetchQueries: [
          {
            query: GET_PROPERTY,
            variables: { id: propertyId },
          },
        ],
      }).then((data) => data.data?.admin?.updateDocument?.success && onClose());
    },
    [editDocument, propertyId, onClose, document.id, document.file.id]
  );

  const initialValues: Partial<FormValues> = useMemo(
    () => ({
      type: DocumentTypeOptions.find(
        (o) =>
          o.code === document.code ||
          (!o.code && o.type_code === document.type_code)
      ),
      state: document.state,
      expiration: formatDateForDatepicker(document.expiration),
      description: document.description,
      linkToPurchase: {
        value: document.product_id ? document.product : null,
        label: document.product_id
          ? `Product ID: ${document.product?.id}; ${document.product?.service?.service.title}`
          : null,
      },
    }),
    [document]
  );

  return (
    <ModalWide onCancel={onClose} title="Edit document" withoutOverflow>
      <DocumentName>{document.file.name}</DocumentName>

      <Form<FormValues> initialValues={initialValues} onSubmit={handleSubmit}>
        {({ handleSubmit, hasValidationErrors }) => (
          <form onSubmit={handleSubmit}>
            <UserFormRowFlex>
              <UserFormItem>
                <label htmlFor="type">Document type</label>
                <Field
                  name="type"
                  render={({ input, meta }) => (
                    <Dropdown
                      error={meta.touched && meta.error}
                      input={input}
                      options={DocumentTypeOptions}
                      placeholder="-- select --"
                    />
                  )}
                  validate={required("Required field")}
                />
              </UserFormItem>
              <UserFormItem>
                <label htmlFor="state">Document state</label>
                <Field
                  name="state"
                  render={({ input, meta }) => (
                    <Dropdown
                      error={meta.touched && meta.error}
                      input={input}
                      itemToString={formatGenericEnumValue}
                      options={enumToList(document_api_DocumentStateEnum)}
                      placeholder="-- select --"
                    />
                  )}
                  validate={required("Required field")}
                />
              </UserFormItem>
              <UserFormItem>
                <label htmlFor="expiration">Expire date</label>
                <Field
                  component={InputFieldTouched}
                  name="expiration"
                  type="date"
                />
              </UserFormItem>
            </UserFormRowFlex>
            <UserFormRow>
              <label htmlFor="linkToPurchase">Link to purchase</label>
              <Field
                name="linkToPurchase"
                render={({ input, meta }) => (
                  <Dropdown
                    error={meta.touched && meta.error}
                    input={input}
                    inputStyle={{ width: "100%" }}
                    options={options}
                    style={{ width: "65%" }}
                  />
                )}
              />
            </UserFormRow>
            <UserFormRow>
              <label htmlFor="description">Description</label>
              <Field component={InputFieldTouched} name="description" />
            </UserFormRow>

            {error && <div>{error?.message}</div>}

            <UserFormAction>
              <SecondaryButton onClick={onClose} type="button">
                Cancel
              </SecondaryButton>
              <PrimaryButton disabled={hasValidationErrors || loading}>
                Save
              </PrimaryButton>
            </UserFormAction>
          </form>
        )}
      </Form>
    </ModalWide>
  );
};

export default DocumentEditModal;

const DocumentName = styled.div`
  font-size: 18px;
  margin-bottom: 20px;
`;
