import React, { FC, useCallback, useState } from "react";

import styled from "@emotion/styled";
import { useMutation } from "react-apollo";
import { Field } from "react-final-form";

import { document_api_DocumentStateEnum } from "../../globalTypes";
import LinkWithPreviewFile from "../../ui/atoms/LinkWithPreviewFile";
import Spinner from "../../ui/atoms/Spinner";
import { TdInner, TdInnerWidth30, TrInner } from "../../ui/atoms/Table";
import Dropdown from "../../ui/molecules/Dropdown";
import { Modal, ModalConfirm } from "../../ui/molecules/Modal";
import theme from "../../ui/theme";
import { formatDate } from "../../util/date";
import { enumToList, formatGenericEnumValue } from "../../util/enums";
import { isImageMimeType } from "../../util/helpers";
import { DocumentTypeCodes } from "./constants";
import DocumentEditModal from "./DocumentEditModal";
import { documentTypeFilter } from "./filters";
import {
  DeleteDocument,
  DeleteDocumentVariables,
} from "./graphql/mutations/__generated__/DeleteDocument";
import {
  UpdatePropertyDescription,
  UpdatePropertyDescriptionVariables,
} from "./graphql/mutations/__generated__/UpdatePropertyDescription";
import DELETE_DOCUMENT from "./graphql/mutations/DeleteDocument.graphql";
import UPDATE_PROPERTY_DESCRIPTION from "./graphql/mutations/UpdatePropertyDescription.graphql";
import { GetProperty_admin_Property_documents } from "./graphql/queries/__generated__/GetProperty";
import VerifyContext from "./PropertyEditContext";

type Props = {
  document: GetProperty_admin_Property_documents;
  documents: ReadonlyArray<GetProperty_admin_Property_documents> | null;
  index: number;
  propertyId: string;
  landlordId: string;
};

const DocumentsRow: FC<Props> = ({
  document,
  documents,
  index,
  propertyId,
  landlordId,
}) => {
  const [showEditModal, setShowEditModal] = useState(false);
  const [showConfirm, setConfirm] = useState<boolean>(false);
  const [showAlert, setAlert] = useState<string | null>(null);
  const [deleteDocument, { loading }] = useMutation<
    DeleteDocument,
    DeleteDocumentVariables
  >(DELETE_DOCUMENT);
  const [
    updatePropertyDescription,
    { loading: updatePropertyDescriptionLoading },
  ] = useMutation<
    UpdatePropertyDescription,
    UpdatePropertyDescriptionVariables
  >(UPDATE_PROPERTY_DESCRIPTION);

  const handleUpdatePropertyDescription = useCallback(async () => {
    const isPlan = document.type_code === DocumentTypeCodes.plan;

    if (isPlan) {
      const plans = documents
        ?.filter(
          (doc) =>
            doc.type_code === DocumentTypeCodes.plan && doc.id !== document.id
        )
        .map((doc) => doc.id);

      const response = await updatePropertyDescription({
        variables: {
          description: {
            plans,
          },
          id: propertyId,
        },
      });

      if (response.data?.admin?.updateProperty?.id) {
        setAlert("Document was deleted successfully!");
      } else {
        const errorMessage =
          response.errors?.map((err) => err.message).join(", ") || null;
        setAlert(errorMessage || "Something was wrong, try again later");
      }
    }
  }, [
    document.id,
    documents,
    document.type_code,
    propertyId,
    updatePropertyDescription,
  ]);

  const handleDelete = useCallback(() => {
    deleteDocument({
      variables: {
        id: document.id,
      },
      awaitRefetchQueries: true,
      refetchQueries: ["GetAllDocuments"],
    }).then((res) => {
      setConfirm(false);

      if (res.data?.admin?.deleteDocument?.success) {
        handleUpdatePropertyDescription();
      } else {
        const errorMessage =
          (res.errors ?? []).length > 0
            ? res.errors?.map((err) => err.message).join(", ")
            : null;

        setAlert(
          errorMessage != null
            ? errorMessage
            : "Something was wrong, try again later"
        );
      }
    });
  }, [deleteDocument, document.id, handleUpdatePropertyDescription]);

  const fileName =
    document.file.name.length > 28 ? (
      <>
        {document.file.name.substring(0, 20)}
        &nbsp;&hellip;&nbsp;
        {document.file.name.substr(-8)}
      </>
    ) : (
      document.file.name
    );

  return (
    <>
      {showConfirm && (
        <ModalConfirm
          onCancel={() => {
            setConfirm(false);
          }}
          onConfirm={() => {
            handleDelete();
          }}
        >
          <span>
            Are you sure you really <br />
            want to delete document <br />
            <b>{fileName}</b>?
          </span>
        </ModalConfirm>
      )}
      {showAlert && (
        <Modal
          onCancel={() => {
            setAlert(null);
          }}
        >
          <b>{showAlert}</b>
        </Modal>
      )}
      <VerifyContext.Consumer>
        {({ documentsVerified, submitting }) => (
          <TrInner>
            <TdInnerWidth30>
              <LinkWithPreviewFile
                href={`/media/${document.file.id}`}
                rel="noopener noreferrer"
                target="_blank"
                title={document.file.name}
              >
                {isImageMimeType(document.file.mime) && (
                  <img alt="" src={`/media/thumb-${document.file.id}`} />
                )}
                <span>{fileName}</span>
              </LinkWithPreviewFile>
            </TdInnerWidth30>
            <TdInner>
              <b>{documentTypeFilter(document.type, document.code)}</b>
            </TdInner>
            <TdInner>{document.description}</TdInner>
            <TdInner>{formatDate(document.expiration)}</TdInner>
            <TdInner>
              {documentsVerified && document ? (
                formatGenericEnumValue(document.state)
              ) : (
                <Field
                  name={`documents[${index}].state`}
                  render={({ input }) => {
                    return (
                      <Dropdown
                        disabled={documentsVerified || submitting}
                        input={input}
                        itemToString={formatGenericEnumValue}
                        options={enumToList(document_api_DocumentStateEnum)}
                      />
                    );
                  }}
                />
              )}
            </TdInner>
            <TdInner>
              {!document.removed && (
                <ButtonEdit
                  onClick={() => setShowEditModal(true)}
                  type="button"
                >
                  Edit
                </ButtonEdit>
              )}
            </TdInner>
            <TdInner>
              {document.removed ? (
                <b>Deleted</b>
              ) : (
                <ButtonEdit onClick={() => setConfirm(true)} type="button">
                  Delete
                  {loading || updatePropertyDescriptionLoading ? (
                    <>
                      &nbsp;&nbsp;
                      <Spinner size="sm" />
                    </>
                  ) : null}
                </ButtonEdit>
              )}
            </TdInner>
          </TrInner>
        )}
      </VerifyContext.Consumer>

      {showEditModal && (
        <DocumentEditModal
          document={document}
          landlordId={landlordId}
          onClose={() => setShowEditModal(false)}
          propertyId={propertyId}
        />
      )}
    </>
  );
};

export default DocumentsRow;

const ButtonEdit = styled.button`
  font-weight: 700;
  font-size: 1em;
  color: ${theme.colors.icons.darkBlue};
  background-color: transparent;
  border: none;
  cursor: pointer;
`;
