// WORKAROUND: keep PropertyHead on top to prevent css-imports issue while build
import React, { useEffect, useState } from "react";

import { NormalizedCacheObject } from "apollo-cache-inmemory";
import ApolloClient from "apollo-client";
import { isEqual } from "apollo-utilities";
// eslint-disable-next-line
import { FORM_ERROR } from "final-form";
import { useApolloClient, useQuery } from "react-apollo";
import { Form } from "react-final-form";
import { RouteComponentProps } from "react-router-dom";

import PageError from "../../components/PageError";
import SystemError from "../../ui/atoms/SystemError";
import Tabs from "../../ui/molecules/Tabs";
import { TABS } from "./constants";
import EditForm from "./EditForm";
import updatePropertyFilter from "./filters/update_property";
import {
  UnpublishPropertyOnMashroom,
  UnpublishPropertyOnMashroomVariables,
} from "./graphql/mutations/__generated__/UnpublishPropertyOnMashroom";
import {
  UpdateDocumentState,
  UpdateDocumentStateVariables,
} from "./graphql/mutations/__generated__/UpdateDocumentState";
import {
  UpdateProperty,
  UpdatePropertyVariables,
} from "./graphql/mutations/__generated__/UpdateProperty";
import UNPUBLISH_PROPERTY_ON_MASHROOM from "./graphql/mutations/UnpublishPropertyOnMashroom.graphql";
import UPDATE_DOCUMENT_STATE from "./graphql/mutations/UpdateDocumentState.graphql";
import UPDATE_PROPERTY from "./graphql/mutations/UpdateProperty.graphql";
import {
  GetProperty,
  GetProperty_admin_Property,
  GetPropertyVariables,
} from "./graphql/queries/__generated__/GetProperty";
import GET_PROPERTY from "./graphql/queries/GetProperty.graphql";
import VerifyContext from "./PropertyEditContext";
import PropertyHead from "./PropertyHead";

type ParamsProps = {
  id?: string;
};
type Props = RouteComponentProps<ParamsProps> & {
  client: ApolloClient<NormalizedCacheObject>;
};

const Edit = (props: Props) => {
  const client = useApolloClient();
  const propertyId = props.match.params.id;
  const { data, error: errorProperty, refetch: refetchProperty } = useQuery<
    GetProperty,
    GetPropertyVariables
  >(GET_PROPERTY, {
    variables: { id: propertyId },
    fetchPolicy: "network-only",
  });

  const property = data?.admin?.Property;
  const ownerships = data?.admin?.getOwnerships?.ownerships;
  const ownership = ownerships && ownerships[0];

  const handleSubmit = async (values: GetProperty_admin_Property) => {
    await Promise.all(
      (values.documents ?? []).map((doc, idx) => {
        if (doc.state !== property?.documents?.[idx].state) {
          return client
            .mutate<UpdateDocumentState, UpdateDocumentStateVariables>({
              mutation: UPDATE_DOCUMENT_STATE,
              variables: {
                id: doc.id,
                state: doc.state,
              },
            })
            .then((res) =>
              res.errors
                ? { [FORM_ERROR]: res.errors.map((e) => e.message) }
                : undefined
            );
        } else {
          return Promise.resolve(undefined);
        }
      })
    );

    const data = await client.mutate<UpdateProperty, UpdatePropertyVariables>({
      mutation: UPDATE_PROPERTY,
      variables: updatePropertyFilter(values),
      refetchQueries: [
        {
          query: GET_PROPERTY,
          variables: { id: propertyId },
        },
      ],
    });
    const propertyResult = data.data?.admin?.updateProperty;
    if (propertyResult != null) {
      const verify = propertyResult.admin_verify;

      const isPublish = verify.publication.verified;
      const addressVerified = verify.address.verified;
      const descriptionVerified = verify.description.verified;
      const documentsVerified = verify.documents.verified;
      const feeVerified = verify.fee.verified;
      const photosVerified = verify.photos.verified;

      if (
        propertyId != null &&
        isPublish &&
        (!addressVerified ||
          !descriptionVerified ||
          !documentsVerified ||
          !feeVerified ||
          !photosVerified)
      ) {
        await client.mutate<
          UnpublishPropertyOnMashroom,
          UnpublishPropertyOnMashroomVariables
        >({
          mutation: UNPUBLISH_PROPERTY_ON_MASHROOM,
          variables: { id: propertyId },
          refetchQueries: [
            {
              query: GET_PROPERTY,
              variables: { id: propertyId },
            },
          ],
        });
      }
    }
  };

  const [publishedInMashroom, setPublishedInMashroom] = useState<
    boolean | undefined
  >(property?.admin_verify.publication.verified);
  useEffect(() => {
    if (property != null) {
      setPublishedInMashroom(property?.admin_verify.publication.verified);
    }
  }, [property]);

  if (errorProperty) {
    return <SystemError error={errorProperty} />;
  }
  if (property == null) {
    return <PageError code={404} />;
  }
  return (
    <Form
      initialValues={property}
      onSubmit={handleSubmit}
      render={({
        form,
        handleSubmit,
        values,
        submitting,
        valid,
        submitError,
      }) => {
        const somethingWasChanged = !isEqual(property, values);

        return (
          <VerifyContext.Provider
            value={{
              publishedInMashroom,
              addressVerified: property.admin_verify.address.verified,
              descriptionVerified: property.admin_verify.description.verified,
              documentsVerified: property.admin_verify.documents.verified,
              photosVerified: property.admin_verify.photos.verified,
              publishedInZoopla: property.zoopla.is_publish,
              publishedInRightmove: property.rightmove.is_publish,
              handleFormReset: form.reset,
              handleFormSubmit: form.submit,
              ownership,
              refetchProperty,
              setPublishedInMashroom,
              somethingWasChanged,
              submitting,
              valid,
            }}
          >
            <form onSubmit={handleSubmit}>
              <PropertyHead property={property} />
              {submitError != null ? <SystemError error={submitError} /> : null}
              <Tabs defaultTab={0}>
                {({
                  curTabIndex,
                  setTab,
                  TabList,
                  TabItem,
                  TabActive,
                  TabBody,
                  TabDisabled,
                }) => {
                  return (
                    <>
                      <TabList>
                        {TABS.map((tab, index) => {
                          if (index === curTabIndex) {
                            return <TabActive key={tab}>{tab}</TabActive>;
                          } else if (somethingWasChanged) {
                            return <TabDisabled key={tab}>{tab}</TabDisabled>;
                          }

                          return (
                            <TabItem key={tab} onClick={() => setTab(index)}>
                              {tab}
                            </TabItem>
                          );
                        })}
                      </TabList>
                      <TabBody>
                        <EditForm
                          curTabIndex={curTabIndex}
                          property={property}
                          values={values}
                        />
                      </TabBody>
                    </>
                  );
                }}
              </Tabs>
            </form>
          </VerifyContext.Provider>
        );
      }}
    />
  );
};

export default Edit;
