import React, { useCallback, useState } from "react";

import { useMutation, useQuery } from "@apollo/react-hooks";
import { Form } from "react-final-form";

import { document_api_DocumentStateEnum } from "../../../globalTypes";
import { ApiError } from "../../../ui/atoms/ApiError/ApiError";
import { SuccessButton } from "../../../ui/atoms/Button";
import { ModalWide } from "../../../ui/molecules/Modal";
import {
  CreateDocument,
  CreateDocumentVariables,
} from "../graphql/mutations/__generated__/CreateDocument";
import CREATE_DOCUMENT from "../graphql/mutations/CreateDocument.graphql";
import {
  Products,
  Products_admin_products_product,
  ProductsVariables,
} from "../graphql/queries/__generated__/Products";
import GET_PROPERTY from "../graphql/queries/GetProperty.graphql";
import GET_PRODUCTS from "../graphql/queries/Products.graphql";
import { DocumentForm } from "./DocumentForm";

import styles from "./styles.module.css";

type Props = {
  propertyId: string;
  disabled?: boolean;
  client?: any;
  landlordId: string;
};

export interface FormValues {
  file: string;
  type: {
    code?: string;
    type_code: string;
    label: string;
  };
  state: document_api_DocumentStateEnum | undefined;
  expiration?: string;
  description?: string;
  linkToPurchase?: {
    value: Products_admin_products_product | null;
    label: string | null;
  };
}

const FormDocumentAdd = ({ propertyId, disabled, landlordId }: Props) => {
  const [modalOpened, toggleModal] = useState<boolean>(false);

  const { data, error } = useQuery<Products, ProductsVariables>(GET_PRODUCTS, {
    variables: {
      propertyId,
      landlordId,
    },
  });

  const [createDocument, { loading: createLoading }] = useMutation<
    CreateDocument,
    CreateDocumentVariables
  >(CREATE_DOCUMENT);

  const handleModalClose = useCallback(() => toggleModal(false), []);

  const handleSubmit = useCallback(
    (values: FormValues) => {
      const {
        expiration,
        file,
        type,
        state,
        description,
        linkToPurchase,
      } = values;

      const expirationDate = expiration
        ? new Date(expiration).toUTCString()
        : undefined;

      return createDocument({
        variables: {
          property: propertyId,
          file,
          code: type.code,
          type_code: type.type_code,
          state,
          expiration: expirationDate,
          description,
          product_id: linkToPurchase?.value?.id,
        },
        refetchQueries: [
          {
            query: GET_PROPERTY,
            variables: { id: propertyId },
          },
          "GetAllDocuments",
        ],
      }).then((res) => {
        if (!res.data?.admin?.createDocument?.success) {
          window &&
            alert("Something went wrong. Please, try send document later.");
        } else {
          toggleModal(false);
        }
      });
    },
    [createDocument, propertyId, toggleModal]
  );

  return (
    <>
      <div className={styles.container}>
        <SuccessButton
          disabled={disabled || createLoading}
          onClick={() => toggleModal(true)}
          type="button"
        >
          Add Document
        </SuccessButton>
      </div>
      <ModalWide
        isOpen={modalOpened}
        onCancel={handleModalClose}
        title="Add document"
        withoutOverflow
      >
        <ApiError errors={[error]} />
        <Form<FormValues>
          onSubmit={handleSubmit}
          render={() => (
            <DocumentForm
              onCancel={handleModalClose}
              products={data?.admin?.products?.product}
            />
          )}
        />
      </ModalWide>
    </>
  );
};

export default FormDocumentAdd;
