import React, { useState } from "react";

import { FORM_ERROR } from "final-form";
import { useApolloClient, withApollo } from "react-apollo";
import { Field, Form } from "react-final-form";

import { document_api_DocumentTypeEnum } from "../../globalTypes";
import {
  PrimaryButton,
  SecondaryButton,
  SuccessButton,
} from "../../ui/atoms/Button";
import { ButtonsGroup } from "../../ui/atoms/UserForm";
import { FileField } from "../../ui/molecules/FileField/FileField";
import { ModalWide } from "../../ui/molecules/Modal";
import { required } from "../../util/validations";
import {
  CreateDocument,
  CreateDocumentVariables,
} from "./graphql/mutations/__generated__/CreateDocument";
import { UpdateProperty_admin_updateProperty_description } from "./graphql/mutations/__generated__/UpdateProperty";
import CREATE_DOCUMENT from "./graphql/mutations/CreateDocument.graphql";
import UPDATE_PROPERTY_DESCRIPTION from "./graphql/mutations/UpdatePropertyDescription.graphql";
import { GetProperty_admin_Property } from "./graphql/queries/__generated__/GetProperty";
import ALL_DOCUMENTS from "./graphql/queries/GetAllDocuments.graphql";
import GET_PROPERTY from "./graphql/queries/GetProperty.graphql";

type Props = {
  property: GetProperty_admin_Property;
  disabled?: boolean;
  client?: any;
  type: "addPhoto" | "addPlan";
};

type DescriptionType = Pick<
  UpdateProperty_admin_updateProperty_description,
  "post_area_pitch" | "post_comment" | "post_headline" | "post_property_pitch"
> & {
  photos: string[];
  plans: string[];
};

const FormPhotoAdd = (props: Props) => {
  const [modalOpened, toggleModal] = useState<boolean>(false);

  const client = useApolloClient();
  const handleSubmit = (values) => {
    const d = props.property.description;
    const description: DescriptionType = {
      post_area_pitch: d.post_area_pitch,
      post_comment: d.post_comment,
      post_headline: d.post_headline,
      post_property_pitch: d.post_property_pitch,
      photos: [
        ...(d.photos ? d.photos : []).map((i) => i.id),
        props.type === "addPhoto" ? values.file : "",
      ].filter(Boolean),
      plans: [
        ...(d.plans ? d.plans : []).map((i) => i.id),
        props.type === "addPlan" ? values.file : "",
      ].filter(Boolean),
    };

    const documentExists = props.property.documents?.some(
      (el) => el.file.id === values.file
    );
    const photoExists = props.property.description.photos?.some((el) =>
      el.id.match(values.file)
    );

    const type =
      props.type === "addPhoto"
        ? document_api_DocumentTypeEnum.PHOTO
        : props.type === "addPlan"
        ? document_api_DocumentTypeEnum.PLAN
        : null;

    try {
      !documentExists &&
        client
          .mutate<CreateDocument, CreateDocumentVariables>({
            mutation: CREATE_DOCUMENT,
            variables: {
              property: props.property.id,
              type,
              file: values.file,
            },
            refetchQueries: [
              {
                query: GET_PROPERTY,
                variables: {
                  id: props.property.id,
                },
              },
            ],
          })
          .then((data) => {
            if (!data?.data?.admin?.createDocument?.success) {
              window &&
                alert("Something went wrong. Please, try send document later.");
            }
          });

      return !photoExists
        ? props.client
            .mutate({
              mutation: UPDATE_PROPERTY_DESCRIPTION,
              variables: {
                description,
                id: props.property.id,
              },
              refetchQueries: [
                {
                  query: GET_PROPERTY,
                  variables: {
                    id: props.property.id,
                  },
                },
                {
                  query: ALL_DOCUMENTS,
                  variables: {
                    property: props.property.id,
                    removed: true,
                  },
                },
              ],
            })
            .then((response) => {
              if (response?.data.admin.updateProperty.id) {
                toggleModal(false);
              } else {
                window &&
                  alert("Something went wrong. Please, try more later.");
              }
            })
        : toggleModal(false);
    } catch (error) {
      return { [FORM_ERROR]: String(error) };
    }
  };
  const acceptedFileTypes =
    props.type === "addPhoto"
      ? ["image/gif", "image/jpeg", "image/png"]
      : ["application/pdf"];

  return (
    <>
      <SuccessButton
        disabled={props.disabled}
        onClick={() => toggleModal(true)}
        type="button"
      >
        {props.type === "addPhoto"
          ? "Add photo"
          : props.type === "addPlan"
          ? "Add floorplan"
          : "Add"}
      </SuccessButton>
      {modalOpened && (
        <ModalWide
          onCancel={() => toggleModal(false)}
          title="Add Photo"
          withoutOverflow
        >
          <Form
            onSubmit={handleSubmit}
            render={({ handleSubmit, submitting, valid }) => (
              <form onSubmit={handleSubmit}>
                <Field
                  name="file"
                  render={({ input, meta }) => {
                    return (
                      <FileField
                        acceptedTypes={acceptedFileTypes}
                        error={meta.touched && meta.error}
                        handleChange={input.onChange}
                      />
                    );
                  }}
                  validate={required()}
                />
                <ButtonsGroup>
                  <SecondaryButton
                    onClick={() => toggleModal(false)}
                    type="reset"
                  >
                    Cancel
                  </SecondaryButton>
                  <PrimaryButton disabled={!valid || submitting}>
                    &nbsp; Add &nbsp;
                  </PrimaryButton>
                </ButtonsGroup>
              </form>
            )}
          />
        </ModalWide>
      )}
    </>
  );
};

export default withApollo<Props>(FormPhotoAdd);
