import React, { useCallback, useMemo, useState } from "react";

import { format } from "date-fns";
import { useMutation, useQuery } from "react-apollo";
import { Field, Form } from "react-final-form";

import { account_api_OwnershipStatusEnum } from "../../../../globalTypes";
import { PrimaryButton, SecondaryButton } from "../../../../ui/atoms/Button";
import { TextInput } from "../../../../ui/atoms/Input";
import { UserFormRow } from "../../../../ui/atoms/UserForm";
import { CheckboxToggle } from "../../../../ui/molecules/CheckboxToggle";
import ErrorMessage from "../../../../ui/molecules/ErrorMessage";
import { FileField } from "../../../../ui/molecules/FileField/FileField";
import { Modal } from "../../../../ui/molecules/Modal";
import { TextareaField } from "../../../../ui/molecules/TextareaField";
import { required } from "../../../../util/validations";
import {
  UpdateOwnershipVerification,
  UpdateOwnershipVerificationVariables,
} from "../graphql/mutations/__generated__/UpdateOwnershipVerification";
import UPDATE_OWNERSHIP_VERIFICATION from "../graphql/mutations/UpdateOwnershipVerification.graphql";
import {
  GetAdmin,
  GetAdminVariables,
} from "../graphql/queries/__generated__/GetAdmin";
import GET_ADMIN from "../graphql/queries/GetAdmin.graphql";
import { ACCEPTED_FILE_TYPES } from "./constants";
import { FormValues, OwnershipModalProps } from "./types";

import styles from "./styles.module.css";

export const OwnershipModal = ({
  ownership,
  hideModal,
  refetchProperty,
}: OwnershipModalProps) => {
  const [updateOwnershipVerification, { loading, error }] = useMutation<
    UpdateOwnershipVerification,
    UpdateOwnershipVerificationVariables
  >(UPDATE_OWNERSHIP_VERIFICATION);

  const { data: verifiedByData } = useQuery<GetAdmin, GetAdminVariables>(
    GET_ADMIN,
    {
      variables: { adminId: ownership.verified_by_id },
      skip: !ownership.verified_by_id,
    }
  );

  const [documents, setDocuments] = useState<readonly string[]>(
    ownership?.document_ids ?? []
  );
  const [showFileField, setShowFileField] = useState(true);

  const handleSubmit = useCallback(
    (values: FormValues) => {
      const {
        first_name,
        last_name,
        is_owned_by_a_company,
        company_name,
        status,
        is_purchased_in_6_months,
        notes,
      } = values;

      updateOwnershipVerification({
        variables: {
          ownershipId: ownership.ownership_id,
          firstName: first_name,
          lastName: last_name,
          isCompany: is_owned_by_a_company,
          companyName: company_name,
          status: status
            ? account_api_OwnershipStatusEnum.OWNERSHIP_APPROVED
            : account_api_OwnershipStatusEnum.OWNERSHIP_DECLINED,
          isPurchased: is_purchased_in_6_months,
          documentIds: documents,
          notes,
        },
      });
      refetchProperty();
      hideModal();
    },
    [
      documents,
      hideModal,
      ownership.ownership_id,
      refetchProperty,
      updateOwnershipVerification,
    ]
  );

  const initialValues = useMemo(
    () => ({
      first_name: ownership.first_name,
      last_name: ownership.last_name,
      is_owned_by_a_company: ownership.is_company,
      company_name: ownership.company_name,
      is_purchased_in_6_months: ownership.is_purchased,
      notes: ownership.notes,
      status:
        ownership.status === account_api_OwnershipStatusEnum.OWNERSHIP_APPROVED,
    }),
    [ownership]
  );

  return (
    <Modal onCancel={hideModal} title="Ownership" withoutOverflow>
      <Form<FormValues>
        initialValues={initialValues}
        onSubmit={handleSubmit}
        render={({ handleSubmit, hasValidationErrors, values }) => (
          <form onSubmit={handleSubmit}>
            <UserFormRow className={styles.verifiedCheckbox}>
              <Field
                component={CheckboxToggle}
                name="status"
                type="checkbox"
                labelText="Verified"
                // initialValue={
                //   values.status ===
                //   account_api_OwnershipStatusEnum.OWNERSHIP_APPROVED
                //     ? true
                //     : false
                // }
              />
              {ownership.status ===
                account_api_OwnershipStatusEnum.OWNERSHIP_APPROVED && (
                <>
                  <div>
                    {`Verified by: ${verifiedByData?.admin?.Admin?.details?.full_name}`}
                  </div>
                  <div>
                    {format(
                      new Date(ownership.updated_at),
                      "dd MMM yyyy HH:mm"
                    )}
                  </div>
                </>
              )}
            </UserFormRow>
            <div className={styles.name}>
              <UserFormRow>
                <label>First name</label>
                <Field
                  component={TextInput}
                  name="first_name"
                  validate={required("Value is required")}
                />
                <Field component={ErrorMessage} name="first_name" />
              </UserFormRow>
              <UserFormRow>
                <label>Last name</label>
                <Field
                  component={TextInput}
                  name="last_name"
                  validate={required("Value is required")}
                />
                <Field component={ErrorMessage} name="last_name" />
              </UserFormRow>
            </div>
            <UserFormRow>
              <Field
                name="is_owned_by_a_company"
                type="checkbox"
                render={({ input }) => (
                  <div className={styles.checkbox}>
                    <input id="is_owned_by_a_company" {...input} />
                    <label htmlFor="is_owned_by_a_company">
                      Is owned by a company
                    </label>
                  </div>
                )}
              />
              <Field component={ErrorMessage} name="is_owned_by_a_company" />
            </UserFormRow>
            {values.is_owned_by_a_company && (
              <UserFormRow>
                <label>Company name</label>
                <Field
                  component={TextInput}
                  name="company_name"
                  validate={required("Value is required")}
                />
                <Field component={ErrorMessage} name="company_name" />
              </UserFormRow>
            )}
            <UserFormRow>
              <Field
                name="is_purchased_in_6_months"
                type="checkbox"
                render={({ input }) => (
                  <div className={styles.checkbox}>
                    <input id="is_purchased_in_6_months" {...input} />
                    <label htmlFor="is_purchased_in_6_months">
                      Property purchased in last 6 months
                    </label>
                  </div>
                )}
              />
              <Field component={ErrorMessage} name="is_purchased_in_6_months" />
            </UserFormRow>
            {values.is_purchased_in_6_months && (
              <UserFormRow>
                <label>Documents</label>
                {documents.map((document: string, key: number) => (
                  <div key={key} className={styles.filePreview}>
                    <a
                      href={`/media/${document}`}
                      target="_blank"
                      rel="noopener noreferrer"
                    >
                      <img
                        alt=""
                        className={styles.image}
                        src={`/media/${document}`}
                      />
                      <span>See document</span>
                    </a>
                    <button
                      type="button"
                      onClick={() =>
                        setDocuments(
                          documents.filter((doc) => doc !== document)
                        )
                      }
                    >
                      X
                    </button>
                  </div>
                ))}
                {showFileField && (
                  <Form
                    onSubmit={() => {}}
                    render={() => (
                      <Field
                        name="file"
                        render={({ input, meta }) => {
                          return (
                            <FileField
                              acceptedTypes={ACCEPTED_FILE_TYPES}
                              error={meta.touched && meta.error}
                              fileId={input.value}
                              handleChange={(document) => {
                                setDocuments([...documents, document]);

                                // I don't want to write a new file uploader. So I use existing one. In order to reload it I remount it. I know this is ugly.
                                setShowFileField(false);
                                setTimeout(() => setShowFileField(true));
                              }}
                            />
                          );
                        }}
                      />
                    )}
                  ></Form>
                )}
              </UserFormRow>
            )}
            <UserFormRow>
              <label>Notes</label>
              <Field component={TextareaField} name="notes" rows="5" />
              <Field component={ErrorMessage} name="notes" />
            </UserFormRow>

            {error && <div>{error.message}</div>}

            <div className={styles.buttons}>
              <SecondaryButton onClick={hideModal} type="button">
                Cancel
              </SecondaryButton>
              <PrimaryButton
                disabled={
                  hasValidationErrors ||
                  (values.is_purchased_in_6_months && documents.length === 0) ||
                  loading
                }
              >
                Save
              </PrimaryButton>
            </div>
          </form>
        )}
      />
    </Modal>
  );
};
