import React from "react";

import { withApollo } from "react-apollo";
import { Field, Form } from "react-final-form";

import FormAsideAdminUser from "../../../components/FormAsideAdminUser";
import {
  PropertyAsideZoopla,
  PropertyCardItemTitle,
  PropertyCardItemValue,
  PropertyContent,
  PropertyItemPublish,
  PropertyItemZoopla,
  PublicationBlocked,
} from "../../../ui/atoms/Property";
import { TitleError } from "../../../ui/atoms/TitleError";
import { TitleSuccess } from "../../../ui/atoms/TitleSuccess";
import { UserFormRow } from "../../../ui/atoms/UserForm";
import { CheckboxToggle } from "../../../ui/molecules/CheckboxToggle";
import CopyText from "../../../ui/molecules/CopyToClipboard";
import { ModalAlert, ModalConfirm } from "../../../ui/molecules/Modal";
import { PropertyZoopla } from "../graphql/queries/__generated__/PropertyZoopla";
import VerifyContext from "../PropertyEditContext";
import { usePublish } from "./use-publish";

type Props = {
  propertyId: string;
  publishedInMashroom: boolean | undefined;
  client?: any;
  version_id: string;
  zoopla: PropertyZoopla;
};

/** TODO save changes after BE will be ready */
const PublishZoopla = ({ version_id, zoopla }: Props) => {
  const {
    errorText,
    modalMode,
    onClearMessage,
    onSubmit,
    publishing,
    setModalMode,
    successText,
    unpublishing,
  } = usePublish(version_id, "zoopla");

  return (
    <>
      {errorText || successText ? (
        <ModalAlert
          onCancel={onClearMessage}
          title={
            errorText != null ? (
              <TitleError>Error</TitleError>
            ) : (
              <TitleSuccess>Message</TitleSuccess>
            )
          }
        >
          <b>{errorText || successText}</b>
        </ModalAlert>
      ) : null}
      <VerifyContext.Consumer>
        {({ publishedInZoopla, publishedInMashroom }) => (
          <Form
            initialValues={{
              publish_property_zoopla: publishedInZoopla,
            }}
            onSubmit={(fn) => fn}
            render={({ form }) => {
              return (
                <>
                  <ModalConfirm
                    disabled={publishing}
                    isOpen={modalMode === "confirm" || publishing}
                    loading={publishing}
                    onCancel={() => {
                      form.reset();
                      setModalMode(undefined);
                    }}
                    onConfirm={() => {
                      onSubmit(form, "publish");
                      setModalMode(undefined);
                    }}
                  >
                    <span>
                      Please confirm
                      <br />
                      <b>publish</b> property on Zoopla?
                    </span>
                  </ModalConfirm>
                  <ModalConfirm
                    disabled={unpublishing}
                    isOpen={modalMode === "delete" || unpublishing}
                    loading={unpublishing}
                    onCancel={() => {
                      form.reset();
                      setModalMode(undefined);
                    }}
                    onConfirm={() => {
                      onSubmit(form, "unpublish");
                      setModalMode(undefined);
                    }}
                  >
                    <span>
                      Are you sure you really want to
                      <br />
                      <b>unpublish</b> property from Zoopla?
                    </span>
                  </ModalConfirm>
                  <PropertyItemZoopla>
                    <PropertyContent>
                      <PropertyItemPublish>
                        <h3>Zoopla</h3>
                        {publishedInZoopla && (
                          <>
                            <PropertyCardItemTitle>
                              Property ID
                            </PropertyCardItemTitle>
                            <PropertyCardItemValue>
                              {zoopla.id}
                              <CopyText
                                text="Copy Zoopla ID"
                                textForCopy={zoopla.id}
                              />
                            </PropertyCardItemValue>
                            <br />
                            <PropertyCardItemTitle>
                              Property Link
                            </PropertyCardItemTitle>
                            <PropertyCardItemValue>
                              <a
                                className="link"
                                href={zoopla.url}
                                rel="noopener noreferrer"
                                target="_blank"
                                title={`Open in new tab ${zoopla.url}`}
                              >
                                {`${zoopla.url.substring(
                                  0,
                                  30
                                )} . . . ${zoopla.url.substr(-10)}`}
                              </a>
                              <CopyText
                                text="Copy link"
                                textForCopy={zoopla.url}
                              />
                            </PropertyCardItemValue>
                          </>
                        )}
                        {!publishedInZoopla && publishedInMashroom && (
                          <h4>
                            Something went wrong. Please, try again later.
                          </h4>
                        )}
                        {!publishedInZoopla && !publishedInMashroom && (
                          <h4>
                            Property will not be published on Zoopla until it is
                            published on Mashroom
                          </h4>
                        )}
                      </PropertyItemPublish>
                    </PropertyContent>
                    <PropertyAsideZoopla>
                      <UserFormRow>
                        {!publishedInZoopla && !publishedInMashroom ? (
                          <PublicationBlocked>
                            <Field
                              component={CheckboxToggle}
                              disabled={
                                publishedInZoopla ||
                                (!publishedInZoopla && !publishedInMashroom)
                              }
                              labelText="Published"
                              name="publish_property_zoopla"
                              onChange={(event) => {
                                form.change(
                                  "publish_property_zoopla",
                                  event.target.checked
                                );
                                setModalMode("confirm");
                              }}
                              type="checkbox"
                            />
                          </PublicationBlocked>
                        ) : (
                          <Field
                            component={CheckboxToggle}
                            disabled={
                              !publishedInZoopla && !publishedInMashroom
                            }
                            labelText="Published"
                            name="publish_property_zoopla"
                            onChange={(event) => {
                              form.change(
                                "publish_property_zoopla",
                                event.target.checked
                              );
                              setModalMode(
                                event.target.checked ? "confirm" : "delete"
                              );
                            }}
                            type="checkbox"
                          />
                        )}
                      </UserFormRow>
                      {publishedInZoopla && (
                        <FormAsideAdminUser
                          date={zoopla.created_at as string}
                          id={zoopla.published_by as string}
                        />
                      )}
                    </PropertyAsideZoopla>
                  </PropertyItemZoopla>
                </>
              );
            }}
          />
        )}
      </VerifyContext.Consumer>
    </>
  );
};

export default withApollo<Props>(PublishZoopla);
