import React, { FC, useCallback, useMemo } from "react";

import styled from "@emotion/styled";
import { useMutation } from "react-apollo";
import { Field } from "react-final-form";

import { property_api_ReorderPhotosStageEnum as ReorderPhotosStageEnum } from "../../../globalTypes";
import { PropertyBody, PropertyContent } from "../../../ui/atoms/Property";
import Spinner from "../../../ui/atoms/Spinner";
import SystemError from "../../../ui/atoms/SystemError";
import { Table, Tbody, Thead, ThInner, Tr } from "../../../ui/atoms/Table";
import theme from "../../../ui/theme";
import { formatAllowUndefined } from "../../../util/formatters/misc";
import DraggableFile from "../DraggableFile";
import {
  ReorderPhotos,
  ReorderPhotosVariables,
} from "../graphql/mutations/__generated__/ReorderPhotos";
import REORDER_PHOTOS from "../graphql/mutations/ReorderPhotos.graphql";
import {
  GetProperty_admin_Property,
  GetProperty_admin_Property_description_photos,
} from "../graphql/queries/__generated__/GetProperty";
import GET_PROPERTY from "../graphql/queries/GetProperty.graphql";
import PhotosRow from "../PhotosRow";
import VerifyContext from "../PropertyEditContext";
import PropertyVerification from "../PropertyVerification";

type Props = {
  property: GetProperty_admin_Property;
  values: GetProperty_admin_Property;
};

const Photos: FC<Props> = ({ values, property }) => {
  const propertyId = property.id;

  const [reorderPhotosMutation, { loading, error }] = useMutation<
    ReorderPhotos,
    ReorderPhotosVariables
  >(REORDER_PHOTOS);

  const videos = property.description.video_embeds ?? [];

  // Reordering array of images: "move" element from dragIndex to hoverIndex
  const handleMoveImage = useCallback(
    (
      photos: GetProperty_admin_Property_description_photos[] | null,
      onChange: (event: any) => void
    ) => (dragIndex: number, hoverIndex: number) => {
      if (photos == null) {
        return;
      }

      let newPhotos: GetProperty_admin_Property_description_photos[];

      if (hoverIndex < dragIndex) {
        newPhotos = [
          ...photos.slice(0, hoverIndex),
          photos[dragIndex],
          ...photos.slice(hoverIndex, dragIndex),
          ...photos.slice(dragIndex + 1),
        ];
      } else {
        newPhotos = [
          ...photos.slice(0, dragIndex),
          ...photos.slice(dragIndex + 1, hoverIndex + 1),
          photos[dragIndex],
          ...photos.slice(hoverIndex + 1),
        ];
      }

      onChange(newPhotos);
    },
    []
  );

  const handlePhotosEditClick = useCallback(async () => {
    const res = await reorderPhotosMutation({
      variables: {
        property: propertyId,
        stage: ReorderPhotosStageEnum.REORDER_PHOTOS_EDIT_STATUS,
      },
      awaitRefetchQueries: true,
      refetchQueries: [
        {
          query: GET_PROPERTY,
          variables: {
            id: propertyId,
          },
        },
      ],
    });
    if (res.data?.admin?.reorderPhotos?.error) {
      // eslint-disable-next-line no-console
      console.log(res.data);
    }
  }, [propertyId, reorderPhotosMutation]);

  const photos = useMemo(() => values.description?.photos ?? [], [
    values.description,
  ]);

  const handlePhotoOrderUpdate = useCallback(() => {
    if (photos.length > 0) {
      return reorderPhotosMutation({
        variables: {
          property: propertyId,
          stage: ReorderPhotosStageEnum.REORDER_PHOTOS_UPDATE,
          photos: photos?.map((f) => f.id),
        },
      });
    } else {
      return null;
    }
  }, [photos, propertyId, reorderPhotosMutation]);

  return (
    <VerifyContext.Consumer>
      {({ photosVerified }) => (
        <PropertyBody>
          <PropertyContent>
            {loading && <Spinner />}
            {error && <SystemError error={error} />}
            {photos.length === 0 ? (
              <h3>No photos</h3>
            ) : (
              <>
                <h3>
                  Photos{" "}
                  {!photosVerified && (
                    <Text>
                      You can click &amp; drag images to change the order in
                      which they’ll appear on the listing
                    </Text>
                  )}
                </h3>
                <Table style={{ marginBottom: "10px" }}>
                  <Thead>
                    <Tr>
                      <ThInner>File</ThInner>
                      <ThInner>Type</ThInner>
                      <ThInner>Description</ThInner>
                      <ThInner>Date</ThInner>
                      <ThInner></ThInner>
                    </Tr>
                  </Thead>
                  <Tbody>
                    <>
                      <Field<
                        GetProperty_admin_Property_description_photos[] | null
                      >
                        allowNull
                        format={formatAllowUndefined}
                        name="description.photos"
                        render={({ input }) =>
                          input.value?.map((item, index: number) => {
                            const photo = photos?.find(
                              (photo) => photo.id === item.id
                            );
                            return photo != null ? (
                              <DraggableFile
                                key={item.id}
                                id={item.id}
                                index={index}
                                onMove={handleMoveImage(
                                  input.value,
                                  input.onChange
                                )}
                              >
                                <PhotosRow
                                  key={photo.id}
                                  item={photo}
                                  propertyId={propertyId}
                                  type="Photo"
                                  withoutParent
                                />
                              </DraggableFile>
                            ) : (
                              <>
                                Image has no corresponding document record,
                                contact your administator (file id is{" "}
                                {`"${item.id}"`} file name is {`"${item.name}$`}
                                )
                              </>
                            );
                          })
                        }
                      />
                    </>
                  </Tbody>
                </Table>
                <DownloadLink
                  className="link"
                  href={`/media/bundle?property=${propertyId}`}
                  rel="noopener noreferrer"
                  target="_blank"
                >
                  Download all photos as a ZIP-archive
                </DownloadLink>
              </>
            )}

            {videos.length === 0 ? (
              <h3>No videos</h3>
            ) : (
              <>
                <h3>Video</h3>
                <DownloadLink
                  href={videos[0]}
                  rel="noopener noreferrer"
                  target="_blank"
                >
                  {videos[0]}
                </DownloadLink>
              </>
            )}
          </PropertyContent>

          <PropertyVerification
            adminVerify={property.admin_verify}
            isVerified={!!photosVerified}
            onPhotosEdit={handlePhotosEditClick}
            onPhotosOrderUpdate={handlePhotoOrderUpdate}
            property={property}
            propertyStatus={property.status}
            verifiedLabel="Verified"
            verifiedName="admin_verify.photos.verified"
            verifyKey="photos"
          />
        </PropertyBody>
      )}
    </VerifyContext.Consumer>
  );
};

const Text = styled.span`
  font-weight: 400;
  font-size: 15px;
`;

const DownloadLink = styled.a`
  display: inline-block;
  margin-bottom: 20px;
  color: ${theme.colors.main.mainBlue};
`;

export default Photos;
