import React, { useState } from "react";

import styled from "@emotion/styled";
import { Query } from "react-apollo";
import { Field, Form } from "react-final-form";

import {
  common_api_generic_query_QueryFilterOperatorEnum,
  common_api_generic_query_QueryParamsFilterInput,
} from "../../globalTypes";
import { PrimaryButton, SecondaryButton } from "../../ui/atoms/Button";
import {
  FilterSort as FilterSortOrigin,
  FilterSortItem as FilterSortItemOrigin,
} from "../../ui/atoms/FilterSort";
import { TextInput } from "../../ui/atoms/Input";
import { SearchFieldWide } from "../../ui/atoms/SearchField";
import SystemError from "../../ui/atoms/SystemError";
import { Table as TableOrigin, Tbody } from "../../ui/atoms/Table";
import TableOverlay from "../../ui/atoms/TableOverlay";
import TableWrapper from "../../ui/atoms/TableWrapper";
import FilterFieldSort from "../../ui/molecules/FilterFieldSort";
import Pagebar from "../../ui/molecules/Pagebar";
import Pagination from "../../ui/molecules/Pagination";
import Sortable from "../../ui/molecules/Sortable";
import getSortOrder from "../../util/ger_sort_order";
import { Option, PlaceOptions, StatusOptions } from "./constants";
import {
  GetAllMaintenances,
  GetAllMaintenances_admin_allMaintenances_data,
  GetAllMaintenancesVariables,
} from "./graphql/queries/__generated__/GetAllMaintenances";
import ALL_MAINTENANCE from "./graphql/queries/GetAllMaintenances.graphql";
import ListHead from "./ListHead";
import ListItem from "./ListItem";

type MainTenanceFilterState = common_api_generic_query_QueryParamsFilterInput[];
type FormProps = {
  search: string;
  category: Option;
  subcategory: Option;
  status: Option;
  place: Option;
  specialist_type: Option;
  created_at: string;
};

const List = () => {
  const [filter, setFilter] = useState<MainTenanceFilterState>([]);

  const handleFilterSubmit = (values) => {
    const newFilter: MainTenanceFilterState = [];
    const operator = common_api_generic_query_QueryFilterOperatorEnum.EQUALS;

    if (values.search) {
      newFilter.push({
        fields: ["property", "id"],
        value: [values.search],
        operator,
      });
    }

    if (values.status) {
      newFilter.push({
        operator,
        fields: ["status"],
        value: [values.status.value],
      });
    }

    if (values.place) {
      newFilter.push({
        operator,
        fields: ["place"],
        value: [values.place.value],
      });
    }

    if (values.created_at) {
      newFilter.push({
        operator,
        fields: ["created_at"],
        value: [values.created_at],
      });
    }

    setFilter(newFilter);
  };

  return (
    <div>
      <h2>Maintenance Requests</h2>

      <Form<FormProps>
        onSubmit={handleFilterSubmit}
        render={({ handleSubmit, values, form, pristine }) => (
          <form onSubmit={handleSubmit}>
            <SearchFieldWide>
              <Field
                component={TextInput}
                name="search"
                placeholder="Maintenance ID or Property ID"
              />
              <PrimaryButton
                disabled={!values.search || values.search.length === 0}
                onClick={() => handleFilterSubmit(values)}
                type="submit"
              >
                Search
              </PrimaryButton>
              {pristine ? null : (
                <>
                  &nbsp;&nbsp;
                  <SecondaryButton
                    onClick={() => {
                      setFilter([]);
                      form.reset();
                    }}
                    type="reset"
                  >
                    Clear
                  </SecondaryButton>
                </>
              )}
            </SearchFieldWide>
            <FilterSort>
              <FilterFieldSort
                name="status"
                options={StatusOptions}
                placeholder="Status"
              />
              <FilterFieldSort
                name="place"
                options={PlaceOptions}
                placeholder="Place"
              />
              <FilterSortItem>
                <Field
                  className="input-sort-date"
                  component={TextInput}
                  name="created_at"
                  type="date"
                />
              </FilterSortItem>

              <FilterSortItem>
                <PrimaryButton
                  disabled={pristine}
                  onClick={() => handleFilterSubmit(values)}
                >
                  Filter
                </PrimaryButton>
              </FilterSortItem>

              <FilterSortItem />
            </FilterSort>
          </form>
        )}
      />

      <Sortable field={["created_at"]} order={-1}>
        {({ field, order, setSort }) => (
          <Pagination filter={filter} perPage={25}>
            {({ page, setPage, perPage }) => {
              return (
                <div>
                  <Query<GetAllMaintenances, GetAllMaintenancesVariables>
                    query={ALL_MAINTENANCE}
                    variables={{
                      filter,
                      pagination: { page, perPage },
                      sort: field.map((v) => ({ field: v, order })),
                    }}
                  >
                    {({ data, loading, error }) => {
                      if (error) {
                        return <SystemError error={error} />;
                      }

                      const getOrder = (fieldName) =>
                        getSortOrder({ field, fieldName, order });
                      const onSetPage = (pageNumber: number) => {
                        setPage(pageNumber);
                      };
                      const totalRows =
                        data?.admin?.allMaintenances?.total || 20;

                      return (
                        <TableWrapper>
                          {loading && <TableOverlay />}

                          <Pagebar
                            changePage={onSetPage}
                            curPage={page}
                            showNPages={3}
                            total={Math.ceil(totalRows / perPage)}
                          />

                          <Table>
                            <ListHead
                              field={field}
                              getOrder={getOrder}
                              order={order}
                              setSort={setSort}
                            />
                            <Tbody>
                              {(data?.admin?.allMaintenances?.data ?? [])
                                .filter((i) => !!i.id)
                                .map(
                                  (
                                    item: GetAllMaintenances_admin_allMaintenances_data
                                  ) => {
                                    if (item.id) {
                                      return (
                                        <ListItem key={item.id} data={item} />
                                      );
                                    }
                                    return null;
                                  }
                                )}
                            </Tbody>
                          </Table>
                          <Pagebar
                            changePage={onSetPage}
                            curPage={page}
                            showNPages={3}
                            total={Math.ceil(totalRows / perPage)}
                          />
                        </TableWrapper>
                      );
                    }}
                  </Query>
                </div>
              );
            }}
          </Pagination>
        )}
      </Sortable>
    </div>
  );
};

const FilterSort = styled(FilterSortOrigin)`
  flex-wrap: wrap;
  padding-bottom: 0;
`;

const FilterSortItem = styled(FilterSortItemOrigin)`
  margin-bottom: 10px;

  input.input-sort-date {
    width: 200px;
  }
`;

const Table = styled(TableOrigin)`
  table-layout: fixed;
`;

export default List;
