import React, { useEffect, useState } from "react";

import { useQuery } from "react-apollo";
import { Field, Form } from "react-final-form";

import { common_api_generic_query_QueryFilterOperatorEnum } from "../../../globalTypes";
import { PrimaryButton, SecondaryButton } from "../../../ui/atoms/Button";
import { TextInput } from "../../../ui/atoms/Input";
import { SearchField } from "../../../ui/atoms/SearchField";
import { Table, Tbody, Thead } from "../../../ui/atoms/Table";
import TableOverlay from "../../../ui/atoms/TableOverlay";
import TableWrapper from "../../../ui/atoms/TableWrapper";
import { ModalWide } from "../../../ui/molecules/Modal";
import Pagebar from "../../../ui/molecules/Pagebar";
import {
  AllLeads,
  AllLeadsVariables,
} from "../graphql/queries/__generated__/AllLeads";
import GET_LEAD_LIST from "../graphql/queries/AllLeads.graphql";
import AddLead from "./components/AddLead";
import TableHeader from "./components/TableHeder";
import LeadRow from "./components/TableItem";

import css from "./index.module.css";

const perPage = 15;

const LeadList: React.FC = () => {
  const [searchQuery, setSearchQuery] = useState("");
  const [isModalOpened, setIsModalOpened] = useState(false);
  const [page, setPage] = useState(1);
  const [currentSort, setCurrentSort] = useState({
    field: "name",
    order: 1,
  });

  const { data, loading, refetch, error } = useQuery<
    AllLeads,
    AllLeadsVariables
  >(GET_LEAD_LIST, {
    variables: {
      pagination: { page, perPage },
      sort: [currentSort],
      filter: [
        {
          operator: common_api_generic_query_QueryFilterOperatorEnum.CONTAINS,
          fields: ["name"],
          value: [""],
        },
      ],
    },
  });

  const leadList = data?.admin?.mortgageAllLeads?.leads;

  const totalPages = Math.ceil(
    (data?.admin?.mortgageAllLeads?.total ?? 0) / perPage
  );

  useEffect(() => {
    refetch({
      pagination: { page, perPage },
      sort: [currentSort],
      filter: [
        {
          operator: common_api_generic_query_QueryFilterOperatorEnum.CONTAINS,
          fields: ["name", "assigned_to", "email", "stage"],
          value: [searchQuery],
        },
      ],
    });
  }, [page, refetch, searchQuery, currentSort]);

  const handleFilterSubmit = (values) => {
    setSearchQuery(values.search);
  };
  const handleAddLeadClick = () => {
    setIsModalOpened(true);
  };

  const handleModalClose = () => {
    setIsModalOpened(false);
  };

  const handlePageChange = (newPage: number) => {
    setPage(newPage);
  };
  return (
    <>
      <div className={css.container}>
        <h2>Mortgages Leads</h2>
        <div className={css.searchForm}>
          <Form
            initialValues={{
              search: searchQuery,
            }}
            onSubmit={handleFilterSubmit}
            render={({ handleSubmit, values, valid, submitting, form }) => (
              <form onSubmit={handleSubmit}>
                <SearchField>
                  <Field
                    component={TextInput}
                    name="search"
                    placeholder="Enter full name, email or assigned to search"
                  />
                  <PrimaryButton
                    disabled={
                      (values.postcode && !valid) ||
                      !values.search ||
                      submitting
                    }
                    onClick={() => handleFilterSubmit(values)}
                    type="submit"
                  >
                    Search
                  </PrimaryButton>
                  <SecondaryButton
                    className={css.resetButton}
                    onClick={() => setSearchQuery("")}
                    type="reset"
                  >
                    Clear
                  </SecondaryButton>
                </SearchField>
              </form>
            )}
          />
          <PrimaryButton
            className={css.addButton}
            onClick={handleAddLeadClick}
            type={"button"}
          >
            Add new lead
          </PrimaryButton>
        </div>
        <TableWrapper>
          {loading && <TableOverlay />}

          <Table>
            <Thead>
              <TableHeader
                onSortChange={(field: string, order: number) => {
                  setCurrentSort({
                    field,
                    order,
                  });
                  setPage(1);
                }}
              />
            </Thead>
            <Tbody>
              {error && <span>{error.message}</span>}
              {leadList?.map((lead) => (
                <LeadRow key={lead.id} lead={lead} />
              ))}
            </Tbody>
          </Table>
          <Pagebar
            changePage={handlePageChange}
            curPage={page}
            showNPages={3}
            total={totalPages}
          />
        </TableWrapper>
      </div>
      {isModalOpened && (
        <ModalWide onCancel={handleModalClose} title={"Add Lead"} width={800}>
          <AddLead onClose={handleModalClose} />
        </ModalWide>
      )}
    </>
  );
};

export default LeadList;
