import React, { useCallback, useState } from "react";

import styled from "@emotion/styled";
import { Query } from "react-apollo";
import { Field, Form } from "react-final-form";
import { oc } from "ts-optchain";

import {
  common_api_generic_query_QueryFilterOperatorEnum as FilterOperatorEnum,
  common_api_generic_query_QueryParamsFilterInput,
} from "../../globalTypes";
import { PrimaryButton, SecondaryButton } from "../../ui/atoms/Button";
import { FilterSort, FilterSortItem } from "../../ui/atoms/FilterSort";
import { TextInput } from "../../ui/atoms/Input";
import { SearchField } from "../../ui/atoms/SearchField";
import Spinner from "../../ui/atoms/Spinner";
import SystemError from "../../ui/atoms/SystemError";
import { Table as TableOrigin, Tbody, Thead } from "../../ui/atoms/Table";
import TableOverlay from "../../ui/atoms/TableOverlay";
import TableWrapper from "../../ui/atoms/TableWrapper";
import Dropdown from "../../ui/molecules/Dropdown";
import Pagebar from "../../ui/molecules/Pagebar";
import Pagination from "../../ui/molecules/Pagination";
import Sortable from "../../ui/molecules/Sortable";
import { OFFER_STAGE, STAGE_FILTER_OPTIONS } from "./constants";
import {
  OffersList,
  OffersListVariables,
} from "./graphql/queries/__generated__/OffersList";
import ALL_OFFERS from "./graphql/queries/GetListOfOffers.graphql";
import ListHead from "./ListHead";
import ListItem from "./ListItem";

const searchFieldNames = [
  // "id",
  // "property.id",
  "people.landlord.phone_primary",
  "people.landlord.email_primary",
  "people.landlord.first_name",
  "people.landlord.last_name",
  "property_ivr",
];

export type ListProps = {};
type OfferFilterState = common_api_generic_query_QueryParamsFilterInput[];
interface FilterFormValues {
  search: string;
  offerId: string;
  postcode: string;
  stage: { label: string; value: string };
}

const List: React.FC<ListProps> = () => {
  const [offerFilter, setOfferFilter] = useState<OfferFilterState>([]);

  const handleSearchSubmit = useCallback((values: FilterFormValues) => {
    setOfferFilter((prevOfferFilter) => [
      {
        fields: searchFieldNames,
        value: [values.search],
        operator: FilterOperatorEnum.CONTAINS_ANY,
      },
      ...prevOfferFilter.filter((item) => item.fields !== searchFieldNames),
    ]);
  }, []);

  const handleFilterSubmit = useCallback((values: FilterFormValues) => {
    const newFilter: OfferFilterState = [];
    if (values.offerId) {
      newFilter.push({
        operator: FilterOperatorEnum.EQUALS,
        fields: ["id"],
        value: [values.offerId],
      });
    }

    if (values.postcode) {
      newFilter.push({
        operator: FilterOperatorEnum.EQUALS,
        fields: ["property.location.postcode"],
        value: [values.postcode],
      });
    }

    if (values.stage) {
      newFilter.push({
        operator: FilterOperatorEnum.EQUALS,
        fields: ["state"],
        value: [values.stage.value],
      });
    }

    setOfferFilter((prevOfferFilter) =>
      prevOfferFilter
        .filter((item) => item.fields === searchFieldNames)
        .concat(newFilter)
    );
  }, []);

  return (
    <>
      <h2>View Offers</h2>
      <Form
        onSubmit={handleSearchSubmit}
        render={({ form, handleSubmit, values, submitting }) => {
          const isTextModified = offerFilter.some(
            (item) =>
              item.fields === searchFieldNames &&
              item.value &&
              item.value[0] !== values.search
          );
          const isTextExist = offerFilter.some(
            (item) => item.fields === searchFieldNames
          );
          const handleSearchClear = () => {
            form.reset();
            setOfferFilter((prevOfferFilter) =>
              prevOfferFilter.filter((item) => item.fields !== searchFieldNames)
            );
          };
          return (
            <form onSubmit={handleSubmit}>
              <SearchField>
                <Field
                  component={TextInput}
                  name="search"
                  placeholder="Enter landlord name, phone, email or ivr"
                />
                {isTextModified || !isTextExist ? (
                  <PrimaryButton
                    disabled={!values.search || submitting}
                    type="submit"
                  >
                    Search
                  </PrimaryButton>
                ) : (
                  <SecondaryButton onClick={handleSearchClear}>
                    Clear
                  </SecondaryButton>
                )}
              </SearchField>
            </form>
          );
        }}
      />

      <Form
        onSubmit={handleFilterSubmit}
        render={({ handleSubmit, form }) => {
          const handleReset = () => {
            form.reset();
            setOfferFilter((prevOfferFilter) => {
              return prevOfferFilter.filter(
                (item) => item.fields === searchFieldNames
              );
            });
          };
          return (
            <form onSubmit={handleSubmit}>
              <FilterSort>
                <FilterSortItem>
                  <label htmlFor="offerId">Offer ID:</label>
                  <Field
                    className="input-320"
                    component={TextInput}
                    name="offerId"
                  />
                </FilterSortItem>
                <FilterSortItem>
                  <label htmlFor="postcode">Postcode:</label>
                  <Field component={TextInput} name="postcode" />
                </FilterSortItem>
                <FilterSortItem>
                  <label htmlFor="stage">Current stage:</label>
                  <Field
                    name="stage"
                    render={({ input }) => (
                      <Dropdown
                        input={input}
                        options={STAGE_FILTER_OPTIONS}
                        placeholder="Choose stage"
                      />
                    )}
                  />
                </FilterSortItem>
                <FilterSortItem>
                  {offerFilter.length > 0 && (
                    <FilterSortItem>
                      <SecondaryButton onClick={handleReset} type="button">
                        Clear
                      </SecondaryButton>
                    </FilterSortItem>
                  )}
                  <FilterSortItem>
                    <PrimaryButton>Filter</PrimaryButton>
                  </FilterSortItem>
                </FilterSortItem>
              </FilterSort>
            </form>
          );
        }}
      />

      <Sortable field={["created_at_value", "updated_at_value"]}>
        {({ field, order, setSort }) => (
          <Pagination filter={offerFilter}>
            {({ page, setPage, perPage }) => {
              return (
                <div>
                  <Query<OffersList, OffersListVariables>
                    query={ALL_OFFERS}
                    variables={{
                      pagination: { page, perPage },
                      filter: offerFilter,
                    }}
                  >
                    {({ data, loading, error }) => {
                      if (error) {
                        return <SystemError error={error} />;
                      } else if (loading) {
                        return <Spinner />;
                      }

                      const getOrder = (fieldName) => {
                        return field.includes(fieldName)
                          ? order === 1
                            ? -1
                            : 1
                          : 1;
                      };

                      return (
                        <TableWrapper>
                          <Pagebar
                            changePage={setPage}
                            curPage={page}
                            showNPages={3}
                            total={Math.ceil(
                              oc(data).admin.allOffers.total(1) / perPage
                            )}
                          />

                          {loading && <TableOverlay />}

                          <Table>
                            <Thead>
                              <ListHead
                                field={field}
                                getOrder={getOrder}
                                order={order}
                                setSort={setSort}
                              />
                            </Thead>
                            <Tbody>
                              {oc(data)
                                .admin.allOffers.data([])
                                .map((offer) => {
                                  const {
                                    id,
                                    state,
                                    property,
                                    landlord,
                                    tenants,
                                    created_at_value,
                                    updated_at_value,
                                  } = offer;

                                  if (
                                    property != null &&
                                    tenants &&
                                    tenants[0] != null
                                  ) {
                                    return (
                                      <ListItem
                                        key={id}
                                        changed={updated_at_value}
                                        initiated={created_at_value}
                                        landlord={landlord}
                                        offerId={id}
                                        property={property}
                                        stage={OFFER_STAGE[state]}
                                        tenant={tenants[0]}
                                      />
                                    );
                                  }

                                  return null;
                                })}
                            </Tbody>
                          </Table>

                          <Pagebar
                            changePage={setPage}
                            curPage={page}
                            showNPages={3}
                            total={Math.ceil(
                              oc(data).admin.allOffers.total(1) / perPage
                            )}
                          />
                        </TableWrapper>
                      );
                    }}
                  </Query>
                </div>
              );
            }}
          </Pagination>
        )}
      </Sortable>
    </>
  );
};

export default List;

const Table = styled(TableOrigin)`
  table-layout: fixed;
`;
