import React from "react";

import styled from "@emotion/styled";
import format from "date-fns/format";
import { Link } from "react-router-dom";

import { offer_api_DepositTypeEnum as DepositTypeEnum } from "../../globalTypes";
import EmailLink from "../../ui/atoms/EmailLink";
import { NoWrap } from "../../ui/atoms/Table";
import Collapse from "../../ui/molecules/Collapse";
import { ModalWide } from "../../ui/molecules/Modal";
import theme from "../../ui/theme";
import strip_typenames from "../../util/strip_typenames";
import { EMPLOYMENT_STATUS } from "./constants";
import {
  OffersList_admin_allOffers_data as Offer,
  OffersList_admin_allOffers_data_property as Property,
} from "./graphql/queries/__generated__/OffersList";
import { calcTenantsCheckPrice } from "./utils";

import css from "@emotion/css";

type Props = {
  offer: Offer;
  property: Property | null;
  onCancel: () => void;
};

const OfferDetailsModal = ({ offer, property, onCancel }: Props) => {
  const propertyId = property?.id;
  const location = property?.location;
  const landlord = property?.people?.landlord;
  const manager = property?.people?.manager;

  const { accepted_at_value, created_at_value, conditions, tenants } = offer;
  const offerProperty = offer.property;

  if (!offerProperty) {
    return null;
  }

  const { listing, rental } = offerProperty;

  return (
    <ModalWide onCancel={onCancel} title={`Offer ID: ${offer.id}`}>
      <>
        <Collapse opened>
          {({ isOpened, toggle }) => (
            <Group>
              <GroupTitle>
                <GroupTitleIcon onClick={toggle}>
                  <TitleIcon state={isOpened} />
                </GroupTitleIcon>
                <GroupTitleText>
                  <span>Property details</span>
                  <br />
                  {listing?.started_listing_at != null &&
                  listing.started_listing_at !== "" ? (
                    <>
                      <b>Listing date</b>
                      <span>
                        {format(
                          new Date(listing.started_listing_at),
                          "dd MMM yyyy HH:mm:ss"
                        )}
                      </span>
                    </>
                  ) : (
                    <>
                      <b>Created date</b>
                      <span>
                        {format(
                          new Date(property?.created_at ?? ""),
                          "dd MMM yyyy HH:mm:ss"
                        )}
                      </span>
                    </>
                  )}

                  <b>Listed price</b>
                  <span>
                    {rental?.rental_price ? `£  ${conditions.price}` : "—"}
                  </span>
                </GroupTitleText>
              </GroupTitle>

              {isOpened && (
                <GroupBody>
                  <GroupRow>
                    <GroupColumn size="l">
                      <Label>Property</Label>
                      <Value>
                        {Object.values(strip_typenames(location))
                          .filter(Boolean)
                          .join(", ")}
                        <br />
                        <Link
                          className="link"
                          to={`/listing/${propertyId}/edit`}
                        >
                          {propertyId}
                        </Link>
                      </Value>
                    </GroupColumn>
                    <GroupColumn size="s">
                      <Label>Landlord</Label>
                      <Value>
                        <Link
                          className="link"
                          to={`/users/${landlord!.id}/view`}
                        >
                          {landlord!.details?.first_name}
                          &nbsp;
                          {landlord!.details?.last_name}
                        </Link>
                      </Value>
                    </GroupColumn>
                    <GroupColumn size="s">
                      <Label>Manager</Label>
                      <Value>
                        {manager!.id ? (
                          <Link
                            className="link"
                            to={`/users/${manager!.id}/view`}
                          >
                            {manager!.details?.first_name}
                            &nbsp;
                            {manager!.details?.last_name}
                          </Link>
                        ) : (
                          "—"
                        )}
                      </Value>
                    </GroupColumn>
                  </GroupRow>
                  <GroupRow>
                    <GroupColumn size="l">
                      <Label>Bills included</Label>
                      <Value>{rental?.bills_included ? "Yes" : "No"}</Value>
                    </GroupColumn>
                    <GroupColumn size="s">
                      <Label>DSS</Label>
                      <Value>{rental?.dss_available ? "Yes" : "No"}</Value>
                    </GroupColumn>
                    <GroupColumn size="s">
                      <Label>DRP</Label>
                      <Value>
                        {rental?.tenancy_deposit_insurance ? "Yes" : "No"}
                      </Value>
                    </GroupColumn>
                  </GroupRow>
                </GroupBody>
              )}
            </Group>
          )}
        </Collapse>

        <Collapse opened={false}>
          {({ isOpened, toggle }) => (
            <Group>
              <GroupTitle>
                <GroupTitleIcon onClick={toggle}>
                  <TitleIcon state={isOpened} />
                </GroupTitleIcon>
                <GroupTitleText>
                  <span>Offer details</span>
                  <br />
                  <b>Offer date</b>
                  <span>
                    {format(new Date(created_at_value), "dd MMM yyyy HH:mm:ss")}
                  </span>
                  <b>Offered price</b>
                  <span>
                    {conditions.price ? `£  ${conditions.price}` : "—"}
                  </span>
                </GroupTitleText>
              </GroupTitle>

              {isOpened && (
                <GroupBody>
                  <GroupRow>
                    <GroupColumn size="m">
                      <Label>Move in date</Label>
                      <Value>
                        {conditions.movein_date !== ""
                          ? format(
                              new Date(conditions.movein_date),
                              "dd MMM yyyy"
                            )
                          : "—"}
                      </Value>
                    </GroupColumn>
                    <GroupColumn size="m">
                      <Label>Rent term</Label>
                      <Value>
                        {conditions.term_months != null
                          ? `${conditions.term_months} month(s)`
                          : "—"}
                      </Value>
                    </GroupColumn>
                    <GroupColumn size="m">
                      <Label>Check costs</Label>
                      <Value>£ {calcTenantsCheckPrice(tenants)}</Value>
                    </GroupColumn>
                  </GroupRow>
                  <GroupRow>
                    <GroupColumn size="m">
                      <Label>Under 18</Label>
                      <Value>
                        {tenants && tenants.length > 0 && tenants[0].kids_no
                          ? tenants[0].kids_no
                          : "No"}
                      </Value>
                    </GroupColumn>
                    <GroupColumn size="m">
                      <Label>Pets</Label>
                      <Value>{conditions.pets ? "Yes" : "No"}</Value>
                    </GroupColumn>
                    {conditions.pets && conditions.pets_description !== "" && (
                      <GroupColumn size="m">
                        <Label>Pets details</Label>
                        <Value>{conditions.pets_description}</Value>
                      </GroupColumn>
                    )}
                    <GroupColumn size="m">
                      <Label>Break clause</Label>
                      <Value>{conditions.break_clause ?? "No"}</Value>
                    </GroupColumn>
                  </GroupRow>
                </GroupBody>
              )}
            </Group>
          )}
        </Collapse>

        {(tenants ? tenants : []).map((t, idx) => (
          <div key={`${t.first_name}${t.last_name}${t.user}`}>
            <Collapse opened={false}>
              {({ isOpened, toggle }) => (
                <Group>
                  <GroupTitle>
                    <GroupTitleIcon onClick={toggle}>
                      <TitleIcon state={isOpened} />
                    </GroupTitleIcon>
                    <GroupTitleText>
                      <span>
                        {idx ? `Tenant # ${idx + 1}` : "Leading tenant"}
                      </span>
                      <b>Needs a guarantor</b>
                      <span>
                        {t.guarantor_required || t.guarantor != null
                          ? "Yes"
                          : "No"}
                      </span>
                    </GroupTitleText>
                  </GroupTitle>

                  {isOpened && (
                    <GroupBody>
                      <GroupRow>
                        <GroupColumn size="m">
                          <Label>Full name</Label>
                          <Value>
                            {t.first_name}
                            &nbsp;
                            {t.last_name}
                          </Value>
                        </GroupColumn>
                        <GroupColumn size="m">
                          <Label>Email address</Label>
                          <Value>
                            {t.email_primary && (
                              <NoWrap title={t.email_primary}>
                                <EmailLink href={`mailto:${t.email_primary}`}>
                                  {t.email_primary}
                                </EmailLink>
                              </NoWrap>
                            )}
                          </Value>
                        </GroupColumn>
                        <GroupColumn size="m">
                          <Label>Age</Label>
                          <Value>{t.age}</Value>
                        </GroupColumn>
                      </GroupRow>
                      <GroupRow>
                        <GroupColumn size="m">
                          <Label>Income</Label>
                          <Value>
                            {t.annual_income
                              ? `£ ${t.annual_income} / year`
                              : "0"}
                          </Value>
                        </GroupColumn>
                        <GroupColumn size="m">
                          <Label>Employment</Label>
                          <Value>
                            {EMPLOYMENT_STATUS[t.employment_status]}
                          </Value>
                        </GroupColumn>
                        <GroupColumn size="m">
                          <Label>Paying rent?</Label>
                          <Value>{t.paying ? "Yes" : "No"}</Value>
                        </GroupColumn>
                      </GroupRow>
                    </GroupBody>
                  )}
                </Group>
              )}
            </Collapse>

            {t.guarantor != null && (
              <Collapse opened={false}>
                {({ isOpened, toggle }) => (
                  <Group>
                    <GroupTitle>
                      <GroupTitleIcon onClick={toggle}>
                        <TitleIcon state={isOpened} />
                      </GroupTitleIcon>
                      <GroupTitleText>
                        <span>
                          Guarantor (for{" "}
                          {idx ? `tenant # ${idx + 1}` : "leading tenant"})
                        </span>
                      </GroupTitleText>
                    </GroupTitle>

                    {isOpened && (
                      <GroupBody>
                        <GroupRow>
                          <GroupColumn size="m">
                            <Label>Full name</Label>
                            <Value>
                              {t.guarantor?.first_name}
                              &nbsp;
                              {t.guarantor?.last_name}
                            </Value>
                          </GroupColumn>
                          <GroupColumn size="m">
                            <Label>Email address</Label>
                            <Value>
                              {t.guarantor?.email_primary && (
                                <NoWrap title={t.guarantor.email_primary}>
                                  <EmailLink
                                    href={`mailto:${t.guarantor.email_primary}`}
                                  >
                                    {t.guarantor.email_primary}
                                  </EmailLink>
                                </NoWrap>
                              )}
                            </Value>
                          </GroupColumn>
                          <GroupColumn size="m">
                            <Label>Phone number</Label>
                            <Value>{t.guarantor?.phone_primary}</Value>
                          </GroupColumn>
                        </GroupRow>
                      </GroupBody>
                    )}
                  </Group>
                )}
              </Collapse>
            )}
          </div>
        ))}

        <Collapse opened={false}>
          {({ isOpened, toggle }) => (
            <Group>
              <GroupTitle>
                <GroupTitleIcon onClick={toggle}>
                  <TitleIcon state={isOpened} />
                </GroupTitleIcon>
                <GroupTitleText>
                  <span>Other</span>
                </GroupTitleText>
              </GroupTitle>

              {isOpened && (
                <GroupBody>
                  <GroupRow>
                    <GroupColumn>
                      <Label>Comments</Label>
                      <Value>{conditions.comments ?? "—"}</Value>
                    </GroupColumn>
                  </GroupRow>
                  <GroupRow>
                    <GroupColumn>
                      <Label>Why are they the perfect tenants</Label>
                      <Value>{conditions.tenants_pitch ?? "—"}</Value>
                    </GroupColumn>
                  </GroupRow>
                </GroupBody>
              )}
            </Group>
          )}
        </Collapse>

        {conditions.deposit_type === DepositTypeEnum.DEPOSIT_REPLACEMENT && (
          <Collapse opened={false}>
            {({ isOpened, toggle }) => (
              <Group>
                <GroupTitle>
                  <GroupTitleIcon onClick={toggle}>
                    <TitleIcon state={isOpened} />
                  </GroupTitleIcon>
                  <GroupTitleText>
                    <span>Deposit Replacement Product</span>
                  </GroupTitleText>
                </GroupTitle>

                {isOpened && (
                  <GroupBody>
                    <GroupRow>
                      <GroupColumn size="l">
                        <Label>Tenant DRP Terms</Label>
                        <Value>
                          Accepted at:{" "}
                          {format(
                            new Date(created_at_value),
                            "dd MMM yyyy HH:mm:ss"
                          )}
                        </Value>
                      </GroupColumn>
                      <GroupColumn size="l">
                        <Label>Landlord DRP Terms</Label>
                        <Value>
                          Accepted at:{" "}
                          {accepted_at_value !== ""
                            ? format(
                                new Date(accepted_at_value),
                                "dd MMM yyyy HH:mm:ss"
                              )
                            : "—"}
                        </Value>
                      </GroupColumn>
                    </GroupRow>
                  </GroupBody>
                )}
              </Group>
            )}
          </Collapse>
        )}
      </>
    </ModalWide>
  );
};

export default OfferDetailsModal;

const TitleIcon = ({ state }: { state: boolean }) => (
  <>
    {state ? (
      <i className="material-icons">expand_less</i>
    ) : (
      <i className="material-icons">expand_more</i>
    )}
  </>
);

const Group = styled.div`
  background-color: white;
  border: 1px solid ${theme.colors.border.default};
  padding: 10px 15px;
  margin-bottom: 15px;
`;

const GroupTitle = styled.div`
  display: flex;
`;

const GroupTitleText = styled.div`
  font-size: 16px;
  padding-left: 15px;

  span,
  b {
    margin-right: 15px;
  }
`;

const GroupTitleIcon = styled.div`
  cursor: pointer;
  width: 24px;
  height: 24px;
  user-select: none;
`;

const GroupBody = styled.div`
  padding: 15px 0;
  margin-top: 10px;
  border-top: 1px solid ${theme.colors.border.default};
`;

const GroupRow = styled.div`
  display: flex;
  margin-bottom: 25px;

  &:last-of-type {
    margin-bottom: 0;
  }
`;

type GroupColumnProps = {
  size?: "l" | "m" | "s";
};

const GroupColumn = styled("div")<GroupColumnProps>`
  ${({ size }) => {
    if (size === "l") {
      return css`
        width: 50%;
      `;
    }
    if (size === "m") {
      return css`
        width: 33.33%;
      `;
    }
    if (size === "s") {
      return css`
        width: 25%;
      `;
    } else {
      return css`
        width: 100%;
      `;
    }
  }}
`;

const Label = styled.label`
  display: block;
  font-size: 16px;
  font-weight: 700;
  margin-bottom: 8px;
  text-transform: uppercase;
`;

const Value = styled.div``;
