import React, { FC, useMemo, useState } from "react";

import styled from "@emotion/styled";
import { sentenceCase } from "change-case";
import { useMutation, useQuery } from "react-apollo";

import {
  extras_api_ServiceRoleEnum,
  extras_api_ServiceTypeEnum,
  viewing_api_ManagerRewardStatusEnum as ManagerRewardStatusEnum,
} from "../../../../globalTypes";
import { SuccessButton } from "../../../../ui/atoms/Button";
import {
  PropertyCardItemTitle,
  PropertyCardItemValue,
  PropertyCardTitle,
  PropertyGrid,
} from "../../../../ui/atoms/Property";
import { ModalAlert, ModalConfirm } from "../../../../ui/molecules/Modal";
import theme from "../../../../ui/theme";
import { GetOffer_admin_Offer } from "../../graphql/queries/__generated__/GetOffer";
import {
  UpdateRewardStatus,
  UpdateRewardStatusVariables,
} from "./graphql/mutations/__generated__/UpdateRewardStatus";
import UPDATE_REWARD_STATUS from "./graphql/mutations/UpdateRewardStatus.graphql";
import {
  DepositSelectManager,
  DepositSelectManagerVariables,
} from "./graphql/query/__generated__/DepositSelectManager";
import DEPOSIT_SELECT from "./graphql/query/DepositSelectManager.graphql";

type Props = { offer: GetOffer_admin_Offer; refetchOffer: () => void };

const TabManager: FC<Props> = ({ offer }) => {
  const [isModalVisible, setModalVisibility] = useState<boolean>(false);
  const [alertMessage, setAlertMessage] = useState<string>();
  const { viewing, property } = offer;
  const { manager } = viewing;

  const { data: depositSelectData } = useQuery<
    DepositSelectManager,
    DepositSelectManagerVariables
  >(DEPOSIT_SELECT, {
    variables: {
      offer: offer.id,
    },
    skip: !viewing.id || !manager?.id,
  });

  const monthlyPayment =
    depositSelectData?.offer?.depositselect?.schedule_deposit.monthly_payment;
  const frequency =
    depositSelectData?.offer?.depositselect?.schedule_deposit.months;

  const [updateReward] = useMutation<
    UpdateRewardStatus,
    UpdateRewardStatusVariables
  >(UPDATE_REWARD_STATUS);

  const invoice = useMemo(
    () =>
      offer.invoices.find((p) =>
        (p.services ?? []).find(
          (s) =>
            s.service.type === extras_api_ServiceTypeEnum.RENT &&
            s.service.role === extras_api_ServiceRoleEnum.TENANT_SERVICES
        )
      ),
    [offer.invoices]
  );

  if (!invoice || !invoice.payments) {
    return <h3>No payments have been made yet</h3>;
  }

  const handleConfirm = () => {
    if (property.id) {
      updateReward({
        variables: {
          reward_status:
            ManagerRewardStatusEnum.MANAGER_REWARD_STATUS_REWARD_PAID,
          spec: {
            select: {
              viewing_id: viewing.id,
              property_id: property.id,
            },
          },
        },
      }).then((res) => {
        if (res.data?.admin?.updateRewardStatus?.viewing) {
          setAlertMessage("The status has been updated.");
        } else {
          setAlertMessage("Status was not updated. Try again later.");
        }

        setModalVisibility(false);
      });
    }
  };

  const bankDetails = manager?.bank_details;

  const { society, sort_code, account_number } = bankDetails ?? {};
  const { rental_price } = property.rental;

  return (
    <>
      {!viewing.id ? (
        <Row data-testid="tab-final-payment-havenot-viewing-id">
          <PropertyCardTitle>Viewing ID is unknown.</PropertyCardTitle>
        </Row>
      ) : !manager!.id ? (
        <Row data-testid="tab-final-payment-manager-id-is-unknown">
          <PropertyCardTitle>Manager of viewing is unknown.</PropertyCardTitle>
        </Row>
      ) : (
        <Row>
          <PropertyCardTitle>
            {manager!.id === property!.people!.landlord!.id ? (
              <span data-testid="tab-final-payment-manager-is-landlord">
                Landlord is managing viewings themselves
              </span>
            ) : viewing.reward_status ===
              ManagerRewardStatusEnum.MANAGER_REWARD_STATUS_REWARD_PAID ? (
              <TextSuccess data-testid="tab-final-payment-reward-paid">
                Manager{" "}
                {`${manager!.details!.first_name} ${
                  manager!.details!.last_name
                }`}{" "}
                have got reward paid.
              </TextSuccess>
            ) : viewing.reward_status ===
              ManagerRewardStatusEnum.MANAGER_REWARD_STATUS_NO_REWARD ? (
              "Manager is unknown."
            ) : viewing.reward_status !==
              ManagerRewardStatusEnum.MANAGER_REWARD_STATUS_AWAITING_REWARD_PAYMENT ? (
              <TextError data-testid="tab-final-payment-awaiting">
                Manager{" "}
                {`${manager!.details!.first_name} ${
                  manager!.details!.last_name
                }`}{" "}
                is not ready to reward paid.
              </TextError>
            ) : (
              <TextSuccess data-testid="tab-final-payment-ready-to-reward">
                Manager{" "}
                {`${manager!.details!.first_name} ${
                  manager!.details!.last_name
                }`}
                is ready to reward paid.
                <Amount>
                  Amount owed &ndash;&nbsp;
                  <Sum data-testid="tab-final-payment-amount-sum">
                    &pound;{" "}
                    {((((rental_price as number) * 12) / 52) ^ 0).toFixed(2)}
                  </Sum>
                </Amount>
              </TextSuccess>
            )}
          </PropertyCardTitle>
          {viewing.reward_status ===
          ManagerRewardStatusEnum.MANAGER_REWARD_STATUS_AWAITING_REWARD_PAYMENT ? (
            <Content>
              <BankDetails>
                <Col>
                  <PropertyCardItemTitle>Bank name</PropertyCardItemTitle>
                  <PropertyCardItemValue data-testid="tab-final-payment-society">
                    {society || "- - -"}
                  </PropertyCardItemValue>
                </Col>

                <Col>
                  <PropertyCardItemTitle>Sort code</PropertyCardItemTitle>
                  <PropertyCardItemValue data-testid="tab-final-payment-sort-code">
                    {sort_code || "- - -"}
                  </PropertyCardItemValue>
                </Col>

                <Col>
                  <PropertyCardItemTitle>Account number</PropertyCardItemTitle>
                  <PropertyCardItemValue data-testid="tab-final-payment-account-number">
                    {account_number || "- - -"}
                  </PropertyCardItemValue>
                </Col>
                <Col>
                  <PropertyCardItemTitle>
                    Standing order amount
                  </PropertyCardItemTitle>
                  <PropertyCardItemValue data-testid="tab-final-payment-standing-order-amount">
                    {monthlyPayment || "- - -"}
                  </PropertyCardItemValue>
                </Col>

                <Col>
                  <PropertyCardItemTitle>Frequency</PropertyCardItemTitle>
                  <PropertyCardItemValue data-testid="tab-final-payment-frequency">
                    {frequency || "- - -"}
                  </PropertyCardItemValue>
                </Col>

                <Col>
                  <PropertyCardItemTitle>Status</PropertyCardItemTitle>
                  <PropertyCardItemValue data-testid="tab-final-payment-status">
                    {sentenceCase(invoice.status)}
                  </PropertyCardItemValue>
                </Col>
              </BankDetails>
              <Action>
                <SuccessButton
                  data-testid="tab-final-payment-btn-complete"
                  onClick={() => setModalVisibility(true)}
                >
                  Payment complete
                </SuccessButton>
              </Action>
              {isModalVisible ? (
                <ModalConfirm
                  onCancel={() => setModalVisibility(false)}
                  onConfirm={handleConfirm}
                >
                  <strong>
                    Are you sure you have paid the correct reward to the
                    Viewings Manager? This action cannot be reversed.
                  </strong>
                </ModalConfirm>
              ) : null}
              {alertMessage ? (
                <ModalAlert
                  onCancel={() => {
                    setAlertMessage("");
                  }}
                >
                  <b data-testid="tab-final-payment-modal-alert-message">
                    {alertMessage}
                  </b>
                </ModalAlert>
              ) : null}
            </Content>
          ) : null}
        </Row>
      )}
    </>
  );
};

const Row = styled.div`
  max-width: 800px;
`;
const Col = styled.div`
  padding-right: 30px;
`;
const TextSuccess = styled.div`
  color: ${theme.colors.main.green};
`;
const TextError = styled.div`
  color: ${theme.colors.main.red};
`;
const Sum = styled.span`
  color: ${theme.colors.main.mainBlue};
`;
const Content = styled.div`
  margin-top: 34px;
`;
const Action = styled.div`
  margin: 0 0 30px;
`;
const BankDetails = styled(PropertyGrid)`
  padding: 30px 30px 44px;
  background: #f9f6f1;
  border: dashed 2px ${theme.colors.main.lightBlue};
  margin-bottom: 40px;
`;
const Amount = styled.div`
  color: ${theme.colors.main.darkGrey};
`;

export default TabManager;
