import React, { useCallback, useState } from "react";

import { Config } from "final-form";
import { useMutation } from "react-apollo";
import { Field, Form } from "react-final-form";

import { offer_api_OfferStateEnum } from "../../../../globalTypes";
import { ApiError } from "../../../../ui/atoms/ApiError/ApiError";
import { PrimaryButton, SecondaryButton } from "../../../../ui/atoms/Button";
import { TextInput } from "../../../../ui/atoms/Input";
import { Table, Tbody, Thead } from "../../../../ui/atoms/Table";
import TableWrapper from "../../../../ui/atoms/TableWrapper";
import { FileField } from "../../../../ui/molecules/FileField/FileField";
import { ModalWide } from "../../../../ui/molecules/Modal";
import {
  composeValidators,
  required,
  validEmail,
} from "../../../../util/validations";
import {
  SetRiskReport,
  SetRiskReportVariables,
} from "../../graphql/mutations/__generated__/SetRiskReport";
import {
  SkipReferencingStage,
  SkipReferencingStageVariables,
} from "../../graphql/mutations/__generated__/SkipReferencingStage";
import SET_RISK_REPORT from "../../graphql/mutations/SetRiskReport.graphql";
import SKIP_REFERENCING from "../../graphql/mutations/SkipReferencingStage.graphql";
import { GetOffer_admin_Offer } from "../../graphql/queries/__generated__/GetOffer";
import OfferRentProfileModal from "../../OfferRentProfileModal";
import TabRentprofileGuarantorItem from "./TabRentprofileGuarantorItem";
import TabRetnprofileHead from "./TabRentprofileHead";
import TabRentprofileTenantItem from "./TabRentprofileTenantItem";

import css from "./TabRentprofile.module.css";

type RentProfileForm = {
  file: string;
  email: string;
};

type Props = { offer: GetOffer_admin_Offer; refetchOffer: () => void };

const TabRetnprofile: React.FC<Props> = ({ offer, refetchOffer }) => {
  const [showModal, setShowModal] = useState(false);
  const [rentProfileModal, setRentProfileModal] = useState<boolean>(false);

  const closeModal = useCallback(() => {
    setShowModal(false);
  }, [setShowModal]);

  const [skipReferencing, { loading, error }] = useMutation<
    SkipReferencingStage,
    SkipReferencingStageVariables
  >(SKIP_REFERENCING);

  const [setRiskProfileMutation, { error: RPError }] = useMutation<
    SetRiskReport,
    SetRiskReportVariables
  >(SET_RISK_REPORT);

  const handleSkip = async () => {
    await skipReferencing({
      variables: {
        offer: offer.id,
      },
    });
    await refetchOffer();
  };

  const openRPModal = useCallback(() => {
    setRentProfileModal(true);
  }, []);

  const closeRPModal = useCallback(() => {
    setRentProfileModal(false);
  }, []);

  const handleRentProfileSubmit: Config<
    RentProfileForm
  >["onSubmit"] = useCallback(
    ({ file, email }) => {
      void setRiskProfileMutation({
        variables: {
          offer_id: offer.id,
          user_email: email,
          file_id: file,
        },
      }).then(({ errors }) => {
        if (!errors) {
          closeRPModal();
        }
      });
    },
    [closeRPModal, offer, setRiskProfileMutation]
  );

  return (
    <TableWrapper>
      <Table>
        <Thead>
          <TabRetnprofileHead />
        </Thead>
        <Tbody>
          {(offer.tenants ?? []).map((t, idx) => (
            <TabRentprofileTenantItem
              key={`${t.first_name}${t.email_primary}`}
              idx={idx}
              tenant={t}
            />
          ))}
          {(offer.guarantors ?? []).map((guarantor) => (
            <TabRentprofileGuarantorItem
              key={guarantor.user ?? ""}
              guarantor={guarantor}
            />
          ))}
        </Tbody>
      </Table>
      <div className={css.buttonWrapper}>
        <PrimaryButton
          disabled={checkStatus(offer.state)}
          onClick={() => setShowModal(true)}
        >
          Resend Rentprofile requests
        </PrimaryButton>
        <SecondaryButton
          disabled={checkStatusForSkip(offer.state) || loading}
          onClick={handleSkip}
        >
          Skip referencing stage
        </SecondaryButton>
        {isRPUploadAvailable(offer.state) && (
          <SecondaryButton className={css.addRpButton} onClick={openRPModal}>
            Upload rentprofile
          </SecondaryButton>
        )}
      </div>
      <ApiError errors={[error]} />

      <OfferRentProfileModal
        handleCancel={closeModal}
        isOpen={showModal}
        offer={offer}
      />

      <ModalWide
        isOpen={rentProfileModal}
        onCancel={closeRPModal}
        title={"Upload rent profile"}
      >
        <Form<RentProfileForm>
          onSubmit={handleRentProfileSubmit}
          render={({ handleSubmit, valid }) => (
            <form onSubmit={handleSubmit}>
              <Field<string>
                name="file"
                render={({ input, meta }) => (
                  <FileField
                    acceptedTypes={acceptedFileTypes}
                    error={meta.touched && meta.error}
                    fileId={input.value}
                    handleChange={input.onChange}
                  />
                )}
                validate={required()}
              />
              <Field<string>
                name="email"
                placeholder="e-mail"
                render={TextInput}
                validate={composeValidators(required(), validEmail())}
              />
              <ApiError errors={[RPError]} />
              <div className={css.rpModalButtons}>
                <SecondaryButton onClick={closeRPModal}>Close</SecondaryButton>
                <PrimaryButton disabled={!valid} type="submit">
                  Add
                </PrimaryButton>
              </div>
            </form>
          )}
        />
      </ModalWide>
    </TableWrapper>
  );
};

const checkStatus = (value: string) => {
  switch (value) {
    case offer_api_OfferStateEnum.DEPOSIT_CONFIRMED:
    case offer_api_OfferStateEnum.CHECKING:
    case offer_api_OfferStateEnum.AWAITING_FEES:
    case offer_api_OfferStateEnum.AWAITING_CHECKS:
    case offer_api_OfferStateEnum.AWAITING_DOCUMENTS:
    case offer_api_OfferStateEnum.AWAITING_APPROVE:
    case offer_api_OfferStateEnum.CONTRACT_PREPARATION:
    case offer_api_OfferStateEnum.SIGNING:
    case offer_api_OfferStateEnum.SIGNED:
    case offer_api_OfferStateEnum.RENTPAID:
    case offer_api_OfferStateEnum.AWAITING_APPLICANT_CHECK_PAYMENT:
    case offer_api_OfferStateEnum.AWAITING_GUARANTOR:
      return false;
    default:
      return true;
  }
};

const isRPUploadAvailable = (value: string) => {
  if (value === offer_api_OfferStateEnum.CONTRACT_REVIEW_AND_CLAUSES) {
    return false;
  }

  return checkStatus(value);
};

const checkStatusForSkip = (value: offer_api_OfferStateEnum) => {
  switch (value) {
    case offer_api_OfferStateEnum.AWAITING_APPLICANT_CHECK_PAYMENT:
    case offer_api_OfferStateEnum.AWAITING_CHECKS:
      return false;
    default:
      return true;
  }
};
export default TabRetnprofile;

const acceptedFileTypes = ["application/pdf"];
