import React, { FC, useCallback, useMemo, useState } from "react";

import styled from "@emotion/styled";
import { useQuery } from "react-apollo";

import { expenses_api_PayoutStatusEnum } from "../../../globalTypes";
import { PrimaryButton } from "../../../ui/atoms/Button";
import { ColumnTitle, ColumnValue } from "../../../ui/atoms/Payment";
import Spinner from "../../../ui/atoms/Spinner";
import SystemError from "../../../ui/atoms/SystemError";
import { ModalConfirm } from "../../../ui/molecules/Modal";
import {
  PaymentsGetUser,
  PaymentsGetUserVariables,
} from "../graphql/queries/__generated__/PaymentsGetUser";
import GETCH_USER from "../graphql/queries/PaymentsGetUser.graphql";

type Props = {
  payoutStatus: expenses_api_PayoutStatusEnum | undefined;
  landlordId: string | undefined;
  loading: boolean;
  onPayout: () => void;
  sum: string | number;
  visible: boolean;
};

const BankDetails: FC<Props> = ({
  payoutStatus,
  landlordId,
  loading,
  onPayout,
  sum,
  visible,
}) => {
  const [isConfirmVisible, setConfirm] = useState<boolean>(false);
  const { data, loading: userLoading, error } = useQuery<
    PaymentsGetUser,
    PaymentsGetUserVariables
  >(GETCH_USER, {
    variables: {
      id: landlordId,
    },
    skip: landlordId == null,
  });

  const bankDetails = useMemo(() => {
    return data?.admin?.User?.bank_details;
  }, [data]);

  const handleClickPayout = useCallback(() => {
    setConfirm(true);
  }, []);

  const handleConfirm = useCallback(() => {
    setConfirm(false);
    onPayout();
  }, [onPayout]);

  const handleCancel = useCallback(() => {
    setConfirm(false);
  }, []);

  return (
    <>
      {isConfirmVisible ? (
        <ModalConfirm onCancel={handleCancel} onConfirm={handleConfirm}>
          <Message>
            Are you sure you want <br />
            to payout <b>{sum}</b>?
          </Message>
        </ModalConfirm>
      ) : null}
      <Container>
        <Title>
          <span>Bank Details</span>
          {payoutStatus ===
            expenses_api_PayoutStatusEnum.PAYOUT_STATUS_PENDING && visible ? (
            <PrimaryButton
              disabled={loading}
              onClick={handleClickPayout}
              type="button"
            >
              Payout
            </PrimaryButton>
          ) : null}
        </Title>
        {userLoading ? (
          <Spinner size="sm" />
        ) : error ? (
          <SystemError error={error} />
        ) : data ? (
          <Row>
            <Col>
              <ColumnTitle>Account Number</ColumnTitle>
              <ColumnValue>
                {bankDetails?.account_number
                  ? bankDetails?.account_number
                  : "- - -"}
              </ColumnValue>
            </Col>
            <Col>
              <ColumnTitle>Society</ColumnTitle>
              <ColumnValue>
                {bankDetails?.society ? bankDetails?.society : "- - -"}
              </ColumnValue>
            </Col>
            <Col>
              <ColumnTitle>Sort Code</ColumnTitle>
              <ColumnValue>
                {bankDetails?.sort_code ? bankDetails?.sort_code : "- - -"}
              </ColumnValue>
            </Col>
          </Row>
        ) : null}
      </Container>
    </>
  );
};

const Message = styled.div`
  font-size: 18px;
  font-weight: 500;
`;

const Container = styled.div`
  margin: 20px 0;
  background: #f5f5f6;
  border: solid 1px #d8d8d8;
  padding: 14px 20px;
`;

const Title = styled.div`
  width: 100%;
  margin: 0 0 24px;
  font-size: 16px;
  font-weight: 700;
  display: flex;
  align-items: center;
  justify-content: space-between;
`;

const Row = styled.div`
  display: flex;
  justify-content: space-between;
`;

const Col = styled.div`
  width: 30%;
`;

export default BankDetails;
