import React, { useMemo } from "react";

import { sentenceCase } from "change-case";
import { useMutation } from "react-apollo";
import { Link } from "react-router-dom";

import {
  expenses_api_PayoutStatusEnum,
  invoice_api_PaymentStatusEnum,
  invoice_api_TransactionReconciliationEnum,
  invoice_api_TransactionTypeEnum,
} from "../../../globalTypes";
import { PrimaryButton } from "../../../ui/atoms/Button";
import { Td } from "../../../ui/atoms/Table";
import { formatDateTime } from "../../../util/date";
import { formatPrice } from "../../../util/price";
import { InvoiceTransactionOutputFields } from "../graphql/fragments/__generated__/InvoiceTransactionOutputFields";
import {
  CreatePayment,
  CreatePaymentVariables,
} from "../graphql/mutations/__generated__/CreatePayment";
import CREATE_PAYMENT from "../graphql/mutations/CreatePayment.graphql";

import css from "./index.module.css";

export type ListItemProps = {
  transaction: InvoiceTransactionOutputFields;
};

const ListItem: React.FC<ListItemProps> = ({ transaction }) => {
  const {
    reconciliation,
    description,
    id,
    ivr,
    created,
    amount,
    type,
    matches,
    scheduled_records,
  } = transaction;
  const isFullyMatched =
    reconciliation ===
    invoice_api_TransactionReconciliationEnum.TRANSACTION_RECONCILIATION_FULL;
  const payoutValue = (scheduled_records?.scheduledRecords ?? [])[0]?.record
    .type.income?.rent?.payouts[0].value;
  const isPayoutPaid =
    payoutValue?.payout_status ===
    expenses_api_PayoutStatusEnum.PAYOUT_STATUS_PAID;
  const { amount: payoutAmount, invoice_id } = payoutValue ?? {};
  const isCredit =
    type === invoice_api_TransactionTypeEnum.TRANSACTION_TYPE_CREDIT;
  const payoutInformation = useMemo(() => {
    if (
      !isFullyMatched ||
      !matches ||
      matches?.length === 0 ||
      !matches[0].payment?.property
    ) {
      return null;
    }
    const { property } = matches[0].payment;
    const { landlord, tenants } = property.people;
    const { account_name, account_number, sort_code, personal_reference } =
      property.bank_details ?? {};
    return `IVR: ${property.ivr} LL: ${landlord?.details?.full_name} TT: ${tenants[0]?.user?.details?.full_name} AMOUNT: ${payoutAmount} BANK DETAILS: Acc name: ${account_name} Acc number: ${account_number} Sort code: ${sort_code} Personal ref: ${personal_reference}`;
  }, [payoutAmount, isFullyMatched, matches]);

  const [createPayment, { loading: createPaymentLoading }] = useMutation<
    CreatePayment,
    CreatePaymentVariables
  >(CREATE_PAYMENT);

  return (
    <tr className={isFullyMatched ? css.hasMatches : css.notMatches}>
      <Td>{id}</Td>
      <Td>{ivr}</Td>
      <Td>{formatDateTime(created)}</Td>
      <Td>{formatPrice(amount)}</Td>
      <Td>{payoutInformation}</Td>
      <Td>{description}</Td>
      <Td>
        <b>{sentenceCase(type.replace("TRANSACTION_TYPE_", ""))}</b>
      </Td>
      <Td>
        {isPayoutPaid ? (
          <b>Paid out</b>
        ) : (
          <>
            {isFullyMatched && isCredit ? (
              <PrimaryButton
                disabled={createPaymentLoading}
                onClick={() =>
                  createPayment({
                    variables: {
                      invoice_id,
                      status: invoice_api_PaymentStatusEnum.PAYMENT_SUCCESS,
                    },
                    refetchQueries: ["PaymentsGetAllTransactions"],
                  })
                }
                type="button"
              >
                Mark as paid out
              </PrimaryButton>
            ) : (
              "Match transaction required"
            )}
          </>
        )}
      </Td>
      <Td>
        <Link
          className="link"
          to={{
            pathname: `/statement/${id}`,
          }}
        >
          View -&gt;
        </Link>
      </Td>
    </tr>
  );
};

export default ListItem;
