import React, { FC, useCallback, useEffect, useState } from "react";

import AntdButton from "antd-button-color";
import cn from "classnames";
import { useMutation, useQuery } from "react-apollo";
import { Field, Form } from "react-final-form";
import { OnChange } from "react-final-form-listeners";

import {
  common_api_generic_query_QueryFilterOperatorEnum as OperatorEnum,
  invoice_api_InvoiceStatusEnum,
} from "../../../../../globalTypes";
import { Invoice_Api_PaymentStatusEnum } from "../../../../../graphql-schema-types.generated";
import { TextInput } from "../../../../../ui/atoms/Input";
import Dropdown from "../../../../../ui/molecules/Dropdown";
import {
  AVAILABLE_CORRELATION_TARGETS,
  CORRELATION_TARGET_LABEL,
} from "../../../../unmatched-transactions/constants";
import FindAddress from "../../../../unmatched-transactions/FindAddress";
import { useTransactionMatchPropertyAddress } from "../../../../unmatched-transactions/hooks";
import {
  getEarliestItemFromArray,
  isEarlier,
} from "../../../../unmatched-transactions/utils";
import {
  AddPaymentTransactionMatch,
  AddPaymentTransactionMatchVariables,
} from "../../../graphql/mutations/__generated__/AddPaymentTransactionMatch";
import {
  UpdateInvoice,
  UpdateInvoiceVariables,
} from "../../../graphql/mutations/__generated__/UpdateInvoice";
import ADD_TRANSACTION_MATCH from "../../../graphql/mutations/AddPaymentTransactionMatch.graphql";
import UPDATE_INVOICE from "../../../graphql/mutations/UpdateInvoice.graphql";
import {
  PaymentsGetAllInvoices,
  PaymentsGetAllInvoicesVariables,
} from "../../../graphql/queries/__generated__/PaymentsGetAllInvoices";
import FETCH_INVOICES from "../../../graphql/queries/PaymentsGetAllInvoices.graphql";
import {
  PaymentsGetInvoiceQuery,
  PaymentsGetInvoiceQueryVariables,
} from "../../../graphql/queries/PaymentsGetInvoice.generated";
import GET_INVOICE from "../../../graphql/queries/PaymentsGetInvoice.graphql";
import {
  FormValues,
  MatchTransactionFormProps,
  PaymentOption,
  PropertyInput,
} from "./types";

import styles from "./styles.module.css";

export const MatchTransactionForm: FC<MatchTransactionFormProps> = ({
  className,
  buttonText = "Match",
  onSubmit,
  transactionId,
  transactionMatches,
}) => {
  const [paymentOptions, setPaymentOptions] = useState<PaymentOption[]>([]);
  const [createPayment, { loading: createPaymentLoading }] = useMutation<
    AddPaymentTransactionMatch,
    AddPaymentTransactionMatchVariables
  >(ADD_TRANSACTION_MATCH);

  const [updateInvoice, { loading: updateInvoiceLoading }] = useMutation<
    UpdateInvoice,
    UpdateInvoiceVariables
  >(UPDATE_INVOICE);

  const [propertyInput, setPropertyInput] = useState<PropertyInput>({
    location: "",
    property_id: "",
    invoice_id: "",
    payment_id: "",
  });
  const admin_id = window.localStorage.getItem("adminId") ?? "";

  const {
    address: transactionMatchPropertyAddress,
    cityPostcodeAndCountry: transactionMatchPropertyCityPostcodeAndCountry,
  } = useTransactionMatchPropertyAddress(transactionMatches);

  const withPrefilledPropertyAddress =
    Boolean(transactionMatchPropertyAddress) ||
    Boolean(transactionMatchPropertyCityPostcodeAndCountry);

  const {
    data: dataInvoices,
    loading: invoicesLoading,
    networkStatus: invoicesStatus,
  } = useQuery<PaymentsGetAllInvoices, PaymentsGetAllInvoicesVariables>(
    FETCH_INVOICES,
    {
      variables: {
        pagination: { page: 1, perPage: 400 },
        filter: [
          {
            fields: ["status"],
            not: true,
            operator: OperatorEnum.ONEOF,
            value: [
              invoice_api_InvoiceStatusEnum.INVOICE_FULFILLED,
              invoice_api_InvoiceStatusEnum.INVOICE_CANCELED,
            ],
          },
          {
            fields: ["correlation_target"],
            operator: OperatorEnum.CONTAINS_ANY,
            value: AVAILABLE_CORRELATION_TARGETS,
          },
          {
            fields: ["property"],
            operator: OperatorEnum.EQUALS,
            value: [propertyInput.property_id],
          },
        ],
      },
      skip: propertyInput.location === "" || propertyInput.property_id === "",
    }
  );

  useEffect(() => {
    setPaymentOptions([]);
    const newPaymentOptions = [] as PaymentOption[];

    const allInvoices = [...(dataInvoices?.admin?.allInvoices?.invoices ?? [])];

    const invoiceByCorrelationTarget = allInvoices.reduce((group, invoice) => {
      const { correlation_target } = invoice;
      if (AVAILABLE_CORRELATION_TARGETS.includes(correlation_target)) {
        group[correlation_target] =
          !group[correlation_target] ||
          isEarlier(invoice.updated_at, group[correlation_target].updated_at)
            ? invoice
            : group[correlation_target];
      }
      return group;
    }, {});

    Object.keys(invoiceByCorrelationTarget).forEach((key) => {
      const selected = invoiceByCorrelationTarget[key];
      const option = {
        label: `${CORRELATION_TARGET_LABEL[selected.correlation_target]}: ${
          selected.amount
        }`,
        value: selected.invoice_id,
      };
      newPaymentOptions.push(option);
    });

    setPaymentOptions(newPaymentOptions);
  }, [dataInvoices]);

  const {
    data: dataInvoicePayments,
    loading: invoicePaymentsLoading,
  } = useQuery<PaymentsGetInvoiceQuery, PaymentsGetInvoiceQueryVariables>(
    GET_INVOICE,
    {
      variables: {
        invoice_id: propertyInput.invoice_id,
      },
      skip: propertyInput.invoice_id === "",
    }
  );

  const xeroCode = dataInvoicePayments?.admin?.Invoice?.xero_code;

  useEffect(() => {
    if (dataInvoicePayments !== undefined) {
      const filteredPayments = dataInvoicePayments.admin?.Invoice?.payments?.filter(
        ({ status }) => status === Invoice_Api_PaymentStatusEnum.PaymentPending
      );
      if (filteredPayments && filteredPayments.length > 0) {
        const selectedPayment = getEarliestItemFromArray(filteredPayments);
        setPropertyInput((state) => ({
          ...state,
          payment_id: selectedPayment?.id ?? "",
        }));
      }
    }
  }, [dataInvoicePayments]);

  const handleUpdateStatus = useCallback(
    async ({
      invoiceId,
      transactionId,
      xeroCode,
    }: {
      invoiceId: string | undefined;
      transactionId: string;
      xeroCode: string | undefined;
    }) => {
      if (xeroCode) {
        await updateInvoice({
          variables: {
            invoiceId,
            xeroCode,
          },
        });
      }

      await createPayment({
        variables: {
          transaction_id: transactionId,
          comment: "custom matching",
          admin_id,
          find: { invoice_id: invoiceId },
        },
      });

      window.location.reload();
    },
    [admin_id, createPayment, updateInvoice]
  );

  if (withPrefilledPropertyAddress) {
    return null;
  }

  const isMergeLoading =
    createPaymentLoading || updateInvoiceLoading || invoicePaymentsLoading;

  return (
    <Form<FormValues>
      onSubmit={(values) => {
        if (propertyInput.invoice_id && transactionId) {
          handleUpdateStatus({
            invoiceId: propertyInput.invoice_id,
            transactionId,
            xeroCode: values.xero_code,
          });
        }
      }}
    >
      {({ values }) => {
        return (
          <div className={cn(styles.matchForm, className)}>
            <div className={styles.address}>
              <FindAddress setPropertyInput={setPropertyInput} />
            </div>
            {invoicesLoading ? (
              <div className={styles.spinner} />
            ) : paymentOptions.length > 0 ? (
              <div className={styles.fields}>
                <Field
                  name="invoice_id"
                  render={({ input }) => (
                    <Dropdown
                      fullWidth
                      input={input}
                      options={paymentOptions}
                      placeholder="Payment type"
                    />
                  )}
                />
                {values?.invoice_id?.value &&
                  !xeroCode &&
                  !invoicePaymentsLoading && (
                    <div>
                      <Field
                        component={TextInput}
                        name="xero_code"
                        placeholder="Xero code"
                      />
                    </div>
                  )}
              </div>
            ) : Boolean(invoicesStatus) ? (
              <span>no appropriate invoices</span>
            ) : null}

            <OnChange name="invoice_id">
              {(value: any) => {
                if (value) {
                  setPropertyInput((state) => ({
                    ...state,
                    invoice_id: value.value,
                  }));
                }
              }}
            </OnChange>

            <AntdButton
              className={styles.matchButton}
              disabled={
                !(propertyInput.invoice_id && transactionId) || isMergeLoading
              }
              htmlType="button"
              onClick={() => {
                handleUpdateStatus({
                  invoiceId: propertyInput.invoice_id,
                  transactionId,
                  xeroCode: values.xero_code,
                });

                if (onSubmit) {
                  onSubmit();
                }
              }}
              type="primary"
            >
              {isMergeLoading ? "Loading" : buttonText}
            </AntdButton>
          </div>
        );
      }}
    </Form>
  );
};
