import React, { FC, useState } from "react";

import { useQuery } from "@apollo/react-hooks";
import { format } from "date-fns";
import { Field, Form } from "react-final-form";
import { Link } from "react-router-dom";

import {
  common_api_generic_query_QueryFilterOperatorEnum,
  common_api_generic_query_QueryParamsFilterInput as ParamsFilterInput,
} from "../../../globalTypes";
import { PrimaryButton, SecondaryButton } from "../../../ui/atoms/Button";
import { TextInput } from "../../../ui/atoms/Input";
import { SearchField } from "../../../ui/atoms/SearchField";
import Spinner from "../../../ui/atoms/Spinner";
import SystemError from "../../../ui/atoms/SystemError";
import { Table, Tbody, Td, Th, Thead, Tr } from "../../../ui/atoms/Table";
import TableWrapper from "../../../ui/atoms/TableWrapper";
import Pagebar from "../../../ui/molecules/Pagebar";
import { usePagination } from "../../../ui/molecules/Pagination";
import { useSortable } from "../../../ui/molecules/Sortable";
import SortableTh from "../../../ui/molecules/SortableTh";
import getSortOrder from "../../../util/ger_sort_order";
import { ScheduledPaymentOutputFields } from "../graphql/fragments/__generated__/ScheduledPaymentOutputFields";
import {
  PaymentsGetAllScheduledPayments,
  PaymentsGetAllScheduledPaymentsVariables,
} from "../graphql/queries/__generated__/PaymentsGetAllScheduledPayments";
import GET_SCHEDULE from "../graphql/queries/PaymentsGetAllScheduledPayments.graphql";
import UserInfo from "../UserInfo";
import { getAdvancedRentStatus } from "./util";

const PER_PAGE = 30;

const searchFieldNames = ["id", "offer", "landlord"];

interface FilterFormValues {
  search: string;
}

const AdvancedRent: FC = () => {
  const { field, order, setSort } = useSortable({
    field: ["date"],
    order: -1,
  });
  const [filter, setFilter] = useState<ParamsFilterInput[]>([]);
  const { page, setPage, perPage } = usePagination({
    perPage: PER_PAGE,
    filter,
  });

  const { data, loading, error } = useQuery<
    PaymentsGetAllScheduledPayments,
    PaymentsGetAllScheduledPaymentsVariables
  >(GET_SCHEDULE, {
    variables: {
      sort: field.map((v) => ({ field: v, order })),
      filter,
      pagination: { page, perPage },
    },
  });

  const scheduledPayments = data?.admin?.allScheduledPayments?.data ?? [];

  const getOrder = (fieldName: string) =>
    getSortOrder({ field, fieldName, order });

  const handleSearchSubmit = (values: FilterFormValues) => {
    setFilter([
      {
        fields: searchFieldNames,
        value: [values.search],
        operator: common_api_generic_query_QueryFilterOperatorEnum.CONTAINS_ANY,
      },
    ]);
  };

  return (
    <>
      <h2>Advanced rent</h2>
      <Form<FilterFormValues>
        onSubmit={handleSearchSubmit}
        render={({ form, handleSubmit, values, submitting }) => {
          const handleSearchClear = () => {
            form.reset();
            setFilter([]);
          };
          return (
            <form onSubmit={handleSubmit}>
              <SearchField>
                <Field
                  component={TextInput}
                  name="search"
                  placeholder="Enter payment ID, offer ID or landlord user ID"
                />
                <PrimaryButton
                  disabled={!values.search || submitting}
                  type="submit"
                >
                  Search
                </PrimaryButton>
                {filter && (
                  <SecondaryButton onClick={handleSearchClear} type="reset">
                    Clear
                  </SecondaryButton>
                )}
              </SearchField>
            </form>
          );
        }}
      />

      {loading ? (
        <Spinner />
      ) : error ? (
        <SystemError error={error} />
      ) : (
        <TableWrapper>
          <Pagebar
            changePage={setPage}
            curPage={page}
            showNPages={3}
            total={Math.ceil(
              (data?.admin?.allScheduledPayments?.total ?? 0) / PER_PAGE
            )}
          />
          <Table>
            <Thead>
              <AdvancedRentHead
                field={field}
                getOrder={getOrder}
                order={order}
                setSort={setSort}
              />
            </Thead>
            <Tbody>
              {scheduledPayments.map((p, idx) => (
                // FIXME Need uniq key/ID here
                <AdvancedRentItem
                  key={`${p.id}_${p.offer?.id}_${idx}`}
                  payment={p}
                />
              ))}
            </Tbody>
          </Table>
          <Pagebar
            changePage={setPage}
            curPage={page}
            showNPages={3}
            total={Math.ceil(
              (data?.admin?.allScheduledPayments?.total ?? 0) / PER_PAGE
            )}
          />
        </TableWrapper>
      )}
    </>
  );
};

export default AdvancedRent;

type HeadProps = {
  order: number;
  setSort: ({ field, order }: { field: string[]; order: number }) => void;
  getOrder: (fieldName: string) => number;
  field: string | string[];
};

const AdvancedRentHead: FC<HeadProps> = ({
  order,
  setSort,
  getOrder,
  field,
}) => (
  <Tr>
    <Th>Payment ID</Th>
    <SortableTh
      handleClick={() =>
        setSort({
          field: ["offer"],
          order: getOrder("offer"),
        })
      }
      include={field.includes("offer")}
      order={order}
    >
      Offer ID
    </SortableTh>
    <Th>Landlord</Th>
    <SortableTh
      handleClick={() =>
        setSort({
          field: ["date"],
          order: getOrder("date"),
        })
      }
      include={field.includes("date")}
      order={order}
    >
      Month
    </SortableTh>
    <Th>Status</Th>
    <Th>Details</Th>
  </Tr>
);

const AdvancedRentItem: FC<{ payment: ScheduledPaymentOutputFields }> = ({
  payment,
}) => (
  <Tr>
    <Td>{payment.id}</Td>
    <Td>
      <Link
        className="link"
        to={{
          pathname: `/offers/${payment.offer?.id}/edit`,
        }}
      >
        {payment.offer?.id}
      </Link>
    </Td>
    <Td>
      {payment.offer?.landlord.user != null &&
      payment.offer.landlord.user !== "" ? (
        <UserInfo userId={payment.offer.landlord.user} />
      ) : (
        "—"
      )}
    </Td>
    <Td>
      {format(new Date(payment.date.year, payment.date.month - 1), "MMMM")}
    </Td>
    <Td>{getAdvancedRentStatus(payment.advance.state)}</Td>
    <Td>
      <Link
        className="link"
        to={{
          pathname: `/advanced/${payment.id}`,
          search: `offer=${payment.offer?.id}`,
        }}
      >
        View -&gt;
      </Link>
    </Td>
  </Tr>
);
