import React, { FC, Fragment, useCallback, useState } from "react";

import { sentenceCase } from "change-case";
import { format } from "date-fns";
import { useMutation, useQuery } from "react-apollo";
import { Link } from "react-router-dom";

import {
  common_api_generic_query_QueryFilterOperatorEnum,
  extras_api_ServiceStatusEnum,
  invoice_api_PaymentStatusEnum,
  offer_api_DepositTypeEnum,
} from "../../../globalTypes";
import { PrimaryButton } from "../../../ui/atoms/Button";
import {
  ColumnNarrow,
  ColumnTitle,
  ColumnValue,
  ColumnWide,
  PaymentBody,
  PaymentClose,
  PaymentConfirmed,
  PaymentCreatedAt,
  PaymentDescription,
  PaymentHead,
  PaymentOpen,
  PaymentRow,
} from "../../../ui/atoms/Payment";
import Spinner from "../../../ui/atoms/Spinner";
import SystemError from "../../../ui/atoms/SystemError";
import { Table, Tbody, Td, Th, Thead, Tr } from "../../../ui/atoms/Table";
import ClickableText from "../../../ui/molecules/ClickableText";
import Collapse from "../../../ui/molecules/Collapse";
import Pagebar from "../../../ui/molecules/Pagebar";
import { formatDateTime, formatDateWordTime } from "../../../util/date";
import { formatDate } from "../../../util/formatters/date";
import { formatPrice, formatPricePcm } from "../../../util/price";
import AssignAdminModal from "../../admin/AssignAdminModal";
import { propertyAddressFilter } from "../../listing/filters";
import { OFFER_STAGE, RENT_PLAN_PACKAGES } from "../../offers/constants";
import { ScheduledPaymentOutputFields_parts } from "../graphql/fragments/__generated__/ScheduledPaymentOutputFields";
import {
  CreatePay,
  CreatePayVariables,
} from "../graphql/mutations/__generated__/CreatePay";
import {
  UpdateInvoiceItems,
  UpdateInvoiceItemsVariables,
} from "../graphql/mutations/__generated__/UpdateInvoiceItems";
import CREATE_PAYMENT from "../graphql/mutations/CreatePay.graphql";
import UPDATE_INVOICE from "../graphql/mutations/UpdateInvoiceItems.graphql";
import {
  GetOfferForPayment,
  GetOfferForPaymentVariables,
} from "../graphql/queries/__generated__/GetOfferForPayment";
import {
  PaymentsGetAllScheduledPayments,
  PaymentsGetAllScheduledPaymentsVariables,
} from "../graphql/queries/__generated__/PaymentsGetAllScheduledPayments";
import {
  PaymentsGetInvoice_admin_Invoice,
  PaymentsGetInvoice_admin_Invoice_payments,
  PaymentsGetInvoice_admin_Invoice_services,
} from "../graphql/queries/__generated__/PaymentsGetInvoice";
import GET_OFFER from "../graphql/queries/GetOfferForPayment.graphql";
import GET_SCHEDULE from "../graphql/queries/PaymentsGetAllScheduledPayments.graphql";

type Props = {
  invoice: PaymentsGetInvoice_admin_Invoice;
  isOpenedItem: boolean;
  onComplete: () => {};
};

const ViewItem: FC<Props> = ({ invoice, isOpenedItem, onComplete }) => {
  const [schedulePage, setSchedulePage] = useState(1);
  const PERPAGE = 10;
  const [modalVisible, setModalVisibility] = useState(false);

  const { data, loading, error } = useQuery<
    PaymentsGetAllScheduledPayments,
    PaymentsGetAllScheduledPaymentsVariables
  >(GET_SCHEDULE, {
    variables: {
      filter: [
        {
          fields: ["offer"],
          operator: common_api_generic_query_QueryFilterOperatorEnum.EQUALS,
          value: [invoice.correlation_id],
        },
      ],
      sort: [
        {
          field: "date",
          order: 1,
        },
      ],
      pagination: {
        page: 0,
        perPage: 1000,
      },
    },
    skip: invoice.correlation_type !== "offer",
  });

  const schedule = data?.admin?.allScheduledPayments?.data ?? [];
  const scheduleOnPage = schedule.slice(
    (schedulePage - 1) * PERPAGE,
    schedulePage * PERPAGE
  );
  const products = invoice.order?.products ?? [];

  const {
    data: offerData,
    loading: offerLoading,
    error: offerError,
  } = useQuery<GetOfferForPayment, GetOfferForPaymentVariables>(GET_OFFER, {
    variables: {
      id: invoice.correlation_id,
    },
    skip: invoice.correlation_type !== "offer",
  });

  const offer = offerData?.admin?.Offer;

  return (
    <div>
      {modalVisible && (
        <AssignAdminModal
          invoiceId={invoice.invoice_id}
          setModalVisibility={setModalVisibility}
        />
      )}
      <Collapse opened={isOpenedItem}>
        {({ toggle, isOpened }) => (
          <PaymentConfirmed>
            <PaymentHead>
              {isOpened ? (
                <PaymentClose onClick={toggle} />
              ) : (
                <PaymentOpen onClick={toggle} />
              )}
              <PaymentDescription>
                Invoice ID <b>{invoice.invoice_id}</b>
              </PaymentDescription>
              <PaymentDescription>
                Reference <b>{invoice.reference}</b>
                <br />
                Xero code <b>{invoice.xero_code ?? "-"}</b>
              </PaymentDescription>
              <PaymentCreatedAt>
                {formatDateTime(invoice.created_at)}
              </PaymentCreatedAt>
            </PaymentHead>

            {isOpened && (
              <PaymentBody>
                <PaymentRow>
                  <ColumnWide>
                    <ColumnTitle>Property</ColumnTitle>
                    <ColumnValue>
                      {products.length > 0 ? (
                        <>
                          {propertyAddressFilter(
                            products[0].details.property.location
                          )}
                          <br />
                          {products[0].details.property.id && (
                            <Link
                              className="link"
                              to={`/listing/${products[0].details.property.id}/edit`}
                            >
                              ...{products[0].details.property.id?.substr(-12)}
                            </Link>
                          )}
                        </>
                      ) : (
                        "—"
                      )}
                    </ColumnValue>
                  </ColumnWide>
                  <ColumnNarrow>
                    <ColumnTitle>Payer</ColumnTitle>
                    <ColumnValue>
                      {invoice.order?.userid ? (
                        <Link
                          className="link"
                          to={`/users/${invoice.order.userid}/view`}
                        >
                          {invoice.order.userdata.details.first_name}&nbsp;
                          {invoice.order.userdata.details.last_name}
                        </Link>
                      ) : (
                        "—"
                      )}
                    </ColumnValue>
                  </ColumnNarrow>
                </PaymentRow>

                {offerLoading && <Spinner />}
                {offerError && <SystemError error={offerError} />}
                {offer && (
                  <>
                    <PaymentRow>
                      <ColumnNarrow>
                        <ColumnTitle>Offer</ColumnTitle>
                        <ColumnValue>{invoice.correlation_id}</ColumnValue>
                      </ColumnNarrow>
                      <ColumnNarrow>
                        <ColumnTitle>Current stage</ColumnTitle>
                        <ColumnValue>{OFFER_STAGE[offer.state]}</ColumnValue>
                      </ColumnNarrow>
                      <ColumnNarrow>
                        <ColumnTitle>IVR</ColumnTitle>
                        <ColumnValue>{offer.property.ivr}</ColumnValue>
                      </ColumnNarrow>
                    </PaymentRow>
                    <PaymentRow>
                      <ColumnNarrow>
                        <ColumnTitle>Rent amount</ColumnTitle>
                        <ColumnValue>
                          {formatPricePcm(offer.conditions.price)}
                        </ColumnValue>
                      </ColumnNarrow>
                      <ColumnNarrow>
                        <ColumnTitle>Move in date</ColumnTitle>
                        <ColumnValue>
                          {formatDateWordTime(offer.conditions.movein_date)}
                        </ColumnValue>
                      </ColumnNarrow>
                      <ColumnNarrow>
                        <ColumnTitle>Deposit type</ColumnTitle>
                        <ColumnValue>
                          {offer.conditions.deposit_type ===
                          offer_api_DepositTypeEnum.DEPOSIT_REPLACEMENT
                            ? "Deposit replacement"
                            : "5 weeks worth of rent"}
                        </ColumnValue>
                      </ColumnNarrow>
                    </PaymentRow>
                  </>
                )}

                <PaymentRow>
                  <ColumnNarrow>
                    <ColumnTitle>ASSIGNED TO</ColumnTitle>
                    <ColumnValue>
                      {invoice.administrator
                        ? `${invoice.administrator.details?.first_name} ${invoice.administrator.details?.last_name}`
                        : "N/A"}
                      <ClickableText
                        onClick={() => setModalVisibility(true)}
                        text="Reassign"
                      />
                    </ColumnValue>
                  </ColumnNarrow>
                </PaymentRow>

                {invoice.order && (
                  <>
                    <ColumnTitle>Order data</ColumnTitle>
                    <PaymentRow>
                      <ColumnNarrow>
                        <ColumnTitle>Name</ColumnTitle>
                        <ColumnValue>
                          {invoice.order.userdata.details.first_name}&nbsp;
                          {invoice.order.userdata.details.last_name}
                        </ColumnValue>
                      </ColumnNarrow>
                      <ColumnNarrow>
                        <ColumnTitle>Phone</ColumnTitle>
                        <ColumnValue>
                          {invoice.order.userdata.contacts.phone_primary ?? "—"}
                        </ColumnValue>
                      </ColumnNarrow>
                      <ColumnNarrow>
                        <ColumnTitle>Email</ColumnTitle>
                        <ColumnValue>
                          {invoice.order.userdata.contacts.email_primary ?? "—"}
                        </ColumnValue>
                      </ColumnNarrow>
                    </PaymentRow>
                    <PaymentRow>
                      <ColumnNarrow>
                        <ColumnTitle>Correspondence address</ColumnTitle>
                        <ColumnValue>
                          {invoice.order.userdata.correspondence.postcode
                            ? propertyAddressFilter(
                                invoice.order.userdata.correspondence
                              )
                            : "—"}
                        </ColumnValue>
                      </ColumnNarrow>
                      <ColumnNarrow>
                        <ColumnTitle>Rent Plan</ColumnTitle>
                        <ColumnValue>
                          {!!offer?.rent_plan
                            ? RENT_PLAN_PACKAGES[offer.rent_plan]
                            : "-"}
                        </ColumnValue>
                      </ColumnNarrow>
                    </PaymentRow>
                  </>
                )}

                {invoice.payments && (
                  <>
                    <ColumnTitle>Payments</ColumnTitle>
                    {invoice.payments.length > 0 ? (
                      <Table>
                        <PaymentsHead />
                        <Tbody>
                          {invoice.payments.map((p) => (
                            <PaymentsItem key={p.payment_id} payment={p} />
                          ))}
                        </Tbody>
                      </Table>
                    ) : (
                      <div style={{ marginBottom: "24px" }}>
                        No payments available
                      </div>
                    )}
                  </>
                )}

                {invoice.services && (
                  <>
                    <ColumnTitle>Services</ColumnTitle>
                    <Table>
                      <ServicesHead />
                      <Tbody>
                        {invoice.services.map((s) => (
                          <ServicesItem
                            key={s.service.id}
                            hasPayments={invoice?.payments?.length !== 0}
                            invoice_id={invoice.invoice_id}
                            onComplete={onComplete}
                            service={s}
                          />
                        ))}
                      </Tbody>
                    </Table>
                  </>
                )}

                {loading ? (
                  <Spinner />
                ) : error ? (
                  <SystemError error={error} />
                ) : (
                  schedule.length > 0 && (
                    <>
                      <ColumnTitle>Schedule</ColumnTitle>
                      {scheduleOnPage.map((s) => (
                        <Fragment key={s.id}>
                          <ColumnTitle>
                            ID:{" "}
                            <span style={{ fontWeight: 300, marginRight: 20 }}>
                              {s.id}
                            </span>{" "}
                            Date:{" "}
                            <span style={{ fontWeight: 300 }}>
                              {format(
                                new Date(s.date.year, s.date.month - 1),
                                "MMM yyyy"
                              )}
                            </span>
                          </ColumnTitle>
                          <Table>
                            {(s.parts ?? []).length > 0 && <ScheduleHead />}
                            <Tbody>
                              {(s.parts ?? []).map((p) => (
                                <ScheduleItem
                                  key={p.payment_id}
                                  schedulePart={p}
                                />
                              ))}
                            </Tbody>
                          </Table>
                        </Fragment>
                      ))}
                      <Pagebar
                        changePage={(n) => setSchedulePage(n)}
                        curPage={schedulePage}
                        showNPages={3}
                        total={Math.ceil((schedule.length ?? 0) / 10)}
                      />
                    </>
                  )
                )}
              </PaymentBody>
            )}
          </PaymentConfirmed>
        )}
      </Collapse>
    </div>
  );
};

export default ViewItem;

const PaymentsHead: FC = () => (
  <Thead>
    <Tr>
      <Th>Payment ID</Th>
      <Th>Created</Th>
      <Th>Updated</Th>
      <Th>Amount</Th>
      <Th>Status</Th>
    </Tr>
  </Thead>
);

const PaymentsItem: FC<{
  payment: PaymentsGetInvoice_admin_Invoice_payments;
}> = ({ payment }) => (
  <Tr>
    <Td>{payment.payment_id}</Td>
    <Td>{formatDateTime(payment.created_at)}</Td>
    <Td>{formatDateTime(payment.updated_at)}</Td>
    <Td>
      <b>{formatPrice(payment.amount)}</b>
    </Td>
    <Td>{sentenceCase(payment.status)}</Td>
  </Tr>
);

const ServicesHead: FC = () => (
  <Thead>
    <Tr>
      <Th>Type</Th>
      <Th>Code</Th>
      <Th>Inventory&nbsp;Code</Th>
      <Th>Price</Th>
      <Th>Updated</Th>
      <Th>Status</Th>
    </Tr>
  </Thead>
);

const ServicesItem: FC<{
  hasPayments: boolean;
  service: PaymentsGetInvoice_admin_Invoice_services;
  invoice_id: string;
  onComplete: () => {};
}> = ({ hasPayments, service, invoice_id, onComplete }) => {
  const [confirmPayment, { loading }] = useMutation<
    UpdateInvoiceItems,
    UpdateInvoiceItemsVariables
  >(UPDATE_INVOICE);

  const [createPayment, { loading: createPaymentLoading }] = useMutation<
    CreatePay,
    CreatePayVariables
  >(CREATE_PAYMENT);

  const handleConfirmPayment = useCallback(() => {
    confirmPayment({
      variables: {
        invoice_id,
        statuses: [
          {
            status: extras_api_ServiceStatusEnum.ACTIVE,
            item_id: service.service.id,
          },
        ],
      },
    }).then(
      (res) => res.data?.admin?.updateInvoiceItems?.invoice_id && onComplete()
    );
  }, [confirmPayment, invoice_id, onComplete, service.service.id]);

  const handleCreatePayment = useCallback(() => {
    createPayment({
      variables: {
        invoiceId: invoice_id,
        status: invoice_api_PaymentStatusEnum.PAYMENT_SUCCESS,
      },
    }).then(
      (res) =>
        res.data?.admin?.createPayment?.payment.invoice_id && onComplete()
    );
  }, [createPayment, invoice_id, onComplete]);

  return (
    <Tr>
      <Td>{sentenceCase(service.service.type)}</Td>
      <Td>{sentenceCase(service.service.code)}</Td>
      <Td>{service.service.inventory_code}</Td>
      <Td>
        <b>{formatPrice(service.service.price + service.service.tax_amount)}</b>
      </Td>
      <Td>{formatDateTime(service.updated_at)}</Td>
      <Td>
        {hasPayments && (
          <>
            {service.status === extras_api_ServiceStatusEnum.PENDING ? (
              <PrimaryButton disabled={loading} onClick={handleConfirmPayment}>
                Confirm
              </PrimaryButton>
            ) : service.status === extras_api_ServiceStatusEnum.ACTIVE ? (
              "Confirmed"
            ) : (
              sentenceCase(service.status)
            )}
          </>
        )}
        {!hasPayments && (
          <PrimaryButton
            disabled={createPaymentLoading}
            onClick={handleCreatePayment}
          >
            Confirm
          </PrimaryButton>
        )}
      </Td>
    </Tr>
  );
};

const ScheduleHead: FC = () => (
  <Thead>
    <Tr>
      <Th>Due date</Th>
      <Th>Status</Th>
      <Th>Code</Th>
      <Th>Amount</Th>
    </Tr>
  </Thead>
);

const ScheduleItem: FC<{
  schedulePart: ScheduledPaymentOutputFields_parts;
}> = ({ schedulePart }) => (
  <Tr>
    <Td>{formatDate(schedulePart.payment_due)}</Td>
    <Td>{sentenceCase(schedulePart.payment_status)}</Td>
    <Td>{schedulePart.services && schedulePart.services[0].code}</Td>
    <Td>
      {schedulePart.services && formatPrice(schedulePart.services[0].price)}
    </Td>
  </Tr>
);
