import React, { FC, useState } from "react";

import { Query } from "react-apollo";
import { Field, Form } from "react-final-form";

import { common_api_generic_query_QueryParamsFilterInput as ParamsFilterInput } from "../../globalTypes";
import { PrimaryButton, SecondaryButton } from "../../ui/atoms/Button";
import {
  FilterSort,
  FilterSortGroup,
  FilterSortItem,
} from "../../ui/atoms/FilterSort";
import { TextInput } from "../../ui/atoms/Input";
import { SearchFieldWide } from "../../ui/atoms/SearchField";
import Spinner from "../../ui/atoms/Spinner";
import SystemError from "../../ui/atoms/SystemError";
import { Table, Tbody } from "../../ui/atoms/Table";
import TableOverlay from "../../ui/atoms/TableOverlay";
import TableWrapper from "../../ui/atoms/TableWrapper";
import Dropdown from "../../ui/molecules/Dropdown";
import Pagebar from "../../ui/molecules/Pagebar";
import Pagination from "../../ui/molecules/Pagination";
import Sortable from "../../ui/molecules/Sortable";
import { wrapInputEmptyOption } from "../../util/dropdown";
import getSortOrder from "../../util/ger_sort_order";
import { makeFilters } from "../../util/search_filters";
import { validRanges } from "../../util/validations";
import { InvoicePaymentStatusOptions } from "./constants";
import {
  PaymentsGetAllPayments,
  PaymentsGetAllPaymentsVariables,
} from "./graphql/queries/__generated__/PaymentsGetAllPayments";
import GET_ALL_PAYMENTS from "./graphql/queries/PaymentsGetAllPayments.graphql";
import ListHead from "./ListHead";
import ListItem from "./ListItem";

const List: FC = () => {
  const [filter, setFilter] = useState<ParamsFilterInput[]>([]);

  const handleFilterSubmit = (values) => {
    setFilter(
      makeFilters({
        values,
        filters: [
          {
            fields: ["id", "reference", "property", "userid", "ivr"],
            key: "search",
            operator: "CONTAINS_ANY",
            transformation: {
              preprocessor: "REGEXP_REPLACE",
              param: ["[- /]", ""],
            },
          },
          {
            fields: ["status"],
            key: "status.value",
            operator: "EQUALS",
          },
          {
            fields: ["created"],
            key: "date",
            operator: "EQUALS",
          },
          {
            fields: ["amount"],
            key: "min,max",
            operator: "RANGE",
          },
        ],
      })
    );
  };

  const handleFilterReset = (cb) => {
    cb();
    setFilter([]);
  };

  return (
    <>
      <h2>View Payments</h2>

      <Form
        onSubmit={handleFilterSubmit}
        render={({
          handleSubmit,
          submitting,
          valid,
          values,
          form,
          pristine,
        }) => (
          <form onSubmit={handleSubmit}>
            <SearchFieldWide>
              <Field
                component={TextInput}
                name="search"
                placeholder="Enter Reference number, Payment ID, Payer ID, Property ID or ivr"
              />
              <PrimaryButton
                disabled={!valid || submitting || pristine}
                onClick={() => handleFilterSubmit(values)}
                type="submit"
              >
                Search
              </PrimaryButton>
              {!(!valid || submitting || pristine) && (
                <>
                  &nbsp;&nbsp;
                  <SecondaryButton
                    onClick={() => handleFilterReset(form.reset)}
                    type="reset"
                  >
                    Clear
                  </SecondaryButton>
                </>
              )}
            </SearchFieldWide>
            <FilterSort>
              <FilterSortItem>
                <Field
                  name="date"
                  render={(props) => (
                    <TextInput className="input-sort-date" {...props} />
                  )}
                  type="date"
                />
              </FilterSortItem>
              <FilterSortItem>
                <Field
                  name="status"
                  render={({ input }) => {
                    return (
                      <Dropdown
                        input={wrapInputEmptyOption(input)}
                        options={InvoicePaymentStatusOptions}
                        placeholder="Status"
                      />
                    );
                  }}
                />
              </FilterSortItem>
              <FilterSortItem>
                <label htmlFor="min">Amount (&pound;):</label>
                <FilterSortGroup>
                  <Field
                    className="input-amount"
                    component={TextInput}
                    name="min"
                    placeholder="from"
                    validate={validRanges("max")}
                  />
                  &nbsp;&ndash;&nbsp;
                  <Field
                    className="input-amount"
                    component={TextInput}
                    name="max"
                    placeholder="to"
                    validate={validRanges("min")}
                  />
                </FilterSortGroup>
              </FilterSortItem>

              <FilterSortItem />
            </FilterSort>
          </form>
        )}
      />

      <Sortable field={["created"]} order={-1}>
        {({ field, order, setSort }) => (
          <Pagination filter={filter} perPage={25}>
            {({ page, setPage, perPage }) => {
              const getOrder = (fieldName) =>
                getSortOrder({ field, fieldName, order });
              const onSetPage = (pageNumber: number) => {
                setPage(pageNumber);
              };

              return (
                <Query<PaymentsGetAllPayments, PaymentsGetAllPaymentsVariables>
                  query={GET_ALL_PAYMENTS}
                  variables={{
                    pagination: { page, perPage },
                    sort: field.map((v) => ({ field: v, order })),
                    filter,
                  }}
                >
                  {({ data, loading, error }) => {
                    if (error) {
                      return <SystemError error={error} />;
                    } else if (loading) {
                      return <Spinner />;
                    }

                    const allPayments =
                      data?.admin?.allPayments?.payments ?? [];
                    const totalRows = data?.admin?.allPayments?.total ?? 0;

                    return (
                      <TableWrapper>
                        {!1 && <TableOverlay />}

                        <Pagebar
                          changePage={onSetPage}
                          curPage={page}
                          showNPages={3}
                          total={Math.ceil(totalRows / perPage)}
                        />

                        <Table>
                          <ListHead
                            field={field}
                            getOrder={getOrder}
                            order={order}
                            setSort={setSort}
                          />
                          <Tbody>
                            {allPayments.map((v) => (
                              <ListItem key={v.payment_id} payment={v} />
                            ))}
                          </Tbody>
                        </Table>
                        <Pagebar
                          changePage={onSetPage}
                          curPage={page}
                          showNPages={3}
                          total={Math.ceil(totalRows / perPage)}
                        />
                      </TableWrapper>
                    );
                  }}
                </Query>
              );
            }}
          </Pagination>
        )}
      </Sortable>
    </>
  );
};

export default List;
