import React, { FC, useMemo } from "react";

import { useMutation } from "react-apollo";
import { sentenceCase } from "sentence-case";

import {
  invoice_api_PaymentStatusEnum,
  offer_api_PayoutStateEnum,
} from "../../../../globalTypes";
import { PrimaryButton, SecondaryButton } from "../../../../ui/atoms/Button";
import {
  ColumnTitle,
  ColumnValue,
  PaymentRow,
} from "../../../../ui/atoms/Payment";
import { StatusActive, StatusPending } from "../../../../ui/atoms/Status";
import SystemError from "../../../../ui/atoms/SystemError";
import { formatPrice } from "../../../../util/price";
import {
  CreatePayment,
  CreatePaymentVariables,
} from "../../graphql/mutations/__generated__/CreatePayment";
import {
  UpdatePaymentStatus,
  UpdatePaymentStatusVariables,
} from "../../graphql/mutations/__generated__/UpdatePaymentStatus";
import CREATE_PAYMENT from "../../graphql/mutations/CreatePayment.graphql";
import UPDATE_PAYMENT_STATUS from "../../graphql/mutations/UpdatePaymentStatus.graphql";
import {
  PaymentsGetAllScheduledPayments_admin_allScheduledPayments_data as AllScheduledPayments,
  PaymentsGetAllScheduledPayments_admin_allScheduledPayments_data_payout_invoice_payments as AllScheduledPayoutPayment,
} from "../../graphql/queries/__generated__/PaymentsGetAllScheduledPayments";
import { PaymentsGetOfferForTenancy_admin_Offer } from "../../graphql/queries/__generated__/PaymentsGetOfferForTenancy";
import CopyButton from "../CopyButton";

import css from "./index.module.css";

enum PAYOUT_RESULT {
  TO_LANDLORD,
  TO_MASHROOM,
  PAID,
}

const getPayoutResultStatus = (scheduledPayment: AllScheduledPayments) => {
  if (scheduledPayment.payout.state === offer_api_PayoutStateEnum.PAYOUT_PAID) {
    return PAYOUT_RESULT.PAID;
  } else if (scheduledPayment.advance.available) {
    return PAYOUT_RESULT.TO_MASHROOM;
  } else {
    return PAYOUT_RESULT.TO_LANDLORD;
  }
};

type PayoutProps = {
  offer: PaymentsGetOfferForTenancy_admin_Offer;
  payment: AllScheduledPayoutPayment | null | undefined;
  scheduledPayment: AllScheduledPayments;
  ivr: string | null;
};

const Payout: FC<PayoutProps> = ({ offer, payment, scheduledPayment, ivr }) => {
  const payoutStatus = getPayoutResultStatus(scheduledPayment);

  const textForCopy = useMemo(
    () =>
      payoutStatus === PAYOUT_RESULT.TO_MASHROOM
        ? `20907413567761${payment?.amount},"",${ivr} RENT M${scheduledPayment.date.month},MASHROOM LTD,""`
        : `${(offer.landlord.bank_details?.sort_code ?? "").replace(
            /-/g,
            ""
          )}${(offer.landlord.bank_details?.account_number ?? "").replace(
            /-/g,
            ""
          )}${payment?.amount},"",${ivr} RENT M${scheduledPayment.date.month},${
            offer.landlord.first_name
          } ${offer.landlord.last_name},""`,
    [payoutStatus, payment, ivr, scheduledPayment, offer.landlord]
  );

  const [
    createPayment,
    { loading: createPaymentLoading, error: createPaymentError },
  ] = useMutation<CreatePayment, CreatePaymentVariables>(CREATE_PAYMENT);

  const [
    updatePayment,
    { loading: updatePaymentLoading, error: updatePaymentError },
  ] = useMutation<UpdatePaymentStatus, UpdatePaymentStatusVariables>(
    UPDATE_PAYMENT_STATUS
  );

  const handleUpdateStatus = (status: invoice_api_PaymentStatusEnum) => {
    if (
      payment &&
      payment.status !== invoice_api_PaymentStatusEnum.PAYMENT_FAILED &&
      payment.status !== invoice_api_PaymentStatusEnum.PAYMENT_CANCELED
    ) {
      updatePayment({
        variables: {
          payment_id: payment.id,
          status,
        },
        refetchQueries: ["PaymentsGetAllScheduledPayments"],
      });
    } else {
      createPayment({
        variables: {
          invoice_id: scheduledPayment.payout.invoice_id,
          status,
        },
        refetchQueries: ["PaymentsGetAllScheduledPayments"],
      });
    }
  };

  return (
    <>
      <h3 className={css.subTitle}>
        Payout&nbsp;&nbsp;
        {scheduledPayment.payout.state ===
        offer_api_PayoutStateEnum.PAYOUT_PAID ? (
          <StatusActive>Paid</StatusActive>
        ) : scheduledPayment.payout.state ===
          offer_api_PayoutStateEnum.PAYOUT_CONFIRMED ? (
          <StatusActive>Confirmed</StatusActive>
        ) : (
          <StatusPending>Pending</StatusPending>
        )}
      </h3>
      <div className={css.blockContainer}>
        {payoutStatus === PAYOUT_RESULT.TO_LANDLORD && (
          <div className={css.blockText}>
            The payment needs to be reconciled before you can transfer the rent
            out of our client account
          </div>
        )}
        {scheduledPayment.payout.invoice_id === "" && (
          <div className={css.blockText}>
            <i>Payout invoice was not found</i>
          </div>
        )}
        <PaymentRow>
          <div className={css.column20}>
            <ColumnTitle>Amount owed</ColumnTitle>
            <ColumnValue>{formatPrice(payment?.amount)}</ColumnValue>
          </div>
          <div className={css.column20}>
            <ColumnTitle>Payment Status</ColumnTitle>
            <ColumnValue>
              {payment ? sentenceCase(payment.status) : "Not found"}
            </ColumnValue>
          </div>
          {payoutStatus === PAYOUT_RESULT.TO_MASHROOM ? (
            <>
              <div className={css.column20}>
                <ColumnTitle>Recipient</ColumnTitle>
                <ColumnValue>Mashroom</ColumnValue>
              </div>
              <div className={css.column20}>
                <ColumnTitle>Sort code</ColumnTitle>
                <ColumnValue>20-90-74</ColumnValue>
              </div>
              <div className={css.column20}>
                <ColumnTitle>Account number</ColumnTitle>
                <ColumnValue>13567761</ColumnValue>
              </div>
            </>
          ) : (
            <>
              <div className={css.column20}>
                <ColumnTitle>Landlord name</ColumnTitle>
                <ColumnValue>
                  {offer.landlord.first_name} {offer.landlord.last_name}
                </ColumnValue>
              </div>
              <div className={css.column20}>
                <ColumnTitle>Sort code</ColumnTitle>
                <ColumnValue>
                  {offer.landlord.bank_details?.sort_code}
                </ColumnValue>
              </div>
              <div className={css.column20}>
                <ColumnTitle>Account number</ColumnTitle>
                <ColumnValue>
                  {offer.landlord.bank_details?.account_number}
                </ColumnValue>
              </div>
            </>
          )}
        </PaymentRow>
        <div className={css.blockButtons}>
          <CopyButton textForCopy={textForCopy}>Copy payout details</CopyButton>
          {payment?.status === invoice_api_PaymentStatusEnum.PAYMENT_SUCCESS ? (
            <SecondaryButton
              disabled={updatePaymentLoading}
              onClick={() =>
                handleUpdateStatus(
                  invoice_api_PaymentStatusEnum.PAYMENT_CANCELED
                )
              }
              type="button"
            >
              Mark as unpaid
            </SecondaryButton>
          ) : scheduledPayment.payout.invoice_id !== "" ? (
            <PrimaryButton
              disabled={updatePaymentLoading || createPaymentLoading}
              onClick={() =>
                handleUpdateStatus(
                  invoice_api_PaymentStatusEnum.PAYMENT_SUCCESS
                )
              }
              type="button"
            >
              Mark as paid
            </PrimaryButton>
          ) : null}
        </div>
        {createPaymentError && <SystemError error={createPaymentError} />}
        {updatePaymentError && <SystemError error={updatePaymentError} />}
      </div>
    </>
  );
};

export default Payout;
