import React, { FC, useMemo } from "react";

import cn from "classnames";
import { Link } from "react-router-dom";

import { offer_api_DepositTypeEnum } from "../../../../globalTypes";
import {
  PropertyCardItemTitle,
  PropertyCardItemValue,
  PropertyContent,
  PropertyGrid,
  PropertyPhoto,
} from "../../../../ui/atoms/Property";
import PropertyPhotoPlaceholder from "../../../../ui/atoms/PropertyPhotoPlaceholder";
import { formatPrice } from "../../../../util/price";
import { propertyAddressFilter } from "../../../listing/filters";
import { OfferForTenancyOutputFields } from "../../graphql/queries/__generated__/OfferForTenancyOutputFields";
import { PropertyDataOutputFields } from "../../graphql/queries/__generated__/PropertyDataOutputFields";

import css from "./index.module.css";

type Props = {
  property: PropertyDataOutputFields;
  offer: OfferForTenancyOutputFields | null | undefined;
};

const PropertyHead: FC<Props> = ({ property, offer }) => {
  const { people } = property;
  const propertyLocation = propertyAddressFilter(property.location);
  const propertyImage = useMemo(() => {
    const propertyPhotos =
      property.description.photos?.filter((p) => !p.deleted_on) ?? [];
    return propertyPhotos.length > 0
      ? `/media/thumb-${propertyPhotos[0].id}`
      : undefined;
  }, [property.description.photos]);

  return (
    <>
      {propertyImage ? (
        <PropertyPhoto alt={propertyLocation} src={propertyImage} />
      ) : (
        <PropertyPhotoPlaceholder />
      )}
      <PropertyContent>
        <h3 className={css.title}>{propertyLocation}</h3>
        <PropertyGrid>
          <div className={css.propertyCardItem}>
            <PropertyCardItemTitle>Agreed rent</PropertyCardItemTitle>
            <PropertyCardItemValue>
              {formatPrice(offer?.conditions.price)}
            </PropertyCardItemValue>
          </div>
          <div className={css.propertyCardItem}>
            <PropertyCardItemTitle>
              Rent payment frequency
            </PropertyCardItemTitle>
            <PropertyCardItemValue>
              {offer
                ? offer.conditions.payment_frequency === 1
                  ? "Monthly"
                  : `Every ${offer.conditions.payment_frequency} month(s)`
                : ""}
            </PropertyCardItemValue>
          </div>
          <div className={css.propertyCardItem}>
            <PropertyCardItemTitle>Deposit type</PropertyCardItemTitle>
            <PropertyCardItemValue>
              {offer
                ? offer.conditions.deposit_type ===
                  offer_api_DepositTypeEnum.DEPOSIT_TRADITIONAL_5WEEK
                  ? "Traditional"
                  : "Deposit Replacement"
                : ""}
            </PropertyCardItemValue>
          </div>
          <div className={css.propertyCardItem}>
            <PropertyCardItemTitle>Viewing host</PropertyCardItemTitle>
            <PropertyCardItemValue>
              {people.landlord?.contact?.email_primary ===
              property.people.manager?.contact?.email_primary
                ? "No team up"
                : "Team Up Reward Eligible"}
            </PropertyCardItemValue>
          </div>
          <div className={cn(css.propertyCardItem, css.width25)}>
            <PropertyCardItemTitle>Property ID</PropertyCardItemTitle>
            <PropertyCardItemValue>
              <Link className="link" to={`/listing/${property.id}/edit`}>
                {`. . . ${property.id.substr(-6)}`}
              </Link>
            </PropertyCardItemValue>
          </div>
          <div className={cn(css.propertyCardItem, css.width25)}>
            <PropertyCardItemTitle>Offer ID</PropertyCardItemTitle>
            <PropertyCardItemValue>
              {offer ? (
                <Link
                  className="link"
                  to={`/offers/${offer.id}/edit`}
                >{`. . . ${offer.id.substr(-6)}`}</Link>
              ) : (
                ""
              )}
            </PropertyCardItemValue>
          </div>
          <div className={cn(css.propertyCardItem, css.width25)}>
            <PropertyCardItemTitle>IVR Ref.</PropertyCardItemTitle>
            <PropertyCardItemValue>{property.ivr}</PropertyCardItemValue>
          </div>
          <div className={cn(css.propertyCardItem, css.width25)}>
            <PropertyCardItemTitle>Assigned to</PropertyCardItemTitle>
            <PropertyCardItemValue>
              {people.administrator
                ? `${people.administrator.details?.first_name} ${people.administrator.details?.last_name}`
                : "N/A"}
            </PropertyCardItemValue>
          </div>
        </PropertyGrid>
      </PropertyContent>
    </>
  );
};

export default PropertyHead;
