import React, { FC } from "react";

import { useMutation } from "react-apollo";
import { Field, Form } from "react-final-form";

import {
  document_api_DocumentStateEnum,
  document_api_DocumentTypeEnum,
} from "../../../../globalTypes";
import { PrimaryButton, SecondaryButton } from "../../../../ui/atoms/Button";
import { ColumnTitle } from "../../../../ui/atoms/Payment";
import SystemError from "../../../../ui/atoms/SystemError";
import {
  ButtonsGroup,
  UserFormRow,
  UserFormRowFlex,
} from "../../../../ui/atoms/UserForm";
import Dropdown from "../../../../ui/molecules/Dropdown";
import { FileField } from "../../../../ui/molecules/FileField/FileField";
import { InputFieldTouched } from "../../../../ui/molecules/InputField";
import { ModalWide } from "../../../../ui/molecules/Modal";
import { enumToList, formatGenericEnumValue } from "../../../../util/enums";
import { required } from "../../../../util/validations";
import {
  PaymentsCreateDocument,
  PaymentsCreateDocumentVariables,
} from "../../graphql/mutations/__generated__/PaymentsCreateDocument";
import CREATE_DOCUMENT from "../../graphql/mutations/PaymentsCreateDocument.graphql";

import css from "./index.module.css";

type Props = {
  onClose: () => void;
  onComplete: () => void;
  propertyId: string;
};

interface FormValues {
  file: string;
  state: document_api_DocumentStateEnum;
  expiration?: string;
  description?: string;
}

const UploadRegistrationModal: FC<Props> = ({
  onClose,
  onComplete,
  propertyId,
}) => {
  const [createDocument, { loading, error }] = useMutation<
    PaymentsCreateDocument,
    PaymentsCreateDocumentVariables
  >(CREATE_DOCUMENT);

  function handleSubmit(values: FormValues) {
    const { expiration, ...rest } = values;
    createDocument({
      variables: {
        type: document_api_DocumentTypeEnum.DEPOSIT_REGISTRATION_CERTIFICATE,
        property: propertyId,
        expiration: expiration ? new Date(expiration).toUTCString() : undefined,
        ...rest,
      },
    }).then((res) => {
      if (res.data?.admin?.createDocument?.success) {
        onComplete();
      }
    });
  }

  return (
    <ModalWide
      onCancel={onClose}
      title="Add Registration Confirmation document"
    >
      <Form<FormValues> onSubmit={handleSubmit}>
        {({ handleSubmit, hasValidationErrors }) => (
          <form onSubmit={handleSubmit}>
            <Field
              name="file"
              render={({ input, meta }) => {
                return (
                  <FileField
                    acceptedTypes={acceptedFileTypes}
                    error={meta.touched && meta.error}
                    fileId={input.value}
                    handleChange={input.onChange}
                  />
                );
              }}
              validate={required()}
            />
            <UserFormRowFlex>
              <div className={css.column}>
                <ColumnTitle>Document state</ColumnTitle>
                <Field
                  name="state"
                  render={({ input, meta }) => (
                    <Dropdown
                      error={meta.touched && meta.error}
                      input={input}
                      itemToString={formatGenericEnumValue}
                      options={enumToList(document_api_DocumentStateEnum)}
                      placeholder="-- select --"
                    />
                  )}
                  validate={required("Required field")}
                />
              </div>
              <div className={css.column}>
                <ColumnTitle>Expire date</ColumnTitle>
                <Field
                  name="expiration"
                  render={({ input, meta }) => (
                    <InputFieldTouched input={input} meta={meta} />
                  )}
                  type="date"
                />
              </div>
            </UserFormRowFlex>

            <UserFormRow>
              <ColumnTitle>Description</ColumnTitle>
              <Field component={InputFieldTouched} name="description" />
            </UserFormRow>

            {error && <SystemError error={error} />}

            <ButtonsGroup>
              <SecondaryButton onClick={onClose} type="button">
                Cancel
              </SecondaryButton>
              <PrimaryButton
                disabled={hasValidationErrors || loading}
                type="submit"
              >
                Add
              </PrimaryButton>
            </ButtonsGroup>
          </form>
        )}
      </Form>
    </ModalWide>
  );
};

export default UploadRegistrationModal;

const acceptedFileTypes = ["application/pdf", "image/jpeg", "image/png"];
