import React, { FC, useState } from "react";

import { useQuery } from "react-apollo";
import { Field, Form } from "react-final-form";

import {
  common_api_generic_query_QueryFilterOperatorEnum as OperatorEnum,
  common_api_generic_query_QueryParamsFilterInput as ParamsFilterInput,
  invoice_api_InvoiceStatusEnum,
  invoice_api_InvoiceTypeEnum,
} from "../../globalTypes";
import { PrimaryButton, SecondaryButton } from "../../ui/atoms/Button";
import {
  FilterSort,
  FilterSortGroup,
  FilterSortItem,
} from "../../ui/atoms/FilterSort";
import { TextInput } from "../../ui/atoms/Input";
import { SearchFieldWide } from "../../ui/atoms/SearchField";
import Spinner from "../../ui/atoms/Spinner";
import { Table, Tbody } from "../../ui/atoms/Table";
import TableWrapper from "../../ui/atoms/TableWrapper";
import Dropdown from "../../ui/molecules/Dropdown";
import Pagebar from "../../ui/molecules/Pagebar";
import { usePagination } from "../../ui/molecules/Pagination";
import { useSortable } from "../../ui/molecules/Sortable";
import getSortOrder from "../../util/ger_sort_order";
import {
  PayoutsGetAllInvoices,
  PayoutsGetAllInvoicesVariables,
} from "./graphql/queries/__generated__/PayoutsGetAllInvoices";
import GET_ALL_INVOICES from "./graphql/queries/PayoutsGetAllInvoices.graphql";
import ListHead from "./ListHead";
import ListItem from "./ListItem";
import { makeFilters } from "./search_filters";

interface FormValues {
  search: string;
  date: string;
  paymentDateMin: string;
  paymentDateMax: string;
  payoutDateMin: string;
  payoutDateMax: string;
  paymentType: {
    label: string;
    value: string;
  };
}

const PAYMENT_TYPES_OPTIONS = [
  {
    label: "Rent",
    value: "rent-payout",
  },
  {
    label: "Others",
    value: "rent-payout",
  },
];

const PayoutsMade: FC = () => {
  const { field, order, setSort } = useSortable({
    field: ["updated"],
    order: -1,
  });
  const basicFilter: ParamsFilterInput[] = [
    {
      fields: ["status"],
      operator: OperatorEnum.EQUALS,
      value: [invoice_api_InvoiceStatusEnum.INVOICE_OPEN],
    },
    {
      fields: ["type"],
      operator: OperatorEnum.EQUALS,
      value: [invoice_api_InvoiceTypeEnum.INVOICE_TYPE_PAYOUT],
    },
  ];
  const [filter, setFilter] = useState<ParamsFilterInput[]>(basicFilter);
  const { page, setPage, perPage } = usePagination({ perPage: 25 });

  const { data, loading } = useQuery<
    PayoutsGetAllInvoices,
    PayoutsGetAllInvoicesVariables
  >(GET_ALL_INVOICES, {
    variables: {
      pagination: { page, perPage },
      sort: field.map((v) => ({ field: v, order })),
      filter,
    },
  });

  const getOrder = (fieldName: string) =>
    getSortOrder({ field, fieldName, order });
  const onSetPage = (pageNumber: number) => setPage(pageNumber);

  const invoicesData = data?.admin?.allInvoices;
  const invoices = invoicesData?.invoices ?? [];
  const total = invoicesData?.total ?? 1;

  const handleFilterSubmit = (values: FormValues) => {
    const notPaymentType = values.paymentType?.label === "Others";
    const filters = makeFilters({
      values,
      filters: [
        {
          fields: ["amount", "ivr"],
          key: "search",
          operator: "CONTAINS_ANY",
        },
        {
          fields: ["created"],
          key: "paymentDateMin,paymentDateMax",
          operator: "RANGE",
        },
        {
          fields: ["updated"],
          key: "payoutDateMin,payoutDateMax",
          operator: "RANGE",
        },
        {
          fields: ["correlation_type"],
          key: "paymentType.value",
          operator: OperatorEnum.EQUALS,
          not: notPaymentType,
        },
      ],
    });
    setFilter([...basicFilter, ...filters]);
  };

  return (
    <>
      <h2>Payouts to be made</h2>
      <Form<FormValues> onSubmit={handleFilterSubmit}>
        {({ form, handleSubmit, hasValidationErrors, dirty }) => (
          <form onSubmit={handleSubmit}>
            <SearchFieldWide>
              <Field
                component={TextInput}
                name="search"
                placeholder="Enter ivr, amount"
              />
            </SearchFieldWide>
            <FilterSort>
              <FilterSortItem>
                <label htmlFor="paymentDateMin">Payment date filter:</label>
                <FilterSortGroup>
                  <Field
                    name="paymentDateMin"
                    render={(props) => (
                      <TextInput className="input-sort-date" {...props} />
                    )}
                    type="date"
                  />
                  &nbsp;&ndash;&nbsp;
                  <Field
                    name="paymentDateMax"
                    render={(props) => (
                      <TextInput className="input-sort-date" {...props} />
                    )}
                    type="date"
                  />
                </FilterSortGroup>
              </FilterSortItem>
              <FilterSortItem>
                <label htmlFor="payoutDateMin">Payout date filter:</label>
                <FilterSortGroup>
                  <Field
                    name="payoutDateMin"
                    render={(props) => (
                      <TextInput className="input-sort-date" {...props} />
                    )}
                    type="date"
                  />
                  &nbsp;&ndash;&nbsp;
                  <Field
                    name="payoutDateMax"
                    render={(props) => (
                      <TextInput className="input-sort-date" {...props} />
                    )}
                    type="date"
                  />
                </FilterSortGroup>
              </FilterSortItem>
              <FilterSortItem>
                <label htmlFor="status">Payment type:</label>
                <div>
                  <Field
                    name="paymentType"
                    render={({ input }) => (
                      <Dropdown
                        input={input}
                        options={PAYMENT_TYPES_OPTIONS}
                        placeholder="Choose payment type"
                      />
                    )}
                  />
                </div>
              </FilterSortItem>

              <PrimaryButton disabled={hasValidationErrors} type="submit">
                Search
              </PrimaryButton>
              {dirty && (
                <>
                  &nbsp;&nbsp;
                  <SecondaryButton
                    onClick={() => {
                      form.reset();
                      setFilter(basicFilter);
                    }}
                    type="button"
                  >
                    Clear
                  </SecondaryButton>
                </>
              )}
              <FilterSortItem />
            </FilterSort>
          </form>
        )}
      </Form>
      {loading ? (
        <Spinner />
      ) : (
        <TableWrapper>
          <Pagebar
            changePage={onSetPage}
            curPage={page}
            showNPages={3}
            total={Math.ceil(total / perPage)}
          />
          <Table>
            <ListHead
              field={field}
              getOrder={getOrder}
              order={order}
              setSort={setSort}
            />
            <Tbody>
              {invoices.map(
                (
                  {
                    amount,
                    correlation_type,
                    created_at,
                    invoice_id,
                    ivr,
                    property_data,
                    updated_at,
                  },
                  index
                ) => {
                  const { location } = property_data ?? {};

                  return (
                    <ListItem
                      key={index}
                      address={[
                        location?.flat,
                        location?.building,
                        location?.street,
                        location?.city,
                        location?.postcode?.replace(" ", "\u00A0"),
                      ]
                        .filter(Boolean)
                        .join(", ")}
                      amount={amount}
                      bank_details={property_data?.bank_details}
                      created_at={created_at}
                      invoice_id={invoice_id}
                      ivr={ivr}
                      landlord_details={
                        property_data?.people?.landlord?.details
                      }
                      propertyId={property_data?.id}
                      tenantDetails={
                        property_data?.people?.tenants[0]?.user?.details
                      }
                      type={correlation_type}
                      updated_at={updated_at}
                    />
                  );
                }
              )}
            </Tbody>
          </Table>
          <Pagebar
            changePage={onSetPage}
            curPage={page}
            showNPages={3}
            total={Math.ceil(total / perPage)}
          />
        </TableWrapper>
      )}
    </>
  );
};

export default PayoutsMade;
