import React, { FC, useMemo } from "react";

import { useMutation } from "react-apollo";
import { Field, Form } from "react-final-form";

import { PrimaryButton, SecondaryButton } from "../../../ui/atoms/Button";
import { TextInput } from "../../../ui/atoms/Input";
import { UserFormRow } from "../../../ui/atoms/UserForm";
import ErrorMessage from "../../../ui/molecules/ErrorMessage";
import { Modal } from "../../../ui/molecules/Modal";
import { required } from "../../../util/validations";
import {
  UpdateUser,
  UpdateUserVariables,
} from "../graphql/mutations/__generated__/UpdateUser";
import UPDATE_USER from "../graphql/mutations/updateUser.graphql";
import { GetUserInfo_admin_User_bank_details } from "../graphql/queries/__generated__/GetUserInfo";

import css from "./index.module.css";

interface FormValues {
  account_number: string;
  account_name: string;
  society: string;
  sort_code: string;
}

type Props = {
  userId: string;
  bankDetails: GetUserInfo_admin_User_bank_details;
  onCancel: () => void;
  onSubmit: () => void;
};

const ChangeBankDetailsModal: FC<Props> = ({
  userId,
  bankDetails,
  onCancel,
  onSubmit,
}) => {
  const [updateUserInfo, { loading, error }] = useMutation<
    UpdateUser,
    UpdateUserVariables
  >(UPDATE_USER);

  function handleSubmit(values: FormValues) {
    updateUserInfo({
      variables: {
        account: {
          id: userId,
          bank_details: { ...values },
        },
      },
    }).then((res) => res.data?.admin?.updateUser && onSubmit());
  }

  const initialValues: FormValues = useMemo(
    () => ({
      account_name: bankDetails.account_name,
      account_number: bankDetails.account_number,
      society: bankDetails.society,
      sort_code: bankDetails.sort_code,
    }),
    [bankDetails]
  );

  return (
    <Modal onCancel={onCancel} title="Change bank details" withoutOverflow>
      <Form<FormValues>
        initialValues={initialValues}
        onSubmit={handleSubmit}
        render={({ handleSubmit, hasValidationErrors }) => (
          <form onSubmit={handleSubmit}>
            <UserFormRow>
              <label>Account name</label>
              <Field
                component={TextInput}
                name="account_name"
                validate={required("Value is required")}
              />
              <Field component={ErrorMessage} name="account_name" />
            </UserFormRow>
            <UserFormRow>
              <label>Account number</label>
              <Field
                component={TextInput}
                name="account_number"
                placeholder="e.g. 12345678"
                validate={required("Value is required")}
              />
              <Field component={ErrorMessage} name="account_number" />
            </UserFormRow>
            <UserFormRow>
              <label>Society</label>
              <Field
                component={TextInput}
                name="society"
                placeholder="e.g. Santander"
                validate={required("Value is required")}
              />
              <Field component={ErrorMessage} name="society" />
            </UserFormRow>
            <UserFormRow>
              <label>Sort code</label>
              <Field
                component={TextInput}
                name="sort_code"
                placeholder="e.g. 12-34-56"
                validate={required("Value is required")}
              />
              <Field component={ErrorMessage} name="sort_code" />
            </UserFormRow>

            {error && <div>{error.message}</div>}

            <div className={css.actionButtons}>
              <SecondaryButton onClick={onCancel} type="button">
                Cancel
              </SecondaryButton>
              <PrimaryButton disabled={hasValidationErrors || loading}>
                Save
              </PrimaryButton>
            </div>
          </form>
        )}
      />
    </Modal>
  );
};

export default ChangeBankDetailsModal;
