import React, { useState } from "react";

import styled from "@emotion/styled";
import { format } from "date-fns";
import { Query, useApolloClient } from "react-apollo";
import { Field, Form } from "react-final-form";

import {
  common_api_generic_query_QueryFilterOperatorEnum,
  common_api_generic_query_QueryParamsFilterInput,
} from "../../globalTypes";
import { PrimaryButton, SecondaryButton } from "../../ui/atoms/Button";
import {
  FilterSort,
  FilterSortItem,
  FilterSortItemInner,
} from "../../ui/atoms/FilterSort";
import SystemError from "../../ui/atoms/SystemError";
import { Table, Tbody, Thead } from "../../ui/atoms/Table";
import TableOverlay from "../../ui/atoms/TableOverlay";
import TableWrapper from "../../ui/atoms/TableWrapper";
import Dropdown from "../../ui/molecules/Dropdown";
import FilterSearch from "../../ui/molecules/FilterSearch";
import Pagebar from "../../ui/molecules/Pagebar";
import Pagination from "../../ui/molecules/Pagination";
import Sortable from "../../ui/molecules/Sortable";
import {
  DownloadAllUsers,
  DownloadAllUsersVariables,
} from "./graphql/queries/__generated__/DownloadAllUsers";
import {
  GetUsersList,
  GetUsersListVariables,
} from "./graphql/queries/__generated__/GetUsersList";
import DOWNLOAD_ALL_USERS_QUERY from "./graphql/queries/export-users.graphql";
import ALL_USERS from "./graphql/queries/getList.graphql";
import ListHead from "./ListHead";
import ListItem from "./ListItem";

type UsersFilterState = common_api_generic_query_QueryParamsFilterInput[];

enum USER_TYPE {
  LANDLORDS = "LANDLORDS",
  TENANTS = "TENANTS",
}

const List = () => {
  const [filter, setFilter] = useState<UsersFilterState>([]);
  const client = useApolloClient();

  const handleFilterSubmit = (values) => {
    const newFilter: UsersFilterState = [];

    if (values.search) {
      newFilter.push({
        fields: [
          "details.first_name",
          "details.last_name",
          "contact.email_primary",
          "contact.phone_primary",
        ],
        value: [values.search],
        operator: common_api_generic_query_QueryFilterOperatorEnum.EQUALS,
      });
    }

    if (values.userType) {
      newFilter.push({
        operator: common_api_generic_query_QueryFilterOperatorEnum.MORE,
        fields:
          values.userType.value === USER_TYPE.LANDLORDS
            ? ["resources.owned_properties"]
            : ["resources.viewing_requests"],
        value: ["1"],
      });
    }

    setFilter(newFilter);
  };

  const handleFilterReset = (cb: () => void) => {
    cb();
    setFilter([]);
  };

  const handleDownloadExport = () => {
    client
      .query<DownloadAllUsers, DownloadAllUsersVariables>({
        query: DOWNLOAD_ALL_USERS_QUERY,
      })
      .then((data) => {
        if (data?.data?.admin?.downloadAllUsersCsv?.success) {
          const csv = data!.data!.admin!.downloadAllUsersCsv.file;

          const link = document.createElement("a");
          link.href = `/media/${csv}`;
          link.setAttribute(
            "download",
            `users_${format(new Date(), "dd-MM-yyyy-HH-mm-ss")}.csv`
          );
          document.body.appendChild(link);
          link.click();
          link.remove();
        } else if (data.errors && data.errors[0]) {
          // eslint-disable-next-line no-console
          console.log(data.errors[0]);
        }
      });
  };

  return (
    <>
      <h2>Platform users</h2>

      <Form
        onSubmit={handleFilterSubmit}
        render={({
          handleSubmit,
          values,
          form,
          valid,
          submitting,
          pristine,
        }) => (
          <StyledForm onSubmit={handleSubmit}>
            <FilterSearch
              handleReset={() => {
                form.reset();
                handleFilterSubmit({});
              }}
              name="search"
              placeholder="Enter full name, email or phone number"
              values={values}
            />
            <FilterSort>
              <FilterSortItem>
                <Field
                  name="userType"
                  render={({ input }) => {
                    return (
                      <Dropdown
                        input={input}
                        options={[
                          {
                            label: "Landlords",
                            value: USER_TYPE.LANDLORDS,
                          },
                          {
                            label: "Tenants",
                            value: USER_TYPE.TENANTS,
                          },
                        ]}
                        placeholder="User type"
                      />
                    );
                  }}
                />
              </FilterSortItem>
              <FilterSortItem>
                {Object.keys(filter).length > 0 && (
                  <FilterSortItemInner>
                    <SecondaryButton
                      onClick={() => handleFilterReset(form.reset)}
                      type="reset"
                    >
                      Clear
                    </SecondaryButton>
                  </FilterSortItemInner>
                )}
                <PrimaryButton
                  disabled={
                    (values.userType && !valid) || submitting || pristine
                  }
                  onClick={() => handleFilterSubmit(values)}
                >
                  Filter
                </PrimaryButton>
              </FilterSortItem>
            </FilterSort>
            <ExportButton onClick={handleDownloadExport} type="button">
              Export as csv
            </ExportButton>
          </StyledForm>
        )}
      />

      <Sortable field={["created_at"]} order={-1}>
        {({ field, order, setSort }) => (
          <Pagination filter={filter}>
            {({ page, setPage, perPage }) => {
              return (
                <div>
                  <Query<GetUsersList, GetUsersListVariables>
                    query={ALL_USERS}
                    variables={{
                      filter,
                      pagination: { page, perPage },
                      sort: field.map((v) => ({ field: v, order })),
                    }}
                  >
                    {({ data, loading, error }) => {
                      if (error) {
                        return <SystemError error={error} />;
                      }

                      const getOrder = (fieldName) => {
                        return field.includes(fieldName)
                          ? order === 1
                            ? -1
                            : 1
                          : 1;
                      };

                      return (
                        <TableWrapper>
                          {loading && <TableOverlay />}
                          <Pagebar
                            changePage={setPage}
                            curPage={page}
                            showNPages={3}
                            total={Math.ceil(
                              (data?.admin?.allUsers?.total || 1) / perPage
                            )}
                          />

                          <Table>
                            <Thead>
                              <ListHead
                                field={field}
                                getOrder={getOrder}
                                order={order}
                                setSort={setSort}
                              />
                            </Thead>
                            <Tbody>
                              {(data?.admin?.allUsers?.data ?? []).map(
                                (user) => (
                                  <ListItem
                                    key={user.id as string}
                                    user={user}
                                  />
                                )
                              )}
                            </Tbody>
                          </Table>

                          <Pagebar
                            changePage={setPage}
                            curPage={page}
                            showNPages={3}
                            total={Math.ceil(
                              (data?.admin?.allUsers?.total || 1) / perPage
                            )}
                          />
                        </TableWrapper>
                      );
                    }}
                  </Query>
                </div>
              );
            }}
          </Pagination>
        )}
      </Sortable>
    </>
  );
};

export default List;

const StyledForm = styled.form`
  display: flex;
`;

const ExportButton = styled(PrimaryButton)`
  margin-left: auto;
`;
