import React, { FC, useCallback, useMemo, useState } from "react";

import { useMutation } from "react-apollo";
import { Field, Form } from "react-final-form";

import { account_api_VeriffStatusEnum } from "../../../../../globalTypes";
import { PrimaryButton, SecondaryButton } from "../../../../../ui/atoms/Button";
import { TextInput } from "../../../../../ui/atoms/Input";
import { UserFormRow } from "../../../../../ui/atoms/UserForm";
import { CheckboxToggle } from "../../../../../ui/molecules/CheckboxToggle";
import ErrorMessage from "../../../../../ui/molecules/ErrorMessage";
import { FileField } from "../../../../../ui/molecules/FileField/FileField";
import { Modal } from "../../../../../ui/molecules/Modal";
import { TextareaField } from "../../../../../ui/molecules/TextareaField";
import { required } from "../../../../../util/validations";
import { ACCEPTED_FILE_TYPES } from "./constants";
import {
  CreateAdminVerificationSession,
  CreateAdminVerificationSessionVariables,
} from "./graphql/mutations/__generated__/CreateAdminVerificationSession";
import {
  UpdateAdminVerificationSession,
  UpdateAdminVerificationSessionVariables,
} from "./graphql/mutations/__generated__/UpdateAdminVerificationSession";
import CREATE_ADMIN_VERIFICATION_SESSION from "./graphql/mutations/createAdminVerificationSession.graphql";
import UPDATE_ADMIN_VERIFICATION_SESSION from "./graphql/mutations/updateAdminVerificationSession.graphql";
import { FormValues, OverrideVerificationSessionModalProps } from "./types";

import styles from "./styles.module.css";

// if session exist – update it. if not – create.
export const OverrideVerificationSessionModal: FC<OverrideVerificationSessionModalProps> = ({
  user,
  session,
  onCancel,
  onSubmit,
}) => {
  const [
    createAdminVerificationSession,
    { loading: loadingCreate, error: errorCreate },
  ] = useMutation<
    CreateAdminVerificationSession,
    CreateAdminVerificationSessionVariables
  >(CREATE_ADMIN_VERIFICATION_SESSION);

  const [
    updateAdminVerificationSession,
    { loading: loadingUpdate, error: errorUpdate },
  ] = useMutation<
    UpdateAdminVerificationSession,
    UpdateAdminVerificationSessionVariables
  >(UPDATE_ADMIN_VERIFICATION_SESSION);

  const [documents, setDocuments] = useState(session?.documents ?? []);
  const [showFileField, setShowFileField] = useState(true);

  const handleSubmit = useCallback(
    (values: FormValues) => {
      if (session) {
        updateAdminVerificationSession({
          variables: {
            accountId: user.id,
            firstName: values.first_name,
            lastName: values.last_name,
            isVerified: values.verified,
            notes: values.notes,
            documentIds: documents,
            sessionId: session?.session_id,
          },
        }).then(() => onSubmit());
      } else {
        createAdminVerificationSession({
          variables: {
            accountId: user.id,
            firstName: values.first_name,
            lastName: values.last_name,
            isVerified: values.verified,
            notes: values.notes,
            documentIds: documents,
          },
        }).then(() => onSubmit());
      }
    },
    [
      session,
      updateAdminVerificationSession,
      user.id,
      documents,
      onSubmit,
      createAdminVerificationSession,
    ]
  );

  const initialValues: FormValues = useMemo(
    () =>
      session
        ? {
            first_name: session.first_name,
            last_name: session.last_name,
            notes: session.notes,
            verified:
              session.status === account_api_VeriffStatusEnum.APPROVED
                ? true
                : false,
          }
        : {
            first_name: user.details?.first_name ?? "",
            last_name: user.details?.last_name ?? "",
            notes: "",
            verified: false,
          },
    [session, user.details]
  );

  return (
    <Modal
      onCancel={onCancel}
      title="Override Verification Session"
      withoutOverflow
    >
      <Form<FormValues>
        initialValues={initialValues}
        onSubmit={handleSubmit}
        render={({ handleSubmit, hasValidationErrors }) => (
          <form onSubmit={handleSubmit}>
            <UserFormRow className={styles.verifiedCheckbox}>
              <Field
                component={CheckboxToggle}
                name="verified"
                type="checkbox"
                labelText="Verified"
              />
            </UserFormRow>
            <UserFormRow>
              <label>First name</label>
              <Field
                component={TextInput}
                name="first_name"
                validate={required("Value is required")}
              />
              <Field component={ErrorMessage} name="first_name" />
            </UserFormRow>
            <UserFormRow>
              <label>Last name</label>
              <Field
                component={TextInput}
                name="last_name"
                validate={required("Value is required")}
              />
              <Field component={ErrorMessage} name="last_name" />
            </UserFormRow>
            <UserFormRow>
              <label>Documents</label>
              {documents.map((document: string, key: number) => (
                <div key={key} className={styles.filePreview}>
                  <a
                    href={`/media/${document}`}
                    target="_blank"
                    rel="noopener noreferrer"
                  >
                    <img
                      alt=""
                      className={styles.image}
                      src={`/media/${document}`}
                    />
                  </a>
                  <button
                    type="button"
                    onClick={() =>
                      setDocuments(documents.filter((doc) => doc !== document))
                    }
                  >
                    X
                  </button>
                </div>
              ))}
              {showFileField && (
                <Form
                  onSubmit={() => {}}
                  render={() => (
                    <Field
                      name="file"
                      render={({ input, meta }) => {
                        return (
                          <FileField
                            acceptedTypes={ACCEPTED_FILE_TYPES}
                            error={meta.touched && meta.error}
                            fileId={input.value}
                            handleChange={(document) => {
                              setDocuments([...documents, document]);

                              // I don't want to write a new file uploader. So I use existing one. In order to reload it I remount it. I know this is ugly.
                              setShowFileField(false);
                              setTimeout(() => setShowFileField(true));
                            }}
                          />
                        );
                      }}
                    />
                  )}
                ></Form>
              )}
            </UserFormRow>
            <UserFormRow>
              <label>Notes</label>
              <Field component={TextareaField} name="notes" rows="5" />
              <Field component={ErrorMessage} name="notes" />
            </UserFormRow>

            {errorCreate && <div>{errorCreate.message}</div>}
            {errorUpdate && <div>{errorUpdate.message}</div>}

            <div className={styles.buttons}>
              <SecondaryButton onClick={onCancel} type="button">
                Cancel
              </SecondaryButton>
              <PrimaryButton
                disabled={hasValidationErrors || loadingCreate || loadingUpdate}
              >
                Save
              </PrimaryButton>
            </div>
          </form>
        )}
      />
    </Modal>
  );
};
