import React, { FC, useCallback, useEffect, useMemo, useState } from "react";

import styled from "@emotion/styled";
import gql from "graphql-tag";
import { useMutation, useQuery } from "react-apollo";
import { RouteComponentProps, useHistory } from "react-router-dom";

import { SetReminderModal } from "../../../components/SetReminderModal/SetReminderModal";
import {
  common_api_generic_query_QueryFilterOperatorEnum,
  notes_api_CorrelationTypeEnum,
} from "../../../globalTypes";
import { SecondaryButton } from "../../../ui/atoms/Button";
import Spinner from "../../../ui/atoms/Spinner";
import { Table, TdInner, ThInner } from "../../../ui/atoms/Table";
import { UserInfoHeader, UserInform } from "../../../ui/atoms/UserCard";
import { UserFormWrapper } from "../../../ui/atoms/UserForm";
import { formatDate } from "../../../util/date";
import { DeleteNote, DeleteNoteVariables } from "./__generated__/DeleteNote";
import {
  FetchUserNotifications,
  FetchUserNotifications_admin_allNotes_notes,
  FetchUserNotificationsVariables,
} from "./__generated__/FetchUserNotifications";

import { ReactComponent as TrashIcon } from "./trash.icon.svg";

type ReminderData = {
  form: {
    comment: string;
    date: any;
  };
  reminderId: string;
  correlatedUserId: string;
};

const perPage = 20;

export const Notifications: FC<RouteComponentProps<{ id?: string }>> = ({
  match,
}) => {
  const [reminderData, setReminderData] = useState<ReminderData | null>(null);
  const [removingId, setRemovingId] = useState<string>("");
  const userId = match.params.id;
  const history = useHistory();
  const [page, setPage] = useState<number>(1);

  const variables: FetchUserNotificationsVariables = useMemo(
    () => ({
      pagination: { page, perPage },
      filter: userId
        ? [
            {
              fields: ["correlation_type"],
              value: [notes_api_CorrelationTypeEnum.CORRELATION_TYPE_USER],
              operator: common_api_generic_query_QueryFilterOperatorEnum.EQUALS,
            },
            {
              fields: ["correlation_id"],
              value: [userId],
              operator: common_api_generic_query_QueryFilterOperatorEnum.EQUALS,
            },
          ]
        : [],
    }),
    [page, userId]
  );

  const { data, loading, error, refetch } = useQuery<
    FetchUserNotifications,
    FetchUserNotificationsVariables
  >(FETCH_USER_NOTIFICATIONS, { variables });

  const [deleteReminder] = useMutation<DeleteNote, DeleteNoteVariables>(
    DELETE_NOTE
  );

  const showPrevButton = useMemo(() => page > 1, [page]);

  const showNextButton = useMemo(
    () =>
      data?.admin?.allNotes?.total &&
      data.admin.allNotes.total > page * perPage,
    [data, page]
  );

  const autoSwitchPage = useCallback(async () => {
    await setPage((prev) => prev - 1);
    return refetch(variables);
  }, [refetch, variables]);

  useEffect(() => {
    if (page > 1 && data?.admin?.allNotes?.notes?.length === 0) {
      void autoSwitchPage();
    }
  }, [page, data, autoSwitchPage]);

  const handleEditModalClose = useCallback(() => {
    setReminderData(null);
  }, []);

  const handleEditModalOpen = (
    notification: FetchUserNotifications_admin_allNotes_notes
  ) => () => {
    setReminderData({
      form: {
        comment: notification.text,
        date: new Date(notification.scheduled).toISOString().substr(0, 10),
      },
      reminderId: notification.id,
      correlatedUserId: notification.correlated_user!.id,
    });
  };

  const handleReminderEdited = useCallback(() => {
    setReminderData(null);
    void refetch(variables);
  }, [refetch, variables]);

  const handleRemove = useCallback(
    (id: string) => async () => {
      setRemovingId(id);
      await deleteReminder({ variables: { id } });
      await refetch(variables);
    },
    [deleteReminder, refetch, variables]
  );

  const handleGoBack = useCallback(() => history.goBack(), [history]);
  const handlePreviousPage = useCallback(() => setPage((prev) => prev - 1), []);
  const handleNextPage = useCallback(() => setPage((prev) => prev + 1), []);

  return (
    <UserInform>
      {reminderData && (
        <SetReminderModal
          form={reminderData.form}
          onCancel={handleEditModalClose}
          onSubmit={handleReminderEdited}
          reminderId={reminderData.reminderId}
          userId={reminderData.correlatedUserId}
        />
      )}

      <UserInfoHeader>
        <h2>Reminders</h2>
      </UserInfoHeader>

      <UserFormWrapper>
        {loading ? (
          <Spinner />
        ) : error ? (
          <>An error occurred: {error.message}</>
        ) : !data?.admin?.allNotes?.notes?.length ? (
          <>No reminders yet</>
        ) : (
          <>
            <Table>
              <thead>
                <tr>
                  <ThInner style={{ width: 140 }}>Reminder name</ThInner>
                  <ThInner style={{ width: 140 }}>Reminder date</ThInner>
                  <ThInner style={{ width: 140 }}>Set by</ThInner>
                  <ThInner>Comments</ThInner>
                  <ThInner style={{ width: 100 }}>Status</ThInner>
                  <ThInner style={{ width: 50 }} />
                </tr>
              </thead>
              <tbody>
                {data.admin.allNotes.notes.map((el) => (
                  <tr key={el.id}>
                    <TdInner>
                      <FakeLink onClick={handleEditModalOpen(el)}>
                        {el.correlation_type ===
                          notes_api_CorrelationTypeEnum.CORRELATION_TYPE_USER &&
                          `${el.correlated_user?.details?.first_name} ${el.correlated_user?.details?.last_name}`}
                      </FakeLink>
                    </TdInner>
                    <TdInner style={{ textAlign: "right" }}>
                      {formatDate(el.scheduled)}
                    </TdInner>
                    <TdInner>
                      {el.user?.details?.last_name}{" "}
                      {el.user?.details?.first_name}
                    </TdInner>
                    <TdInner style={{ wordBreak: "break-all" }}>
                      {el.text}
                    </TdInner>
                    <TdInner
                      style={{
                        color: getStatusColor(el.scheduled),
                        fontWeight: "bold",
                      }}
                    >
                      {getStatusText(el.scheduled)}
                    </TdInner>
                    <TdInner>
                      <FakeButton onClick={handleRemove(el.id)}>
                        {removingId === el.id ? (
                          <Spinner size="sm" />
                        ) : (
                          <TrashIcon />
                        )}
                      </FakeButton>
                    </TdInner>
                  </tr>
                ))}
              </tbody>
            </Table>
            <div style={{ display: "flex", justifyContent: "flex-end" }}>
              {showPrevButton && (
                <SecondaryButton onClick={handlePreviousPage}>
                  Previous page
                </SecondaryButton>
              )}
              {showNextButton && (
                <SecondaryButton onClick={handleNextPage}>
                  Next page
                </SecondaryButton>
              )}
            </div>
          </>
        )}
        <SecondaryButton onClick={handleGoBack}>
          <span className="material-icons" style={{ marginRight: "6px" }}>
            keyboard_backspace
          </span>
          Back
        </SecondaryButton>
      </UserFormWrapper>
    </UserInform>
  );
};

const isDateInFuture = (date: string) =>
  new Date(date).getTime() > new Date().getTime();

const getStatusText = (plannedDate: string) =>
  isDateInFuture(plannedDate) ? "Planned" : "Not solved";

const getStatusColor = (plannedDate: string) =>
  isDateInFuture(plannedDate) ? "#36B8FF" : "#F82600";

const FakeLink = styled.span`
  text-decoration: underline;
  color: blue;
  cursor: pointer;
`;

const FakeButton = styled.div`
  cursor: pointer;
`;

const DELETE_NOTE = gql`
  mutation DeleteNote($id: String) {
    admin {
      deleteNote(id: $id)
    }
  }
`;

export const FETCH_USER_NOTIFICATIONS = gql`
  query FetchUserNotifications(
    $pagination: gateway_impl_resolvers_admin_allParamsPaginationInput
    $sort: [common_api_generic_query_QueryParamsSortInput!]
    $filter: [common_api_generic_query_QueryParamsFilterInput!]
  ) {
    admin {
      allNotes(pagination: $pagination, sort: $sort, filter: $filter) {
        notes {
          id
          text
          scheduled
          correlation_type
          status
          correlated_user {
            id
            details {
              first_name
              last_name
            }
          }
          user {
            id
            details {
              first_name
              last_name
            }
          }
        }
        total
      }
    }
  }
`;
