import React, { FC, useCallback, useEffect, useState } from "react";

import { useMutation } from "react-apollo";

import { PrimaryButton } from "../../../ui/atoms/Button";
import {
  UserDetails as Col,
  UserFlexRow as Row,
} from "../../../ui/atoms/UserCard";
import {
  UserFormGroup,
  UserFormRow as FormRow,
} from "../../../ui/atoms/UserForm";
import { SimpleMessage } from "../../../ui/molecules/ErrorMessage";
import { ModalConfirm } from "../../../ui/molecules/Modal";
import {
  DeleteMessagesByUserID,
  DeleteMessagesByUserIDVariables,
} from "../graphql/mutations/__generated__/DeleteMessagesByUserID";
import DELETE_MESSAGES_BY_USER_ID from "../graphql/mutations/deleteMessagesByUserID.graphql";

type UserDetailsMessagesProps = {
  onComplete: () => void;
  userId: string | undefined;
};

const UserDetailsMessages: FC<UserDetailsMessagesProps> = ({ userId }) => {
  const [message, setMessage] = useState("");
  const [showModal, setShowModal] = useState(false);
  const [deleteMessages, { loading, error }] = useMutation<
    DeleteMessagesByUserID,
    DeleteMessagesByUserIDVariables
  >(DELETE_MESSAGES_BY_USER_ID);

  const handleDeleteClick = useCallback(() => setShowModal(true), []);
  const handleCancel = useCallback(() => setShowModal(false), []);
  const handleSubmit = useCallback(async () => {
    const deleted = await deleteMessages({
      variables: { userId },
    });
    if (deleted.data?.admin?.deleteMessagesByUserID?.success) {
      setShowModal(false);
    } else {
      setMessage("some error occurred");
    }
  }, [userId, deleteMessages]);

  useEffect(() => {
    setMessage("");
  }, [showModal]);

  return (
    <>
      <UserFormGroup>
        <h3>Messages</h3>
        <FormRow>
          <Row>
            <Col>Delete all user’s messages in chat</Col>
            <Col>
              <PrimaryButton onClick={handleDeleteClick}>Delete</PrimaryButton>
            </Col>
          </Row>
        </FormRow>
      </UserFormGroup>
      <ModalConfirm
        disabled={loading}
        isOpen={showModal}
        onCancel={handleCancel}
        onConfirm={handleSubmit}
        title="Messages"
      >
        <span>
          Are you sure you want to delete all user’s messages in chat?
        </span>
        {error && <SimpleMessage>{error.message}</SimpleMessage>}
        {message && <SimpleMessage>{message}</SimpleMessage>}
      </ModalConfirm>
    </>
  );
};

export default UserDetailsMessages;
