import React, { useCallback } from "react";

import { Field, Form } from "react-final-form";

import {
  DeleteButton,
  PrimaryButton,
  SecondaryButton,
} from "../../ui/atoms/Button";
import { CheckboxBlock } from "../../ui/atoms/CheckboxInput";
import {
  ButtonsGroup,
  UserFormAction,
  UserFormOverlay,
  UserFormRow,
} from "../../ui/atoms/UserForm";
import Dropdown from "../../ui/molecules/Dropdown";
import { InputField } from "../../ui/molecules/InputField";
import { TextareaFieldTouched } from "../../ui/molecules/TextareaField";
import {
  composeValidators,
  required,
  requiredWithZero,
} from "../../util/validations";
import {
  CartProductOptions,
  ProductTypeOptions,
  RoleOptions,
  TaxTypeOptions,
} from "./constanst";
import { FormData, ModalModeEnum, Product } from "./types";

import css from "./index.module.css";

type Props = {
  initialValues: FormData;
  currentProductData: Product;
  onSubmit: ({ product, id }) => void;
  onClose: () => void;
  onDelete: (id: string | undefined) => void;
  mode: ModalModeEnum;
};
const EditForm = ({
  mode,
  initialValues,
  onSubmit,
  currentProductData,
  onClose,
  onDelete,
}: Props) => {
  const handleSubmit = useCallback(
    ({ product }) => onSubmit({ product, id: currentProductData?.id }),
    [onSubmit, currentProductData]
  );

  const handleDelete = useCallback(() => onDelete(currentProductData?.id), [
    onDelete,
    currentProductData,
  ]);
  return (
    <Form
      initialValues={initialValues}
      onSubmit={handleSubmit}
      render={({ handleSubmit, submitting, valid }) => (
        <form onSubmit={handleSubmit}>
          {submitting && <UserFormOverlay />}

          <UserFormRow>
            <label htmlFor="product.title">Product Title</label>
            <Field
              component={InputField}
              name="product.title"
              validate={required("Product title is require field")}
            />
          </UserFormRow>
          <UserFormRow>
            <label htmlFor="product.code">Product code</label>
            <Field
              component={InputField}
              name="product.code"
              validate={required("Product code is require field")}
            />
          </UserFormRow>
          <UserFormRow>
            <label htmlFor="product.description">Description</label>
            <Field
              component={TextareaFieldTouched}
              name="product.description"
              rows={5}
            />
          </UserFormRow>
          <div className={css.formRow}>
            <div className={css.inputGroup}>
              <label htmlFor="product.cart_product">Cart Product</label>
              <Field
                name="product.cart_product"
                render={({ input }) => (
                  <Dropdown input={input} options={CartProductOptions} />
                )}
              />
            </div>
            <div className={css.inputGroup}>
              <label htmlFor="product.type">Product type</label>
              <Field
                name="product.type"
                render={({ input }) => (
                  <Dropdown input={input} options={ProductTypeOptions} />
                )}
              />
            </div>
            <div className={css.inputGroup}>
              <label htmlFor="product.role">Role</label>
              <Field
                name="product.role"
                render={({ input }) => (
                  <Dropdown input={input} options={RoleOptions} />
                )}
              />
            </div>
          </div>
          <div className={css.formRow}>
            <div className={css.inputGroup}>
              <label htmlFor="product.tax_type">Tax type</label>
              <Field
                name="product.tax_type"
                render={({ input }) => (
                  <Dropdown input={input} options={TaxTypeOptions} />
                )}
              />
            </div>
            <div className={css.inputGroup}>
              <label htmlFor="product.tax_amount">Tax amount</label>
              <Field
                component={InputField}
                name="product.tax_amount"
                parse={(value) => {
                  const num = Number.parseFloat(value);
                  return num ? num : 0;
                }}
              />
            </div>
            <div className={css.inputGroup}>
              <label htmlFor="product.tax_percentage">Tax percentage</label>
              <Field
                component={InputField}
                name="product.tax_percentage"
                parse={(value) => {
                  const num = Number.parseFloat(value);
                  return num ? num : 0;
                }}
              />
            </div>
          </div>
          <div className={css.formRow}>
            <div className={css.inputGroup}>
              <label htmlFor="product.price">Price</label>
              <Field
                component={InputField}
                name="product.price"
                parse={(value) => {
                  const num = Number.parseFloat(value);
                  return num ? num : 0;
                }}
                validate={composeValidators(
                  requiredWithZero("Price is require field")
                )}
              />
            </div>
            <div className={css.inputGroup}>
              <label htmlFor="product.reference_id">Reference id</label>
              <Field component={InputField} name="product.reference_id" />
            </div>
            <div className={css.inputGroup}>
              <label htmlFor="product.reference_provider">
                Reference provider
              </label>
              <Field component={InputField} name="product.reference_provider" />
            </div>
          </div>
          <div className={css.formRow}>
            <div className={css.inputGroup}>
              <label htmlFor="product.main_icon_url ">Main icon url </label>
              <Field component={InputField} name="product.main_icon_url" />
            </div>
            <div className={css.inputGroup}>
              <label htmlFor="product.page_url">Page url</label>
              <Field component={InputField} name="product.page_url" />
            </div>
          </div>
          <UserFormRow>
            <Field
              name="product.disabled"
              render={({ input }) => (
                <CheckboxBlock>
                  <input type="checkbox" {...input} />
                  <span>Disabled</span>
                </CheckboxBlock>
              )}
              type="checkbox"
            />
          </UserFormRow>
          <UserFormAction>
            <ButtonsGroup>
              <SecondaryButton onClick={onClose} type="button">
                Cancel
              </SecondaryButton>
              {mode === ModalModeEnum.Edit && (
                <DeleteButton onClick={handleDelete} type="button">
                  Delete
                </DeleteButton>
              )}
              <PrimaryButton disabled={submitting || !valid} type="submit">
                Save
              </PrimaryButton>
            </ButtonsGroup>
          </UserFormAction>
        </form>
      )}
    />
  );
};

export default EditForm;
