import React, { memo, useCallback, useMemo, useState } from "react";

import { useMutation, useQuery } from "react-apollo";

import { PrimaryButton } from "../../ui/atoms/Button";
import { Table, Tbody, Thead } from "../../ui/atoms/Table";
import TableOverlay from "../../ui/atoms/TableOverlay";
import TableWrapper from "../../ui/atoms/TableWrapper";
import { ModalWide } from "../../ui/molecules/Modal";
import {
  defaultProduct,
  ProductTypeOptions,
  RoleOptions,
  TaxTypeOptions,
} from "./constanst";
import EditForm from "./EditForm";
import ADD_PRODUCT from "./graphql/AddProduct.graphql";
import DELETE_PRODUCT from "./graphql/DeleteProduct.graphql";
import GET_PRODUCTS from "./graphql/GetProducts.graphql";
import UPDATE_PRODUCT from "./graphql/UpdateProduct.graphql";
import Row from "./ProductRow";
import TableHeader from "./TableHeader";
import { FormData, ModalModeEnum, Product } from "./types";
import ViewProduct from "./ViewProduct";

import css from "./index.module.css";

const ProductsPage = memo(() => {
  const { data, loading, refetch } = useQuery(GET_PRODUCTS);
  const [deleteProduct] = useMutation(DELETE_PRODUCT);
  const [updateProduct] = useMutation(UPDATE_PRODUCT);
  const [addProduct] = useMutation(ADD_PRODUCT);

  const [isModalOpened, setIsModalOpened] = useState<boolean>(false);
  const [modalMode, setModalMode] = useState<ModalModeEnum>(ModalModeEnum.View);

  const [currentProductData, setCurrentProductData] = useState<Product>(
    defaultProduct
  );

  const initialValues = useMemo<FormData>(() => {
    return {
      product: {
        ...currentProductData,
        cart_product: {
          label: currentProductData?.cart_product ? "Yes" : "No",
          value: !!currentProductData?.cart_product,
        },
        type: ProductTypeOptions.find(
          (el) => el.value === currentProductData.type
        ) || { label: " ", value: currentProductData.type },
        role: RoleOptions.find(
          (el) => el.value === currentProductData.role
        ) || { label: " ", value: currentProductData.role },
        tax_type: TaxTypeOptions.find(
          (el) => el.value === currentProductData.tax_type
        ) || { label: " ", value: currentProductData.tax_type },
      },
    };
  }, [currentProductData]);

  const handleAddClick = useCallback(() => {
    setIsModalOpened(true);
    setModalMode(ModalModeEnum.Add);
  }, [setIsModalOpened]);

  const handleOpenDetailsClick = useCallback(
    (product) => {
      setCurrentProductData(product);
      setIsModalOpened(true);
      setModalMode(ModalModeEnum.View);
    },
    [setCurrentProductData, setIsModalOpened]
  );

  const handleEditClick = useCallback(() => {
    setModalMode(ModalModeEnum.Edit);
  }, [setModalMode]);

  const handleCloseModal = useCallback(() => {
    setIsModalOpened(false);
    setModalMode(ModalModeEnum.View);
    setCurrentProductData(defaultProduct);
  }, []);

  const handleSubmit = async ({ product, id }) => {
    const targetMutation = id ? updateProduct : addProduct;
    await targetMutation({
      variables: {
        ...product,
        id,
        role: product.role.value,
        tax_type: product.tax_type.value,
        cart_product: product.cart_product.value,
        type: product.type.value,
        main_icon_url: product.main_icon_url,
        page_url: product.page_url,
      },
    });
    await refetch();
    handleCloseModal();
  };

  const handleDelete = async (id: string | undefined) => {
    await deleteProduct({ variables: { id } });
    await refetch();
    handleCloseModal();
  };

  return (
    <>
      <div className={css.buttonWrapper}>
        <PrimaryButton onClick={handleAddClick}>Add product</PrimaryButton>
      </div>
      <TableWrapper>
        {loading && <TableOverlay />}
        <Table>
          <Thead>
            <TableHeader />
          </Thead>
          <Tbody>
            {data?.products.list.extra_services.map((product) => (
              <Row
                key={product.id}
                onOpenDetailsClick={handleOpenDetailsClick}
                product={product}
              />
            ))}
          </Tbody>
        </Table>
      </TableWrapper>
      {isModalOpened && (
        <ModalWide
          onCancel={handleCloseModal}
          title={currentProductData?.id ? "Product" : "Add Product"}
          width={800}
        >
          {modalMode === ModalModeEnum.View ? (
            <ViewProduct
              onEditClick={handleEditClick}
              product={currentProductData}
            />
          ) : (
            <EditForm
              currentProductData={currentProductData}
              initialValues={initialValues}
              mode={modalMode}
              onClose={handleCloseModal}
              onDelete={handleDelete}
              onSubmit={handleSubmit}
            />
          )}
        </ModalWide>
      )}
    </>
  );
});

export default ProductsPage;
