import React, {
  useCallback,
  useEffect,
  useMemo,
  useRef,
  useState,
} from "react";

import debounce from "lodash.debounce";

import { colors } from "../../theme";
import Spinner from "../Spinner";
import {
  Input,
  InputGroup,
  Option,
  Options,
  SelectContainer,
  SpinnerContainer,
} from "./styles";

export const SelectField = (inputProps) => (
  <Select
    {...inputProps}
    defaultValue={inputProps.input.value}
    data-invalid={inputProps.meta.touched && inputProps.meta.invalid}
  />
);

export type SelectProps<T> = {
  placeholder: string;
  options: Array<T>;
  displayAttr?: string;
  valueAttr?: string;
  defaultValue?: string;
  readOnly?: boolean;
  onSelect?: (T) => void;
  onFilter?: (string) => Promise<void>;
  height?: string;
};

const Select = <T,>(props: SelectProps<T>) => {
  const inputRef = useRef(null);

  const defaultOption = useMemo(
    () => props.options.find((o) => o["value"] === props.defaultValue),
    [props.options, props.defaultValue]
  );

  const getOptionTitle = useCallback(
    (option?: T) =>
      option && props.displayAttr
        ? option[props.displayAttr]
        : JSON.stringify(option),
    [props.displayAttr]
  );

  const [optionsVisible, setOptionsVisible] = useState<boolean>(false);
  const [filteredOptions, setFilteredOptions] = useState<T[]>(props.options);
  const [selectedOption, setSelectedOption] = useState<T | undefined>(
    defaultOption
  );
  const [inputValue, setInputValue] = useState<string>(
    getOptionTitle(defaultOption)
  );
  const [loading, setLoading] = useState<boolean>(false);

  useEffect(() => setFilteredOptions(props.options), [props.options]);

  useEffect(() => {
    const reset = () => {
      setInputValue(getOptionTitle(selectedOption));
      setOptionsVisible(false);
    };

    const onClickEvent = (evt) =>
      evt.target !== inputRef.current &&
      !evt.target.classList.contains("selectOption") &&
      reset();

    const onEscEvent = (evt) => {
      if (evt.key === "Escape") {
        evt.preventDefault();
        reset();
      }
    };

    document.addEventListener("click", onClickEvent);
    document.addEventListener("keydown", onEscEvent);
    return () => {
      document.removeEventListener("click", onClickEvent);
      document.removeEventListener("keydown", onEscEvent);
    };
  }, [selectedOption, getOptionTitle]);

  const selectOption = (
    evt: React.MouseEvent<HTMLElement>,
    option: T
  ): void => {
    evt.preventDefault();

    if (props.onSelect) {
      props.onSelect(option);
    }

    if (props["input"]) {
      props["input"].onChange(option["value"]);
    }

    setSelectedOption(option);
    setInputValue(getOptionTitle(option));
    setOptionsVisible(false);
  };

  const onSearchInputChange = (searchLine: string): void => {
    setInputValue(searchLine);
    filterOptions(searchLine);
  };

  const filterOptions = useMemo(() => {
    const handler = (searchLine: string): void => {
      setLoading(true);
      if (props.onFilter) {
        props.onFilter(searchLine).then(() => {
          setLoading(false);
          setOptionsVisible(true);
        });
      } else {
        setFilteredOptions(
          searchLine
            ? filterLocalOptions(props.options, searchLine)
            : props.options
        );
        setLoading(false);
        setOptionsVisible(true);
      }
    };

    return debounce(handler, 800);
  }, [props]);

  const filterLocalOptions = (options: T[], filterLine: string): T[] => {
    const terms = filterLine?.toLowerCase().split(" ");
    const filter = (item) =>
      terms?.every((term) => item.title?.toLowerCase().indexOf(term) !== -1);
    return terms.length ? options.filter(filter) : [];
  };

  return (
    <SelectContainer>
      <InputGroup>
        <Input
          type="search"
          ref={inputRef}
          placeholder={props.placeholder}
          value={inputValue || ""}
          onChange={(evt) => onSearchInputChange(evt.target.value)}
          onFocus={(_) =>
            filteredOptions.length && !props.readOnly
              ? setOptionsVisible(true)
              : {}
          }
          readOnly={props.readOnly}
          autoComplete={"off"}
          height={props.height}
        ></Input>
        {loading && (
          <SpinnerContainer>
            <Spinner color={colors.mashPaleRed} size="md" />
          </SpinnerContainer>
        )}
      </InputGroup>

      {optionsVisible && (
        <Options height={props.height}>
          {!filteredOptions.length && (
            <Option key={0}>{"No options available."}</Option>
          )}
          {!!filteredOptions.length &&
            filteredOptions.map((opt, i) => (
              <Option key={i}>
                <a
                  href="/#"
                  className="selectOption"
                  onClick={(evt) => selectOption(evt, opt)}
                >
                  {getOptionTitle(opt)}
                </a>
              </Option>
            ))}
        </Options>
      )}
    </SelectContainer>
  );
};

export default Select;
