import React, { FC, ReactNode, useState } from "react";

import { Field, Form } from "react-final-form";
import { useHistory } from "react-router-dom";

import {
  composeValidators,
  required,
  validEmail,
  validPhoneNumber,
} from "../../../../../util/validations";
import { InputField } from "../../../components/Input";
import { PhoneInputField } from "../../../components/PhoneInput";
import Spinner from "../../../components/Spinner";
import { AccountDetails, BankDetails } from "../../../models/account";
import { CardHeader, CardHeaderLink, CardTitle } from "../../../styles";
import { colors } from "../../../theme";
import BankDetailsModal from "../../Main/BankDetailsModal";
import {
  BankDetailsButton,
  ButtonsContainer,
  Footer,
  FormContainer,
  NamesContainer,
} from "./styles";

type AccountFormProps = {
  initialValues: AccountDetails;
  readOnly?: boolean;
  onBankDetailsSubmit?: (details: BankDetails) => void;
  onAccountDetailsSubmit?: (details: AccountDetails) => void;
  buttons?: (
    submitting: boolean,
    valid: boolean,
    formValues: AccountDetails
  ) => ReactNode;
};

const AccountForm: FC<AccountFormProps> = ({
  initialValues,
  readOnly = true,
  onBankDetailsSubmit,
  onAccountDetailsSubmit,
  buttons,
}) => {
  const ACCOUNT_DETAILS_PAGE = "/remote-sales/account-details";
  const history = useHistory();

  const [bankDetailsModal, setBankDetailsModal] = useState<boolean>(false);

  return (
    <>
      <Form<AccountDetails>
        initialValues={initialValues}
        onSubmit={(values) =>
          onAccountDetailsSubmit ? onAccountDetailsSubmit(values) : {}
        }
        render={({ submitting, valid, handleSubmit, values }) => (
          <form
            onSubmit={(e) => {
              e.preventDefault();
              e.stopPropagation();
              handleSubmit();
            }}
          >
            <FormContainer>
              <CardHeader>
                <CardTitle>
                  {readOnly ? "Account details" : "New account"}
                  {initialValues.id && !onAccountDetailsSubmit && (
                    <BankDetailsButton
                      active={!!initialValues?.bankDetails?.accountNumber}
                      onClick={(_) => setBankDetailsModal(true)}
                    >
                      {!!initialValues?.bankDetails?.accountNumber
                        ? "Bank Details"
                        : "NO Bank Details"}
                    </BankDetailsButton>
                  )}
                </CardTitle>

                {readOnly && (
                  <CardHeaderLink
                    onClick={() => {
                      if (window.location.hash.endsWith(ACCOUNT_DETAILS_PAGE)) {
                        window.location.reload();
                      } else {
                        history.push({ pathname: ACCOUNT_DETAILS_PAGE });
                      }
                    }}
                  >
                    Change account
                  </CardHeaderLink>
                )}
              </CardHeader>

              <NamesContainer>
                <Field
                  placeholder="First name"
                  name="firstName"
                  readOnly={submitting || readOnly}
                  defaultValue={initialValues?.firstName}
                  validate={required()}
                >
                  {InputField}
                </Field>

                <Field
                  placeholder="Second name"
                  name="secondName"
                  readOnly={submitting || readOnly}
                  defaultValue={initialValues?.secondName}
                  validate={required()}
                >
                  {InputField}
                </Field>
              </NamesContainer>

              <Field
                placeholder="Email"
                name="email"
                readOnly={submitting || readOnly}
                defaultValue={initialValues?.email}
                validate={composeValidators(required(), validEmail())}
              >
                {InputField}
              </Field>

              <Field
                placeholder="Phone"
                name="phone"
                readOnly={submitting || readOnly}
                defaultValue={initialValues?.phone}
                validate={composeValidators(required(), validPhoneNumber())}
              >
                {PhoneInputField}
              </Field>

              <Footer>
                {buttons && (
                  <ButtonsContainer>
                    {submitting && (
                      <Spinner color={colors.mashPaleRed} size="md" />
                    )}
                    {buttons(submitting, valid, values)}
                  </ButtonsContainer>
                )}
              </Footer>
            </FormContainer>
          </form>
        )}
      ></Form>

      {bankDetailsModal && onBankDetailsSubmit && (
        <BankDetailsModal
          title="Bank Details"
          bankDetails={initialValues.bankDetails || ({} as BankDetails)}
          onSubmit={(bankDetails) => {
            setBankDetailsModal(false);
            onBankDetailsSubmit(bankDetails);
          }}
          onCancel={() => setBankDetailsModal(false)}
        ></BankDetailsModal>
      )}
    </>
  );
};

export default AccountForm;
