import React, { FC, useCallback, useEffect, useState } from "react";

import { withApollo, WithApolloClient } from "react-apollo";
import { useHistory, useLocation } from "react-router-dom";

import Button from "../../components/Button";
import Spinner from "../../components/Spinner";
import {
  UserCreate,
  UserCreateVariables,
} from "../../graphql/mutations/__generated__/UserCreate";
import {
  UserUpdate,
  UserUpdateVariables,
} from "../../graphql/mutations/__generated__/UserUpdate";
import USER_CREATE from "../../graphql/mutations/UserCreate.graphql";
import USER_UPDATE from "../../graphql/mutations/UserUpdate.graphql";
import {
  UserGetByEmail,
  UserGetByEmailVariables,
} from "../../graphql/queries/__generated__/UserGetByEmail";
import USER_GET_BY_EMAIL from "../../graphql/queries/UserGetByEmail.graphql";
import { AccountDetails, BankDetails } from "../../models/account";
import { MainContainer } from "../../styles";
import { colors } from "../../theme";
import AccountForm from "./AccountForm";
import { AccountFound, NoAccountFound } from "./AccountState";
import { ErrorContainer, SpinnerContainer } from "./styles";

export type AccountDetailsPageProps = WithApolloClient<{}>;

const AccountDetailsPage: FC<AccountDetailsPageProps> = ({ client }) => {
  const location = useLocation();
  const history = useHistory();
  const leadDetails = location.state as AccountDetails;

  const [accountDetails, setAccountDetails] = useState<AccountDetails>(
    leadDetails
  );
  const [error, setError] = useState<string>("");
  const [loading, setLoading] = useState<boolean>(false);

  const fetchUserByEmail = useCallback(
    async (email: string): Promise<AccountDetails | undefined> => {
      if (!email) {
        return undefined;
      }

      setLoading(true);
      const resp = await client.query<UserGetByEmail, UserGetByEmailVariables>({
        query: USER_GET_BY_EMAIL,
        fetchPolicy: "no-cache",
        variables: { email },
      });

      const errsCodes =
        resp.errors?.flatMap((e) => e.extensions).map((e) => e.code) || [];
      const errs = resp.errors?.map((e) => e.message) || [];

      if (errs.length && !errsCodes.includes("not_found")) {
        setLoading(false);
        throw new Error(`Error while searching user: ${errs.join(",")}`);
      }

      if (!resp.data?.admin?.UserByEmail?.id) {
        setLoading(false);
        return undefined;
      }

      setLoading(false);
      return {
        id: resp.data?.admin?.UserByEmail?.id || "",
        firstName: resp.data?.admin?.UserByEmail?.details?.first_name || "",
        secondName: resp.data?.admin?.UserByEmail?.details?.last_name || "",
        email: resp.data?.admin?.UserByEmail?.contact?.email_primary || "",
        phone: resp.data?.admin?.UserByEmail?.contact?.phone_primary || "",
        bankDetails: {
          accountName:
            resp.data?.admin?.UserByEmail?.bank_details?.account_name || "",
          personalReference:
            resp.data?.admin?.UserByEmail?.bank_details?.personal_reference ||
            "",
          accountNumber:
            resp.data?.admin?.UserByEmail?.bank_details?.account_number || "",
          sortCode:
            resp.data?.admin?.UserByEmail?.bank_details?.sort_code || "",
        } as BankDetails,
      };
    },
    [client]
  );

  const findUserByEmail = useCallback(
    async (email: string): Promise<void> => {
      setError("");
      const accountDetails = await fetchUserByEmail(email);
      if (accountDetails) {
        setAccountDetails(accountDetails);
      }
    },
    [fetchUserByEmail]
  );

  useEffect(() => {
    (async () => {
      await findUserByEmail(leadDetails?.email);
    })();
  }, [findUserByEmail, leadDetails]);

  const createUser = useCallback(
    async (accountDetails: AccountDetails): Promise<string> => {
      setLoading(true);
      const createUserResp = await client.query<
        UserCreate,
        UserCreateVariables
      >({
        query: USER_CREATE,
        fetchPolicy: "no-cache",
        variables: {
          firstName: accountDetails.firstName,
          secondName: accountDetails.secondName,
          email: accountDetails.email,
          phone: accountDetails.phone,
        },
      });

      const createErrs =
        createUserResp.data?.admin?.createUser?.errors?.map((e) => String(e)) ||
        [];
      if (createErrs.length) {
        setLoading(false);
        throw new Error(`Error while creating user: ${createErrs.join(",")}`);
      }

      setLoading(false);
      return createUserResp.data?.admin?.createUser?.account.id || "";
    },
    [client]
  );

  const updateUser = useCallback(
    async (accountDetails: AccountDetails): Promise<string> => {
      setLoading(true);

      const updateUserResp = await client.query<
        UserUpdate,
        UserUpdateVariables
      >({
        query: USER_UPDATE,
        fetchPolicy: "no-cache",
        variables: {
          id: accountDetails.id,
          firstName: accountDetails.firstName,
          secondName: accountDetails.secondName,
          email: accountDetails.email,
          phone: accountDetails.phone,
          bankDetails: {
            account_name: accountDetails.bankDetails?.accountName,
            personal_reference: accountDetails.bankDetails?.personalReference,
            account_number: accountDetails.bankDetails?.accountNumber,
            sort_code: accountDetails.bankDetails?.sortCode,
          },
        },
      });

      setLoading(false);
      return updateUserResp.data?.admin?.updateUser?.id || "";
    },
    [client]
  );

  const confirmOrCreateAccount = useCallback(
    async (formDetails: AccountDetails) => {
      setError("");
      const user = await fetchUserByEmail(formDetails.email);
      let userId = user?.id;

      try {
        if (!user) {
          userId = await createUser(formDetails);
        } else if (user && !user.phone) {
          await updateUser({ ...user, phone: formDetails.phone });
        }

        if (userId) {
          history.push(`/remote-sales/main?userId=${userId}`);
        }
      } catch (err) {
        setError(String(err));
        setLoading(false);
      }
    },
    [fetchUserByEmail, createUser, updateUser, history]
  );

  const SearchButton = (submitting: boolean, formValues: AccountDetails) => (
    <Button
      type="button"
      disabled={submitting || !formValues?.email}
      onClick={async () => {
        setAccountDetails(formValues);
        await findUserByEmail(formValues?.email);
      }}
    >
      Search by email
    </Button>
  );

  return (
    <MainContainer>
      {!loading &&
        (!accountDetails?.id ? (
          <NoAccountFound></NoAccountFound>
        ) : (
          <AccountFound></AccountFound>
        ))}
      {!loading && (
        <AccountForm
          readOnly={!!accountDetails?.id}
          initialValues={accountDetails || ({} as AccountDetails)}
          onAccountDetailsSubmit={confirmOrCreateAccount}
          buttons={(submitting, valid, formValues) => (
            <>
              {!accountDetails?.id && SearchButton(submitting, formValues)}
              <Button type="submit" disabled={submitting || !valid}>
                {!accountDetails?.id ? "Create" : "Confirm"}
              </Button>
            </>
          )}
        ></AccountForm>
      )}
      {error && <ErrorContainer>{error}</ErrorContainer>}
      {loading && (
        <SpinnerContainer>
          <Spinner color={colors.mashPaleRed} size="lg" />
        </SpinnerContainer>
      )}
    </MainContainer>
  );
};

export default withApollo(AccountDetailsPage);
