import React, { FC, ReactNode } from "react";

import { splitEvery } from "ramda";
import { Field, Form } from "react-final-form";

import {
  composeFieldValidators,
  maxLength,
  minLength,
  required,
} from "../../../../../../util/validations";
import { InputField } from "../../../../components/Input";
import { BankDetails } from "../../../../models/account";
import {
  BankDetailsFormContainer,
  ButtonsContainer,
  ErrorsContainer,
} from "./styles";

type BankDetailsFormProps = {
  bankDetails?: BankDetails;
  onSubmit?: (bankDetails: BankDetails) => void;
  errors?: Array<String>;
  buttons?: (submitting: boolean, valid: boolean) => ReactNode;
};

const parseSortCode = (value: string) =>
  splitEvery(2, value.replace(/\D/g, "")).join("-");

const BankDetailsForm: FC<BankDetailsFormProps> = (
  {
    bankDetails,
    onSubmit,
    buttons,
    errors,
  }) => (
  <Form<BankDetails>
    initialValues={bankDetails}
    onSubmit={(values) => (onSubmit ? onSubmit(values) : {})}
    render={({ submitting, valid, handleSubmit }) => (
      <form
        onSubmit={(e) => {
          e.preventDefault();
          handleSubmit();
        }}
      >
        <BankDetailsFormContainer>
          <Field
            name="accountName"
            placeholder="Account name"
            validate={required()}
            defaultValue={bankDetails?.accountName}
            readOnly={submitting}
          >
            {InputField}
          </Field>
          <Field
            name="personalReference"
            placeholder="Personal reference"
            defaultValue={bankDetails?.personalReference}
            readOnly={submitting}
          >
            {InputField}
          </Field>
          <Field
            name="accountNumber"
            placeholder="Account number"
            validate={composeFieldValidators(required(), maxLength(9))}
            maxLength={9}
            defaultValue={bankDetails?.accountNumber}
            readOnly={submitting}
          >
            {InputField}
          </Field>
          <Field
            name="sortCode"
            placeholder="Sort code"
            validate={composeFieldValidators(
              required(),
              minLength(6),
              maxLength(8)
            )}
            minLength={6}
            maxLength={8}
            parse={parseSortCode}
            defaultValue={bankDetails?.sortCode}
            readOnly={submitting}
          >
            {InputField}
          </Field>
        </BankDetailsFormContainer>

        {!!errors?.length && (
          <ErrorsContainer>{errors.join(",")}</ErrorsContainer>
        )}

        {!!buttons?.length && (
          <ButtonsContainer>{buttons(submitting, valid)}</ButtonsContainer>
        )}
      </form>
    )}
  ></Form>
);

export default BankDetailsForm;
