import React, { FC, useCallback, useState } from "react";

import Checkbox from "../../../../components/Checkbox";
import {
  FurnishedType,
  ProductType,
  PropertyProduct,
} from "../../../../models/product";
import { ErrorContainer } from "../../../../styles";
import ProductAttributes from "./ProductAttributes";
import ProductLastSale from "./ProductLastSale";
import {
  ProductDetailsContainer,
  ProductDetailsLabel,
  ProductDetailsPriceValue,
  ProductDetailsPriceValueSelect,
  ProductItem,
  ProductLine,
  ProductsContainer,
} from "./styles";

type PropertyProductsListProps = {
  propertyId: string;
  products: PropertyProduct[];
  onProductsChange: (products: PropertyProduct[]) => void;
};

const PropertyProductsList: FC<PropertyProductsListProps> = ({
  propertyId,
  products,
  onProductsChange,
}) => {
  const [error, setError] = useState<string>("");
  const onProductClick = useCallback(
    async (prod: PropertyProduct): Promise<void> => {
      if (prod.type === ProductType.INVENTORY) {
        prod.inventoryDetails = {
          furnishedType: FurnishedType.UNFURNISHED,
          bedroomsNum: 1,
        };
      }

      prod.selected = !prod.selected;
      try {
        await prod.calculatePrice(prod);
      } catch (err) {
        setError(String(err));
      }

      const newProduct = products.map((_prod) => {
        if (prod.id === _prod.id && prod.selected) {
          _prod = prod;
        }
        return _prod;
      });
      onProductsChange(newProduct);
    },
    [products, onProductsChange]
  );

  const onProductAttributeChange = useCallback(
    async (prod: PropertyProduct): Promise<void> => {
      try {
        await prod.calculatePrice(prod);
      } catch (err) {
        setError(String(err));
      }
      onProductsChange(products);
    },
    [onProductsChange, products]
  );

  return (
    <>
      {error && <ErrorContainer> {error}</ErrorContainer>}
      <ProductsContainer>
        {products.map((prod, i) => (
          <ProductItem key={`${propertyId}_${i}`}>
            <ProductLine>
              <Checkbox
                id={`${propertyId}_${i}`}
                title={prod.title}
                checked={!!prod.selected}
                onChange={() => onProductClick(prod)}
              />
              {!!prod.lastSale && (
                <ProductLastSale sale={prod.lastSale}></ProductLastSale>
              )}
            </ProductLine>

            {prod.selected && (
              <ProductDetailsContainer>
                <ProductAttributes
                  product={prod}
                  onChange={(updatedProd) =>
                    onProductAttributeChange(updatedProd)
                  }
                />
                <ProductDetailsLabel>Price(£)</ProductDetailsLabel>
                {/* TODO: change this to dropdown */}
                {prod.remote_sales_prices.length > 0 &&
                prod.remote_sales_taxes.length > 0 &&
                prod.remote_sales_prices.length ===
                  prod.remote_sales_taxes.length ? (
                  <ProductDetailsPriceValueSelect
                    onChange={(evt) => {
                      const index = evt.target.selectedIndex;
                      prod.price = prod.remote_sales_prices[index];
                      prod.tax = prod.remote_sales_taxes[index];
                      if (prod.gasSafetyDetails?.boiler) {
                        prod.gasSafetyDetails.selected_index = index;
                      }
                      onProductAttributeChange({ ...prod });
                    }}
                  >
                    {/* TODO: change the value if gas boiler are checked */}
                    {/* TODO: update state when all value are change */}
                    {prod.remote_sales_prices.map((value, index) => (
                      <option key={index}>
                        {Number(value + prod.remote_sales_taxes[index]).toFixed(
                          2
                        )}
                      </option>
                    ))}
                  </ProductDetailsPriceValueSelect>
                ) : (
                  <ProductDetailsPriceValue>
                    {(prod.price + prod.tax).toFixed(2)}
                  </ProductDetailsPriceValue>
                )}
              </ProductDetailsContainer>
            )}
          </ProductItem>
        ))}
      </ProductsContainer>
    </>
  );
};

export default PropertyProductsList;
