import React, { FC, ReactNode } from "react";

import { format } from "date-fns";
import { Field, Form } from "react-final-form";

import { composeValidators, required, validPhoneNumber } from "../../../../../../../util/validations";
import { InputField } from "../../../../../components/Input";
import { PhoneInputField } from "../../../../../components/PhoneInput";
import { SelectField } from "../../../../../components/Select";
import Spinner from "../../../../../components/Spinner";
import { RENT_PLANS, Rto } from "../../../../../models/offer";
import { colors } from "../../../../../theme";
import {
  ButtonsContainer,
  ErrorsContainer,
  RentCollectionFormContainer,
  SpanColumns,
  SpinnerContainer,
} from "./styles";

type RentCollectionFormProps = {
  rentData: Rto;
  onSubmit?: (rentData: Rto) => void;
  errors?: Array<String>;
  buttons?: (rentData: Rto, submitting: boolean, valid: boolean) => ReactNode;
};

const RentCollectionForm: FC<RentCollectionFormProps> = ({
  rentData,
  onSubmit,
  buttons,
  errors,
}) => {
  return (
    <Form<Rto>
      onSubmit={(values) => (onSubmit ? onSubmit(values) : {})}
      render={({ submitting, valid, handleSubmit }) => (
        <form
          onSubmit={(e) => {
            e.preventDefault();
            handleSubmit();
          }}
        >
          <RentCollectionFormContainer>
            <Field
              name="startDate"
              placeholder="Tenancy start date"
              type="date"
              validate={required()}
              defaultValue={
                rentData?.startDate
                  ? format(new Date(rentData?.startDate), "yyyy-MM-dd")
                  : ""
              }
              readOnly={submitting || !!rentData.id}
            >
              {InputField}
            </Field>
            <Field
              name="endDate"
              placeholder="Termination date"
              type="date"
              validate={required()}
              defaultValue={
                rentData?.endDate
                  ? format(new Date(rentData?.endDate), "yyyy-MM-dd")
                  : ""
              }
              readOnly={submitting || !!rentData.id}
            >
              {InputField}
            </Field>
            <Field
              name="paymentDay"
              placeholder="Payment day"
              type="number"
              onKeyUp={(e) =>
                e.target.value < 0
                  ? (e.target.value = 0)
                  : e.target.value > 31
                  ? (e.target.value = 31)
                  : {}
              }
              validate={required()}
              defaultValue={rentData?.paymentDay}
              readOnly={submitting || !!rentData.id}
            >
              {InputField}
            </Field>
            <Field
              name="rentAmount"
              placeholder="Rent amount"
              type="number"
              validate={required()}
              defaultValue={rentData?.rentAmount}
              readOnly={submitting || !!rentData.id}
            >
              {InputField}
            </Field>
            <Field
              name="tenantFirstName"
              placeholder="Tenant first name"
              validate={required()}
              defaultValue={rentData?.tenantFirstName}
              readOnly={submitting || !!rentData.id}
            >
              {InputField}
            </Field>
            <Field
              name="tenantLastName"
              placeholder="Tenant last name"
              validate={required()}
              defaultValue={rentData?.tenantLastName}
              readOnly={submitting || !!rentData.id}
            >
              {InputField}
            </Field>
            <SpanColumns num={2}>
              <Field
                name="tenantEmail"
                placeholder="Tenant email address"
                validate={required()}
                defaultValue={rentData?.tenantEmail}
                readOnly={submitting || !!rentData.id}
              >
                {InputField}
              </Field>
              <Field
                name="tenantPhone"
                placeholder="Tenant phone number"
                defaultValue={rentData?.tenantPhone}
                readOnly={submitting || !!rentData.id}
                validate={composeValidators(validPhoneNumber())}
              >
                {PhoneInputField}
              </Field>
              <Field
                name="rentPlan"
                placeholder="Select a rent collection plan"
                options={RENT_PLANS}
                displayAttr="label"
                validate={required()}
                defaultValue={rentData?.rentPlan}
                readOnly={submitting || !!rentData.id}
              >
                {SelectField}
              </Field>
            </SpanColumns>
          </RentCollectionFormContainer>

          {!!errors?.length && (
            <ErrorsContainer>{errors.join(",")}</ErrorsContainer>
          )}

          {!!buttons?.length && (
            <ButtonsContainer>
              {submitting && (
                <SpinnerContainer>
                  <Spinner color={colors.white} size="md" />
                </SpinnerContainer>
              )}
              {buttons(rentData, submitting, valid)}
            </ButtonsContainer>
          )}
        </form>
      )}
    ></Form>
  );
};

export default RentCollectionForm;
