import ApolloClient from "apollo-client";

import { property_api_PropertyFurnitureEnum } from "../../../../../globalTypes";
import { ServiceCodes } from "../../../../listing/constants";
import {
  ListingCreate,
  ListingCreateVariables,
} from "../../../graphql/mutations/__generated__/ListingCreate";
import LISTING_CREATE from "../../../graphql/mutations/ListingCreate.graphql";
import {
  ListingFilterByUserId,
  ListingFilterByUserIdVariables,
} from "../../../graphql/queries/__generated__/ListingFilterByUserId";
import {
  OrdersList,
  OrdersListVariables,
} from "../../../graphql/queries/__generated__/OrdersList";
import {
  PriceCalculate,
  PriceCalculateVariables,
} from "../../../graphql/queries/__generated__/PriceCalculate";
import { ProductsList } from "../../../graphql/queries/__generated__/ProductsList";
import LISTING_FILTER_BY_USER_ID from "../../../graphql/queries/ListingFilterByUserId.graphql";
import ORDERS_LIST from "../../../graphql/queries/OrdersList.graphql";
import PRICE_CALCULATE from "../../../graphql/queries/PriceCalculate.graphql";
import PRODUCTS_LIST from "../../../graphql/queries/ProductsList.graphql";
import { Address, createAddress } from "../../../models/address";
import { Invoice } from "../../../models/invoice";
import { Rto } from "../../../models/offer";
import { Order, OrderStatus, OrderType } from "../../../models/order";
import {
  ExtrasProduct,
  FurnishedType,
  ProductCodesByType,
  ProductSale,
  ProductStatus,
  ProductType,
  PropertyProduct,
} from "../../../models/product";
import { Property, PropertyStatus } from "../../../models/property";
import { getEnumKeyByEnumValue } from "../../../models/utils";

export interface PropertyListService {
  fetchProperties: (userId: string) => Promise<Array<Property>>;
  fetchOrders: (userId: string) => Promise<Array<Order>>;
  fetchExtrasProducts: () => Promise<Array<ExtrasProduct>>;
  enrichProperties: (
    properties: Array<Property>,
    products: Array<ExtrasProduct>,
    orders: Array<Order>,
    activeRtos: Array<Rto>,
    invoices: Array<Invoice>
  ) => Array<Property>;
  createPropertyVersion: (
    userId: string,
    addrUrl: string,
    addr: Address
  ) => Promise<string>;
}

export const createPropertyListService = (
  client: ApolloClient<any>
): PropertyListService => {
  const createPropertyProductsList = (
    extrasProducts: Array<ExtrasProduct>,
    lastProductSalesMap: Map<string, ProductSale>
  ): Array<PropertyProduct> => {
    const prods: Array<PropertyProduct> = [];
    const extrasProdsMap = new Map(extrasProducts.map((p) => [p.code, p]));

    for (const [typeCode, title] of Object.entries(ProductType)) {
      const type = ProductType[typeCode];
      const codes = ProductCodesByType.get(type) || [];
      const code = codes[0];
      const status = ProductStatus.UNKNOWN;
      const lastSale = lastProductSalesMap.get(code);
      const prod: PropertyProduct = {
        type,
        code,
        title,
        status,
        lastSale,
        price: 0,
        tax: 0,
        referenceID: extrasProdsMap.get(code)?.referenceID || "",
        remote_sales_prices:
          extrasProdsMap.get(code)?.remote_sales_prices || [],
        remote_sales_taxes: extrasProdsMap.get(code)?.remote_sales_taxes || [],
        calculatePrice: (prod: PropertyProduct) =>
          calculateProductPrice(prod, extrasProdsMap),
      };

      prods.push(prod);
    }

    return prods;
  };

  const calculateProductPrice = async (
    prod: PropertyProduct,
    extrasProdsMap: Map<string, ExtrasProduct>
  ): Promise<void> => {
    const codes = ProductCodesByType.get(prod.type);
    let extrasProd: ExtrasProduct | null = null;

    let price: number = 0,
      tax: number = 0;

    if (prod.type === ProductType.GAS_SAFETY) {
      const code = prod.gasSafetyDetails?.boiler
        ? ServiceCodes.landlord_gas_safety_boiler_service
        : ServiceCodes.landlord_gas_certificate;

      const getExtrasProd = extrasProdsMap.get(code);
      if (!getExtrasProd) {
        return;
      }

      extrasProd = getExtrasProd;

      if (typeof prod.gasSafetyDetails?.selected_index !== "undefined") {
        const index = prod.gasSafetyDetails?.selected_index;
        price = extrasProd.remote_sales_prices[index] || 0;
        tax = extrasProd.remote_sales_taxes[index] || 0;
      } else {
        price = extrasProd.price || 0;
        tax = extrasProd.tax || 0;
      }
    } else if (prod.type === ProductType.RENT_GUARANTEE_INSURANCE) {
      const code = prod.code;
      const getExtrasProd = extrasProdsMap.get(code);

      if (!getExtrasProd) {
        return;
      }

      extrasProd = getExtrasProd;
      const { price: calcPrice, taxAmount: calcTax } = await requestPrice(
        prod,
        extrasProd?.id
      );

      price = calcPrice;
      tax = calcTax;
    } else if (prod.type === ProductType.INVENTORY) {
      const code = prod.inventoryDetails?.checkIn
        ? ServiceCodes.landlord_inventory_check_in
        : ServiceCodes.landlord_inventory;

      const getExtrasProd = extrasProdsMap.get(code);
      if (!getExtrasProd) {
        return;
      }

      extrasProd = getExtrasProd;
      const { price: calcPrice, taxAmount: calcTax } = await requestPrice(
        prod,
        extrasProd?.id
      );
      price = calcPrice;
      tax = calcTax;
    } else if (codes?.length === 1) {
      const code = codes?.length ? codes[0] : "";

      const getExtrasProd = extrasProdsMap.get(code);
      if (!getExtrasProd) {
        return;
      }

      extrasProd = getExtrasProd;
      price = extrasProd.price || 0;
      tax = extrasProd.tax || 0;
    }

    if (!!extrasProd) {
      prod.id = extrasProd.id;
      prod.code = extrasProd.code;
      prod.price = price || 0;
      prod.tax = tax || 0;
      prod.referenceID = extrasProd.referenceID;
      prod.remote_sales_prices = extrasProd.remote_sales_prices;
      prod.remote_sales_taxes = extrasProd.remote_sales_taxes;
    }
  };

  const requestPrice = async (
    product: PropertyProduct,
    extrasProdId: string
  ): Promise<{ price: number; taxAmount: number }> => {
    if (!extrasProdId) {
      throw new Error("Could determine extras product id");
    }

    const resp = await client.query<PriceCalculate, PriceCalculateVariables>({
      query: PRICE_CALCULATE,
      fetchPolicy: "no-cache",
      variables: {
        serviceId: extrasProdId,
        property: {
          details: {
            amenity_furniture:
              product.inventoryDetails?.furnishedType ===
              FurnishedType.FURNISHED
                ? property_api_PropertyFurnitureEnum.PROPERTY_FURNITURE_FURNISHED
                : property_api_PropertyFurnitureEnum.PROPERTY_FURNITURE_UNFURNISHED,
            bedrooms_no: product.inventoryDetails?.bedroomsNum,
          },
          rental: {
            rental_price: product.insuranceDetails?.rentAmount || 0,
          },
        },
      },
    });

    if (resp.errors?.length) {
      throw new Error(resp.errors[0].message);
    }

    const data = resp.data?.admin?.PriceCalculate;
    return { price: data?.price || 0, taxAmount: data?.tax_amount || 0 };
  };

  return {
    fetchProperties: async (userId: string): Promise<Array<Property>> => {
      const resp = await client.query<
        ListingFilterByUserId,
        ListingFilterByUserIdVariables
      >({
        query: LISTING_FILTER_BY_USER_ID,
        fetchPolicy: "no-cache",
        variables: { userId },
      });

      if (resp.errors?.length) {
        throw Error(resp.errors[0]?.message);
      }

      return (
        resp.data?.admin?.allListingVersions?.versions?.map((ver) => {
          const addr = createAddress(ver.location);
          return {
            id: ver.property_id,
            status: getEnumKeyByEnumValue(PropertyStatus, String(ver.status)),
            listingId: ver.id,
            address: addr,
            bankDetails: {
              propertyId: ver.property_id,
              accountName: ver.rental.bank_details.account_name,
              personalReference: ver.rental.bank_details.personal_reference,
              accountNumber: ver.rental.bank_details.account_number,
              sortCode: ver.rental.bank_details.sort_code,
            },
            addressLine:
              [
                addr?.flat,
                addr?.address,
                addr?.postcode,
                addr?.city,
                addr?.country,
              ]
                .filter(Boolean)
                .join(", ") || "",
          } as Property;
        }) || []
      );
    },

    fetchOrders: async (userId: string): Promise<Array<Order>> => {
      const resp = await client.query<OrdersList, OrdersListVariables>({
        query: ORDERS_LIST,
        fetchPolicy: "no-cache",
        variables: { userId },
      });

      if (resp.errors?.length) {
        throw new Error(resp.errors[0].message);
      }

      return (
        resp.data?.admin?.allOrders?.orders?.map((ord) => {
          const orderStatus: OrderStatus =
            getEnumKeyByEnumValue(OrderStatus, String(ord.status)) ||
            OrderStatus.UNKNOWN_ORDER_STATUS;
          return {
            id: ord.id,
            userId: ord.userid,
            correlationType: ord.correlation_type,
            correlationId: ord.correlation_id,
            orderType: getEnumKeyByEnumValue(OrderType, String(ord.order_type)),
            status: orderStatus,
            productSales: ord.products
              ?.map((prod) => ({
                code: prod.service?.service?.code,
                status: getProductStatusByOrderStatus(orderStatus),
                saleDate: Math.max(Date.parse(ord.updated), 0),
                startDate: Math.max(Date.parse(prod.details?.time?.start), 0),
                endDate: Math.max(Date.parse(prod.details?.time?.end), 0),
              }))
              .sort((prd1, prd2) => prd2.startDate - prd1.startDate),
          } as Order;
        }) || []
      );
    },

    fetchExtrasProducts: async (): Promise<Array<ExtrasProduct>> => {
      const resp = await client.query<ProductsList>({
        query: PRODUCTS_LIST,
        fetchPolicy: "no-cache",
      });

      if (resp.errors?.length) {
        throw Error(resp.errors[0]?.message);
      }

      return (
        resp.data?.admin?.allProducts?.data?.map(
          (prd) =>
            ({
              id: prd.id,
              code: prd.code,
              title: prd.title,
              price: prd.price,
              tax: prd.tax_amount,
              referenceID: prd.reference_id,
              remote_sales_prices: prd.remote_sales_prices,
              remote_sales_taxes: prd.remote_sales_taxes,
            } as ExtrasProduct)
        ) || []
      );
    },

    createPropertyVersion: async (
      userId: string,
      addrUrl: string,
      addr: Address
    ): Promise<string> => {
      const addressLine = [addr.flat, addr.building, addr.street]
        .filter(Boolean)
        .join(" ");

      const createListingResp = await client.query<
        ListingCreate,
        ListingCreateVariables
      >({
        query: LISTING_CREATE,
        fetchPolicy: "no-cache",
        variables: {
          landlord: userId,
          admin: window.localStorage.getItem("adminId"),
          addrUrl,
          addr: {
            address: addressLine || "",
            address1: addressLine || "",
            flat: addr.flat || "",
            building: addr.building || "",
            street: addr.street || "",
            city: addr.city || "",
            country: addr.country || "",
            postcode: addr.postcode || "",
          },
        },
      });

      if (createListingResp.errors?.length) {
        throw new Error(createListingResp.errors[0].message);
      }

      return createListingResp.data?.admin?.createVersion?.version?.id || "";
    },

    enrichProperties: (
      properties: Array<Property>,
      extrasProducts: Array<ExtrasProduct>,
      orders: Array<Order>,
      activeRtos: Array<Rto>,
      invoices: Array<Invoice>
    ): Array<Property> => {
      const ordersByPropId = new Map<string, Array<Order>>();
      for (const ord of orders) {
        const propOrds = ordersByPropId.get(ord.correlationId) || [];
        propOrds.push(ord);
        ordersByPropId.set(ord.correlationId, propOrds);
      }

      for (const prop of properties) {
        const orders = ordersByPropId.get(prop.id);
        const lastProductSalesMap = createLastProductSalesMap(orders || []);
        prop.products = createPropertyProductsList(
          extrasProducts,
          lastProductSalesMap
        );

        prop.activeRto = activeRtos
          .filter((rto) => rto.propertyId === prop.id)
          .sort((r1, r2) => Date.parse(r2.created) - Date.parse(r1.created))[0];

        const rtoInv = invoices.find(
          (inv) => inv.correlationId === prop.activeRto?.id
        );
        if (prop.activeRto && rtoInv) {
          prop.activeRto.paymentSetup = rtoInv.setup;
        }
      }

      return properties.map((prop) => ({ ...prop } as Property));
    },
  };
};

const createLastProductSalesMap = (
  orders: Array<Order>
): Map<string, ProductSale> => {
  const productSalesByCode = new Map<string, ProductSale>();

  for (const ord of orders) {
    const prodSales = ord.productSales || [];
    for (const prodSale of prodSales) {
      if (!productSalesByCode.get(prodSale.code)) {
        productSalesByCode.set(prodSale.code, prodSale);
      }
    }
  }

  return productSalesByCode;
};

const getProductStatusByOrderStatus = (
  orderStatus: OrderStatus
): ProductStatus => {
  switch (orderStatus) {
    case OrderStatus.ORDER_PAID:
      return ProductStatus.PAID;
    case OrderStatus.ORDER_PENDING:
      return ProductStatus.PENDING;
    default:
      return ProductStatus.UNKNOWN;
  }
};
