import ApolloClient from "apollo-client";

import {
  cardpayment_api_PaymentLinkItemInput,
  common_api_generic_query_QueryFilterOperatorEnum as OperatorEnum,
  invoice_api_InvoiceStatusEnum,
} from "../../../globalTypes";
import {
  CreatePaymentLink,
  CreatePaymentLinkVariables,
} from "../graphql/mutations/__generated__/CreatePaymentLink";
import CREATE_PAYMENT_LINK from "../graphql/mutations/CreatePaymentLink.graphql";
import {
  InvoicesList,
  InvoicesListVariables,
} from "../graphql/queries/__generated__/InvoicesList";
import INVOICES_LIST from "../graphql/queries/InvoicesList.graphql";
import { Invoice } from "../models/invoice";
import { PaymentDetails, PaymentLink } from "../models/payment";

export interface InvoiceService {
  getStandingOrderInvoices: () => Promise<Array<Invoice>>;
  createPaymentLink: (payDetails: PaymentDetails) => Promise<PaymentLink>;
}

export const createInvoiceService = (
  client: ApolloClient<any>
): InvoiceService => {
  return {
    getStandingOrderInvoices: async (): Promise<Array<Invoice>> => {
      const resp = await client.query<InvoicesList, InvoicesListVariables>({
        query: INVOICES_LIST,
        fetchPolicy: "no-cache",
        variables: {
          filter: [
            {
              fields: ["correlation_target"],
              operator: OperatorEnum.EQUALS,
              value: ["standing-order"],
            },
            {
              fields: ["status"],
              operator: OperatorEnum.EQUALS,
              value: [invoice_api_InvoiceStatusEnum.INVOICE_OPEN],
            },
          ],
          pagination: { page: 1, perPage: 1000 },
          sort: [{ field: "created", order: -1 }],
        },
      });

      if (resp.errors?.length) {
        throw Error(resp.errors[0]?.message);
      }

      return (
        resp.data?.admin?.allInvoices?.invoices?.map(
          (inv) =>
            ({
              id: inv.invoice_id,
              correlationTarget: inv.correlation_target,
              correlationId: inv.correlation_id,
              correlationType: inv.correlation_type,
              setup: inv.setup,
            } as Invoice)
        ) || []
      );
    },
    createPaymentLink: async (
      payDetails: PaymentDetails
    ): Promise<PaymentLink> => {
      const resp = await client.query<
        CreatePaymentLink,
        CreatePaymentLinkVariables
      >({
        query: CREATE_PAYMENT_LINK,
        fetchPolicy: "no-cache",
        variables: {
          email: payDetails.email,
          items: payDetails.products.map(
            (pd) =>
              ({
                productID: pd.referenceID,
                quantity: 1,
                amount: pd.price + pd.tax,
              } as cardpayment_api_PaymentLinkItemInput)
          ),
        },
      });

      if (resp.errors?.length) {
        throw Error(resp.errors[0]?.message);
      }

      return {
        id: resp.data?.admin?.createPaymentLink?.id || "",
        url: resp.data?.admin?.createPaymentLink?.url || "",
      } as PaymentLink;
    },
  };
};
