import ApolloClient from "apollo-client";

import { offer_api_rto_RtoStateEnum } from "../../../globalTypes";
import {
  RtoCreate,
  RtoCreateVariables,
} from "../graphql/mutations/__generated__/RtoCreate";
import {
  RtoCreateTenancy,
  RtoCreateTenancyVariables,
} from "../graphql/mutations/__generated__/RtoCreateTenancy";
import RTO_CREATE from "../graphql/mutations/RtoCreate.graphql";
import RTO_CREATE_TENANCY from "../graphql/mutations/RtoCreateTenancy.graphql";
import {
  RtosList,
  RtosListVariables,
} from "../graphql/queries/__generated__/RtosList";
import RTOS_LIST from "../graphql/queries/RtosList.graphql";
import { Rto } from "../models/offer";
import { fixTimezoneOffsetForLocalDate } from "../models/utils";

export interface OfferService {
  fetchRtos: (
    userId: string,
    states: offer_api_rto_RtoStateEnum[]
  ) => Promise<Array<Rto>>;
  createRto: (Rto: Rto) => Promise<string>;
  createRtoTenancy: (Rto: Rto) => Promise<string>;
}

export const createOfferService = (client: ApolloClient<any>): OfferService => {
  return {
    fetchRtos: async (
      userId: string,
      states: offer_api_rto_RtoStateEnum[]
    ): Promise<Array<Rto>> => {
      const resp = await client.query<RtosList, RtosListVariables>({
        query: RTOS_LIST,
        fetchPolicy: "no-cache",
        variables: { userId, states },
      });

      if (resp.errors?.length) {
        throw Error(resp.errors[0]?.message);
      }

      return (
        resp.data?.admin?.rtosByLandlordId?.rtos?.map(
          (rto) =>
            ({
              id: rto.id,
              created: rto.created_at,
              state: rto.state,
              propertyId: rto.property,
              landlordId: rto.landlord?.user,
              rentPlan: rto.rent_plan,
              tenantFirstName: rto.lead_tenant.first_name,
              tenantLastName: rto.lead_tenant.last_name,
              tenantEmail: rto.lead_tenant.email_primary,
              tenantPhone: rto.lead_tenant.phone_primary,
              startDate: rto.conditions?.move_in_date,
              endDate: rto.conditions?.end_date,
              paymentDay: rto.conditions?.rent_payment_day,
              rentAmount: rto.conditions?.price,
            } as Rto)
        ) || []
      );
    },

    createRto: async (Rto: Rto): Promise<string> => {
      const resp = await client.query<RtoCreate, RtoCreateVariables>({
        query: RTO_CREATE,
        fetchPolicy: "no-cache",
        variables: {
          landlordId: Rto.landlordId,
          propertyId: Rto.propertyId,
          rentPlan: Rto.rentPlan,
        },
      });

      if (resp.errors?.length) {
        throw new Error(resp.errors[0].message);
      }

      return resp.data?.admin?.rtoCreate?.rto.id || "";
    },

    createRtoTenancy: async (rto: Rto): Promise<string> => {
      const resp = await client.query<
        RtoCreateTenancy,
        RtoCreateTenancyVariables
      >({
        query: RTO_CREATE_TENANCY,
        fetchPolicy: "no-cache",
        variables: {
          rtoId: rto.id,
          landlordId: rto.landlordId,
          tenantEmail: rto.tenantEmail,
          tenantPhone: rto.tenantPhone,
          tenantFirstName: rto.tenantFirstName,
          tenantLastName: rto.tenantLastName,
          startDate: rto.startDate
            ? fixTimezoneOffsetForLocalDate(rto.startDate, false)
            : "",
          endDate: rto.endDate
            ? fixTimezoneOffsetForLocalDate(rto.endDate, false)
            : "",
          paymentDay: rto.paymentDay,
          rentAmount: rto.rentAmount,
        },
      });

      if (resp.errors?.length) {
        throw new Error(resp.errors[0].message);
      }

      return resp.data?.admin?.rtoCreateTenancy?.rto.id || "";
    },
  };
};
