import React, { FC, useState } from "react";

import { Query } from "react-apollo";
import { Field, Form } from "react-final-form";

import {
  common_api_generic_query_QueryFilterOperatorEnum as OperatorEnum,
  common_api_generic_query_QueryParamsFilterInput,
} from "../../globalTypes";
import { PrimaryButton, SecondaryButton } from "../../ui/atoms/Button";
import {
  FilterSort,
  FilterSortItem,
  FilterSortItemInner,
} from "../../ui/atoms/FilterSort";
import { TextInput } from "../../ui/atoms/Input";
import { SearchFieldWide } from "../../ui/atoms/SearchField";
import Spinner from "../../ui/atoms/Spinner";
import SystemError from "../../ui/atoms/SystemError";
import { Table, Tbody, Thead } from "../../ui/atoms/Table";
import TableWrapper from "../../ui/atoms/TableWrapper";
import Dropdown from "../../ui/molecules/Dropdown";
import Pagebar from "../../ui/molecules/Pagebar";
import Pagination from "../../ui/molecules/Pagination";
import Sortable from "../../ui/molecules/Sortable";
import { makeFilters } from "../../util/search_filters";
import { TENANCY_STATUS_OPTIONS } from "./constants";
import {
  GetTenancies,
  GetTenanciesVariables,
} from "./graphql/__generated__/GetTenancies";
import FETCH_TENANCIES from "./graphql/GetTenancies.graphql";
import ListHead from "./ListHead";
import ListItem from "./ListItem";

type FormValues = {
  postcode: string | undefined;
  status: {
    label: string;
    value: string;
  };
  search: string | undefined;
};

const getFilters = (values: FormValues) => {
  return makeFilters({
    values,
    filters: [
      {
        fields: [
          "property_id",
          "ivr",
          "tenants.tenant_contact.first_name",
          "tenants.tenant_contact.last_name",
          "location.postcode",
          "location.address",
        ],
        key: "search",
        operator: OperatorEnum.CONTAINS,
      },
      {
        fields: ["state"],
        key: "status.value",
        operator: OperatorEnum.EQUALS,
      },
    ],
  });
};

const List: FC = () => {
  const [filter, setFilter] = useState<
    common_api_generic_query_QueryParamsFilterInput[]
  >([]);

  const handleFormSubmit = (values: FormValues) => {
    const filters = getFilters(values);
    setFilter(filters);
  };

  const handleFilterReset = (resetForm: () => void) => {
    resetForm();
    setFilter([]);
  };

  const isFiltersExist = Object.keys(filter).length > 0;

  return (
    <>
      <h2>View Tenancies</h2>

      <Form<FormValues>
        onSubmit={handleFormSubmit}
        render={({ form, values, submitting }) => {
          return (
            <>
              <SearchFieldWide>
                <Field
                  component={TextInput}
                  name="search"
                  placeholder="Enter property ID, address, postcode or reference number"
                />
                <PrimaryButton
                  disabled={!values.search || submitting}
                  onClick={() => handleFormSubmit(values)}
                  type="button"
                >
                  Search
                </PrimaryButton>
                &nbsp;&nbsp;
                {values.search ? (
                  <SecondaryButton
                    onClick={() => handleFilterReset(form.reset)}
                    type="reset"
                  >
                    Clear
                  </SecondaryButton>
                ) : null}
              </SearchFieldWide>
              <FilterSort>
                {/* <FilterSortItem>
                  <label htmlFor="postcode">Postcode:</label>
                  <div>
                    <Field
                      name="postcode"
                      render={(props) => <TextInput errorOnly {...props} />}
                    />
                  </div>
                </FilterSortItem> */}
                <FilterSortItem>
                  <label htmlFor="status">Tenancy status:</label>
                  <div>
                    <Field
                      name="status"
                      render={({ input }) => (
                        <Dropdown
                          input={input}
                          options={TENANCY_STATUS_OPTIONS}
                          placeholder="Choose tenancy status"
                        />
                      )}
                    />
                  </div>
                </FilterSortItem>
                <FilterSortItem>
                  <PrimaryButton
                    disabled={
                      (!values.postcode && !values.status) || submitting
                    }
                    onClick={() => handleFormSubmit(values)}
                  >
                    Filter
                  </PrimaryButton>
                  {isFiltersExist && (
                    <FilterSortItemInner>
                      <SecondaryButton
                        onClick={() => handleFilterReset(form.reset)}
                        type="reset"
                      >
                        Clear
                      </SecondaryButton>
                    </FilterSortItemInner>
                  )}
                </FilterSortItem>
              </FilterSort>
            </>
          );
        }}
      />

      <Sortable field={[]}>
        {() => (
          <Pagination filter={filter}>
            {({ page, setPage, perPage }) => {
              return (
                <Query<GetTenancies, GetTenanciesVariables>
                  query={FETCH_TENANCIES}
                  variables={{
                    pagination: { page, perPage },
                    filter,
                  }}
                >
                  {({ data, loading, error }) => {
                    if (error) {
                      return <SystemError error={error} />;
                    } else if (loading) {
                      return <Spinner />;
                    }

                    return (
                      <TableWrapper>
                        {/* {loading && <TableOverlay />} */}

                        <Pagebar
                          changePage={setPage}
                          curPage={page}
                          showNPages={3}
                          total={Math.ceil(
                            (data?.admin?.allTenancies?.total || 1) / perPage
                          )}
                        />

                        <Table style={{ tableLayout: "fixed" }}>
                          <Thead>
                            <ListHead />
                          </Thead>
                          <Tbody>
                            {(data?.admin?.allTenancies?.data ?? []).map(
                              (item) => {
                                const tenant = item.tenants?.[0];

                                return (
                                  <ListItem
                                    key={item.id}
                                    movedIn={item.movedin}
                                    movedOut={item.movedout}
                                    offerId={item.offer}
                                    propertyId={item.property}
                                    status={item.tenancy_state}
                                    tenantId={tenant?.user?.id}
                                    tenantName={[
                                      tenant?.tenant_contact?.first_name ?? "",
                                      tenant?.tenant_contact?.last_name ?? "",
                                    ]}
                                  />
                                );
                              }
                            )}
                          </Tbody>
                        </Table>

                        <Pagebar
                          changePage={setPage}
                          curPage={page}
                          showNPages={3}
                          total={Math.ceil(
                            (data?.admin?.allTenancies?.total || 1) / perPage
                          )}
                        />
                      </TableWrapper>
                    );
                  }}
                </Query>
              );
            }}
          </Pagination>
        )}
      </Sortable>
    </>
  );
};

export default List;
