import React from "react";

import styled from "@emotion/styled";
import Downshift from "downshift";

import { TextInput } from "../atoms/Input";
import theme from "../theme";

type Props = {
  allowEmpty?: boolean;
  className?: string;
  disabled?: boolean;
  emptyLabel?: string;
  input: any;
  itemToString?: any;
  itemToValue?: any;
  valueFromItem?: any;
  options: any;
  placeholder?: string;
  style?: Object;
  error?: string;
  inputStyle?: { [_key: string]: string };
  fullWidth?: boolean;
};

const Dropdown = ({
  allowEmpty,
  emptyLabel = "",
  options,
  input,
  itemToString = (i: { label: string; value: string }) => i.label,
  itemToValue = (i: { label: string; value: string }) => i,
  valueFromItem = (value: string) =>
    options.find((o) => o.value === value)?.label || value,
  placeholder = "",
  disabled = false,
  className,
  style,
  error,
  inputStyle,
  fullWidth,
}: Props) => {
  return (
    <Downshift
      itemToString={itemToString}
      onChange={(selection) =>
        input.onChange(selection === emptyLabel ? null : itemToValue(selection))
      }
      selectedItem={input.value}
    >
      {({
        getInputProps,
        getItemProps,
        getMenuProps,
        isOpen,
        inputValue,
        getRootProps,
        openMenu,
        closeMenu,
        getToggleButtonProps,
      }) => (
        <DropdownContainer
          {...getRootProps()}
          className={className}
          disabled={disabled}
          fullWidth={fullWidth}
          isOpen={isOpen}
          style={style}
        >
          <TextInput
            {...getInputProps()}
            className={error ? "input-error" : ""}
            data-empty={input.value == null || input.value === ""}
            defaultValue={undefined}
            disabled={disabled}
            onClick={() => openMenu()}
            placeholder={placeholder || ""}
            readOnly
            style={inputStyle}
            value={
              input.value != null && input.value !== ""
                ? typeof input.value === "string"
                  ? valueFromItem(input.value)
                  : itemToString(input.value)
                : emptyLabel
            }
          />
          {isOpen && (
            <DropdownList {...getMenuProps()}>
              {(allowEmpty ? [null].concat(options) : options).map(
                (item, index) => (
                  <DropdownItem
                    key={item.label}
                    selected={inputValue === item.label}
                    {...getItemProps({
                      key: index,
                      index,
                      item: item != null ? item : emptyLabel,
                    })}
                  >
                    {itemToString(item)}
                  </DropdownItem>
                )
              )}
            </DropdownList>
          )}
          {error ? (
            <DropdownButtonError
              {...getToggleButtonProps()}
              disabled={disabled}
              onClick={() => !disabled && openMenu()}
            >
              <span className="material-icons">expand_more</span>
            </DropdownButtonError>
          ) : isOpen ? (
            <DropdownButton
              {...getToggleButtonProps()}
              disabled={disabled}
              onClick={() => !disabled && closeMenu()}
            >
              <span className="material-icons">expand_less</span>
            </DropdownButton>
          ) : (
            <DropdownButtonDown
              {...getToggleButtonProps()}
              disabled={disabled}
              onClick={() => !disabled && openMenu()}
            >
              <span className="material-icons">expand_more</span>
            </DropdownButtonDown>
          )}
        </DropdownContainer>
      )}
    </Downshift>
  );
};

const DropdownButton = styled.div`
  position: absolute;
  top: 1px;
  right: 1px;
  width: 39px;
  height: 38px;
  display: flex;
  align-items: center;
  justify-content: center;
  cursor: pointer;
  background: white;
  appearance: none;
  border-left: solid 1px transparent;
  border-radius: 0 5px 5px 0;
  user-select: none;

  &::before {
    content: "";
    position: absolute;
    top: 0;
    left: -25px;
    bottom: 0;
    background: linear-gradient(
      to right,
      rgba(255, 255, 255, 0.05),
      white 75%,
      white
    );
    width: 24px;
  }
`;
const DropdownButtonDown = styled(DropdownButton)`
  background: ${theme.colors.main.lightGrey};
  border-left-color: ${theme.colors.main.midGrey};
`;
const DropdownButtonError = styled(DropdownButton)`
  background: ${theme.colors.main.lightRed};
  border-left-color: ${theme.colors.main.red};

  &::before {
    display: none;
  }
`;
const DropdownContainer = styled.div<{ fullWidth?: boolean }>`
  position: relative;
  height: 40px;
  opacity: 1;
  display: inline-block;
  width: ${({ fullWidth }) => (fullWidth ? "100%" : "unset")};

  input {
    padding-left: 19px;
    padding-right: 44px;
    width: 200px;
    min-width: 200px;
    width: ${({ fullWidth }) => (fullWidth ? "100%" : "unset")};
  }
`;
const DropdownList = styled.ul`
  list-style: none;
  padding: 0;
  margin: 0;
  position: absolute;
  top: 40px;
  left: 0;
  right: 0;
  width: auto;
  min-width: 85px;
  z-index: 2;
  background: #fff;
  box-shadow: 0 1.5rem 3rem 0 rgba(59, 59, 77, 0.25);
  max-height: 235px;
  overflow: auto;
  border: solid 1px ${theme.colors.main.mainBlue};
  border-radius: 5px;

  &::-webkit-scrollbar {
    width: 6px;
    background-color: ${theme.colors.main.lightGrey};
    border-radius: 6px;
  }

  &::-webkit-scrollbar-thumb {
    border-radius: 6px;
    background-color: ${theme.colors.main.midGrey};
  }

  &::-webkit-scrollbar-track {
    background-color: ${theme.colors.main.lightGrey};
    border-radius: 6px;
  }
`;
const DropdownItem = styled.li`
  padding: 9px 8px 9px 19px;
  cursor: pointer;
  background-color: white;
  font-size: 16px;
  line-height: 20px;
  font-weight: 600;
  border-top: solid 1px ${theme.colors.main.midGrey};
  min-height: 39px;

  &:first-of-type {
    border-top-color: transparent;
  }

  &:hover {
    background-color: ${theme.colors.main.lightGrey};
  }
`;

export default Dropdown;
