import React, { FC, useCallback, useEffect, useMemo, useState } from "react";

import axios from "axios";
import cx from "classnames";
import Dropzone, { DropzoneOptions } from "react-dropzone";

import { cropFileName } from "../../../util/file";
import { FileFieldProps, TFile } from "./types";

import styles from "./styles.module.css";

export const FileField: FC<FileFieldProps> = ({
  fileId,
  acceptedTypes,
  error,
  handleChange,
}) => {
  const [file, setFile] = useState<TFile | undefined>();
  const [loading, setLoading] = useState<boolean>(false);

  const dropzoneCss = useMemo(
    () =>
      cx(styles.dropzoneBox, {
        [styles.withError]: !!error,
      }),
    [error]
  );

  const acceptedTypesToString = useMemo(
    () =>
      acceptedTypes
        .map((el) => {
          const splitType = el.split("/");
          return splitType[splitType.length - 1];
        })
        .join(", "),
    [acceptedTypes]
  );

  const handleDeleteFile = useCallback(() => {
    setFile(undefined);
    handleChange("");
  }, [handleChange]);

  const handleDropFile: DropzoneOptions["onDrop"] = useCallback(
    async (acceptedFiles) => {
      if (!acceptedFiles.length) {
        return;
      }

      setLoading(true);
      const formData = new FormData();
      formData.append("file", acceptedFiles[0]);

      const result = await uploadFile(formData).finally(() =>
        setLoading(false)
      );

      if (result) {
        setFile(result.data);
        handleChange(result.data.id);
      }
    },
    [handleChange]
  );

  const getFileMeta = useCallback(async () => {
    if (fileId) {
      const result = await fetchFileMeta(fileId);
      if (result?.data.name) {
        setFile(result.data);
      }
    }
  }, [fileId]);

  useEffect(() => {
    if (!file && fileId) {
      void getFileMeta();
    }
  }, [file, fileId, getFileMeta]);

  return (
    <>
      {file ? (
        <div className={styles.filePreview}>
          {~file.name.toLowerCase().lastIndexOf(".pdf") ? (
            <div className={styles.pdf}>PDF</div>
          ) : (
            <img alt="" className={styles.image} src={`/media/${file.id}`} />
          )}
          <div className={styles.fileName}>{cropFileName(file.name)}</div>
          <div
            className={styles.fileClear}
            onClick={handleDeleteFile}
            role="presentation"
          >
            <span className="material-icons">close</span>
          </div>
        </div>
      ) : fileId ? (
        <div className={styles.filePreview}>
          <img alt="" className={styles.image} src={`/media/${fileId}`} />
        </div>
      ) : (
        <Dropzone
          accept={acceptedTypes.join(",")}
          multiple={false}
          onDrop={handleDropFile}
        >
          {({ getRootProps, getInputProps }) => (
            <div {...(getRootProps() as any)} className={dropzoneCss}>
              <input {...(getInputProps() as any)} />
              <p style={{ textAlign: "center" }}>
                Drop file here or click for file browse dialog <br />
                <b>{acceptedTypesToString}</b>
              </p>
              {loading && <div className={styles.dropzoneLoading} />}
            </div>
          )}
        </Dropzone>
      )}
    </>
  );
};

// API calls
const uploadFile = (formData: FormData) =>
  axios({
    method: "post",
    url: "/media",
    data: formData,
    headers: {
      "content-type": "multipart/form-data",
    },
  }).catch((e) => {
    // eslint-disable-next-line no-console
    console.error(e);
    window && alert("Error of loading. Try once more");
  });

const fetchFileMeta = (id: string) =>
  axios({
    method: "get",
    url: `/media/info-${id}`,
    headers: {
      "content-type": "multipart/form-data",
    },
  }).catch((e) => {
    // eslint-disable-next-line no-console
    console.error(e);
    window && alert("Error of fetching file metadata");
  });
