import React, { FC, ReactNode } from "react";

import { keyframes } from "@emotion/core";
import styled from "@emotion/styled";

import { DeleteButton, PrimaryButton, SuccessButton } from "../atoms/Button";
import Spinner from "../atoms/Spinner";
import useModal from "../hooks/use-modal";
import theme from "../theme";
import Portal from "./Portal";

import css from "@emotion/css";

type Props = {
  onCancel: () => void;
  title?: string | ReactNode;
  withoutOverflow?: boolean;
  width?: number;
  isOpen?: boolean;
};

type PropsConfirm = {
  onCancel: () => void;
  onConfirm: () => void;
  disabled?: boolean;
  isOpen?: boolean;
  title?: string;
  loading?: boolean;
};

export const Modal: FC<Props> = ({
  onCancel,
  withoutOverflow,
  title,
  children,
  isOpen,
}) => {
  useModal(onCancel);

  // typeof check to provide a fallback to prev usage pattern: "{isOpen && <Modal..."
  if (typeof isOpen === "boolean" && !isOpen) {
    return null;
  }

  return (
    <Portal>
      <ModalWrapper>
        <ModalOverlay onClick={onCancel} />
        <ModalContainer withoutOverflow={withoutOverflow}>
          {title && (
            <ModalHeader>
              <h3>{title}</h3>
            </ModalHeader>
          )}
          <ModalClose onClick={onCancel}>
            <span className="material-icons">close</span>
          </ModalClose>
          <ModalContent>{children}</ModalContent>
        </ModalContainer>
      </ModalWrapper>
    </Portal>
  );
};

export const ModalWide: FC<Props> = ({
  onCancel,
  children,
  width,
  withoutOverflow,
  title,
  isOpen,
}) => {
  useModal(onCancel);

  // typeof check to provide a fallback to prev usage pattern: "{isOpen && <Modal..."
  if (typeof isOpen === "boolean" && !isOpen) {
    return null;
  }

  return (
    <Portal>
      <ModalWrapper>
        <ModalOverlay onClick={onCancel} />
        <ModalContainerWide width={width} withoutOverflow={withoutOverflow}>
          <ModalHeader>
            <h3>{title}</h3>
          </ModalHeader>
          <ModalClose onClick={onCancel}>
            <span className="material-icons">close</span>
          </ModalClose>
          <ModalContent>{children}</ModalContent>
        </ModalContainerWide>
      </ModalWrapper>
    </Portal>
  );
};

export const ModalConfirm: FC<PropsConfirm> = ({
  onCancel,
  onConfirm,
  children,
  disabled,
  isOpen,
  title = "Confirm",
  loading,
}) => {
  useModal(onCancel, onConfirm);

  // typeof check to provide a fallback to prev usage pattern: "{isOpen && <Modal..."
  if (typeof isOpen === "boolean" && !isOpen) {
    return null;
  }

  return (
    <Portal>
      <ModalWrapper>
        <ModalOverlay onClick={onCancel} />
        <ModalContainerConfirm>
          <ModalHeader>
            <h3>{title}</h3>
          </ModalHeader>
          <ModalClose onClick={onCancel}>
            <span className="material-icons">close</span>
          </ModalClose>
          <ModalContent>
            <ModalMessage>{children}</ModalMessage>
            <ModalActions>
              <DeleteButton disabled={disabled} onClick={onCancel}>
                NO
              </DeleteButton>
              <SuccessButton
                disabled={disabled}
                loading={Boolean(loading)}
                onClick={onConfirm}
              >
                <span className="text">YES</span>
                <span className="spinner">
                  <Spinner color="#fff" size="sm-x" />
                </span>
              </SuccessButton>
            </ModalActions>
          </ModalContent>
        </ModalContainerConfirm>
      </ModalWrapper>
    </Portal>
  );
};

export const ModalAlert: FC<Props> = ({
  isOpen,
  onCancel,
  children,
  title = "Attention",
}) => {
  useModal(onCancel);

  // typeof check to provide a fallback to prev usage pattern: "{isOpen && <Modal..."
  if (typeof isOpen === "boolean" && !isOpen) {
    return null;
  }

  return (
    <Portal>
      <ModalWrapper>
        <ModalOverlay onClick={onCancel} />
        <ModalContainerAlert>
          <ModalHeader>
            <h3>{title}</h3>
          </ModalHeader>
          <ModalClose onClick={onCancel}>
            <span className="material-icons">close</span>
          </ModalClose>
          <ModalContentAlert>
            <ModalMessage>{children}</ModalMessage>
            <ModalActions>
              <PrimaryButton onClick={onCancel}>Ok</PrimaryButton>
            </ModalActions>
          </ModalContentAlert>
        </ModalContainerAlert>
      </ModalWrapper>
    </Portal>
  );
};

const showModal = keyframes`
  0% {
    opacity: 0;
  }
  100% {
    opacity: 1;
  }
`;
const showModalContent = keyframes`
  0% {
    transform: scale(1.06);
  }
  100% {
    transform: scale(1);
  }
`;
type ModalContainerProps = {
  withoutOverflow?: boolean;
  width?: number;
};
const ModalContainer = styled.div<ModalContainerProps>`
  background: white;
  width: 600px;
  max-height: calc(100vh - 40px);
  position: relative;
  border-radius: 10px;
  box-shadow: ${theme.boxShadow.grey};
  animation: ${showModalContent} 1 ease 0.15s forwards;

  ${({ withoutOverflow }) =>
    withoutOverflow
      ? css`
          overflow-y: initial;
        `
      : css`
          overflow-y: auto;
        `}
`;
const ModalContainerConfirm = styled(ModalContainer)`
  width: 360px;
`;
const ModalContainerAlert = styled(ModalContainer)`
  width: 360px;
`;
const ModalContainerWide = styled(ModalContainer)<ModalContainerProps>`
  ${({ width = 750 }) => css`
    width: ${width}px;
  `}
`;
const ModalWrapper = styled.div`
  position: fixed;
  top: 0;
  left: 0;
  right: 0;
  bottom: 0;
  overflow: hidden;
  animation: ${showModal} 1 ease 0.25s forwards;
  z-index: 22;
  display: flex;
  align-items: center;
  justify-content: center;
`;
const ModalOverlay = styled.div`
  position: absolute;
  left: 0;
  right: 0;
  bottom: 0;
  top: 0;
  background: rgba(0, 0, 0, 0.5);
`;
const ModalHeader = styled.div`
  padding: 16px 30px;

  h3 {
    margin: 0;
  }
`;
const ModalClose = styled.div`
  position: absolute;
  top: 0;
  right: 0;
  cursor: pointer;
  padding: 10px;
  background: rgba(0, 0, 0, 0);
  font-size: 20px;
  line-height: 1;
  transition: color 0.25s;

  &:hover {
    color: ${theme.colors.main.red};
  }
`;
const ModalContent = styled.div`
  padding: 40px;
  background: ${theme.colors.main.lightGrey};
  border-bottom-left-radius: 10px;
  border-bottom-right-radius: 10px;
  max-height: 70vh;
  overflow-y: auto;
`;
const ModalContentAlert = styled(ModalContent)`
  padding: 40px 30px 60px;
`;
const ModalActions = styled.div`
  display: flex;
  justify-content: space-between;

  button {
    padding: 0;
    width: calc(50% - 15px);

    &:only-child {
      width: 140px;
      margin: 0 auto;
    }
  }
`;
const ModalMessage = styled.div`
  text-align: center;
  font-size: 16px;
  margin: 0 0 50px;
  font-weight: 500;
  line-height: 1.8;
`;
